<?php
/* Copyright (C) 2004-2007 Rodolphe Quiedeville <rodolphe@quiedeville.org>
 * Copyright (C) 2004-2013 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2005      Simon Tosser         <simon@kornog-computing.com>
 * Copyright (C) 2005-2012 Regis Houssin        <regis.houssin@inodbox.com>
 * Copyright (C) 2010	   Pierre Morin         <pierre.morin@auguria.net>
 * Copyright (C) 2010	   Juanjo Menent        <jmenent@2byte.es>
 * Copyright (C) 2022	   Ferran Marcet        <fmarcet@2byte.es>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 * or see https://www.gnu.org/
 */

/**
 *	\file       htdocs/document.php
 *  \brief      Wrapper to download data files
 *  \remarks    Call of this wrapper is made with URL:
 * 				DOL_URL_ROOT.'/document.php?modulepart=repfichierconcerne&file=relativepathoffile'
 * 				DOL_URL_ROOT.'/document.php?modulepart=logs&file=dolibarr.log'
 * 				DOL_URL_ROOT.'/document.php?hashp=sharekey'
 */

/* DyaPress plugin for WordPress
 *
 * Copyright (c) 2019   François Pons   <fpons@aytechnet.fr>
 *
 * the DyaPress namespace is added in order to avoid conflicts
 * with all the various WordPress extensions.
 */
namespace DyaPress;

/**
 * Header empty
 *
 * @ignore
 * @return	void
 */
/* function llxHeader()
{
}
 */
/**
 * Footer empty
 *
 * @ignore
 * @return	void
 */
/*
function llxFooter()
{
}
 */

require 'main.inc.php';	// Load $dyap_user and permissions
require_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/images.lib.php';

$dyap_encoding = '';
$dyap_action = GETPOST('action', 'aZ09');
$dyap_original_file = GETPOST('file', 'alphanohtml'); // Do not use urldecode here ($_GET are already decoded by PHP).
$dyap_hashp = GETPOST('hashp', 'aZ09');
$dyap_modulepart = GETPOST('modulepart', 'alpha');
$dyap_urlsource = GETPOST('urlsource', 'alpha');
$dyap_entity = GETPOST('entity', 'int') ?GETPOST('entity', 'int') : $dyap_conf->entity;

// Security check
if (empty($dyap_modulepart) && empty($dyap_hashp)) {
	accessforbidden('Bad link. Bad value for parameter modulepart', 0, 0, 1);
}
if (empty($dyap_original_file) && empty($dyap_hashp)) {
	accessforbidden('Bad link. Missing identification to find file (original_file or hashp)', 0, 0, 1);
}
if ($dyap_modulepart == 'fckeditor') {
	$dyap_modulepart = 'medias'; // For backward compatibility
}

$dyap_socid = 0;
if ($dyap_user->socid > 0) {
	$dyap_socid = $dyap_user->socid;
}

// For some module part, dir may be privates
if (in_array($dyap_modulepart, array('facture_paiement', 'unpaid'))) {
	if (empty($dyap_user->rights->societe->client->voir) || $dyap_socid) {
		$dyap_original_file = 'private/'.$dyap_user->id.'/'.$dyap_original_file; // If user has no permission to see all, output dir is specific to user
	}
}


/*
 * Actions
 */

// None



/*
 * View
 */

// If we have a hash public (hashp), we guess the original_file.
$ecmfile='';
if (!empty($dyap_hashp)) {
	include_once DOL_DOCUMENT_ROOT.'/ecm/class/ecmfiles.class.php';
	$ecmfile = new EcmFiles($dyap_db);
	$result = $ecmfile->fetch(0, '', '', '', $dyap_hashp);
	if ($result > 0) {
		$tmp = explode('/', $ecmfile->filepath, 2); // $ecmfile->filepath is relative to document directory
		// filepath can be 'users/X' or 'X/propale/PR11111'
		if (is_numeric($tmp[0])) { // If first tmp is numeric, it is subdir of company for multicompany, we take next part.
			$tmp = explode('/', $tmp[1], 2);
		}
		$dyap_moduleparttocheck = $tmp[0]; // moduleparttocheck is first part of path

		if ($dyap_modulepart) {	// Not required, so often not defined, for link using public hashp parameter.
			if ($dyap_moduleparttocheck == $dyap_modulepart) {
				// We remove first level of directory
				$dyap_original_file = (($tmp[1] ? $tmp[1].'/' : '').$ecmfile->filename); // this is relative to module dir
				//var_dump($original_file); exit;
			} else {
				accessforbidden('Bad link. File is from another module part.', 0, 0, 1);
			}
		} else {
			$dyap_modulepart = $dyap_moduleparttocheck;
			$dyap_original_file = (($tmp[1] ? $tmp[1].'/' : '').$ecmfile->filename); // this is relative to module dir
		}
		$dyap_entity = $ecmfile->entity;
		if ($dyap_entity != $dyap_conf->entity) {
			$dyap_conf->entity = $dyap_entity;
			$dyap_conf->setValues($dyap_db);
		}
	} else {
		$dyap_langs->load("errors");
		accessforbidden($dyap_langs->trans("ErrorFileNotFoundWithSharedLink"), 0, 0, 1);
	}
}

// Define attachment (attachment=true to force choice popup 'open'/'save as')
$attachment = true;
if (preg_match('/\.(html|htm)$/i', $dyap_original_file)) {
	$attachment = false;
}
if (isset($_GET["attachment"])) {
	$attachment = GETPOST("attachment", 'alpha') ?true:false;
}
if (!empty($dyap_conf->global->MAIN_DISABLE_FORCE_SAVEAS)) {
	$attachment = false;
}

// Define mime type
$type = 'application/octet-stream'; // By default
if (GETPOST('type', 'alpha')) {
	$type = GETPOST('type', 'alpha');
} else {
	$type = dol_mimetype($dyap_original_file);
}
// Security: Force to octet-stream if file is a dangerous file. For example when it is a .noexe file
// We do not force if file is a javascript to be able to get js from website module with <script src="
// Note: Force whatever is $modulepart seems ok.
if (!in_array($type, array('text/x-javascript')) && !dolIsAllowedForPreview($dyap_original_file)) {
	$type = 'application/octet-stream';
}

// Security: Delete string ../ or ..\ into $original_file
$dyap_original_file = preg_replace('/\.\.+/', '..', $dyap_original_file);	// Replace '... or more' with '..'
$dyap_original_file = str_replace('../', '/', $dyap_original_file);
$dyap_original_file = str_replace('..\\', '/', $dyap_original_file);


// Find the subdirectory name as the reference
$refname = basename(dirname($dyap_original_file)."/");

// Security check
if (empty($dyap_modulepart)) {
	accessforbidden('Bad value for parameter modulepart');
}

// Check security and set return info with full path of file
$check_access = dol_check_secure_access_document($dyap_modulepart, $dyap_original_file, $dyap_entity, $dyap_user, $refname);
$accessallowed              = $check_access['accessallowed'];
$sqlprotectagainstexternals = $check_access['sqlprotectagainstexternals'];
$dyap_fullpath_original_file     = $check_access['original_file']; // $fullpath_original_file is now a full path name

if (!empty($dyap_hashp)) {
	$accessallowed = 1; // When using hashp, link is public so we force $accessallowed
	$sqlprotectagainstexternals = '';
} else {
	// Basic protection (against external users only)
	if ($dyap_user->socid > 0) {
		if ($sqlprotectagainstexternals) {
			$resql = $dyap_db->query($sqlprotectagainstexternals);
			if ($resql) {
				$num = $dyap_db->num_rows($resql);
				$i = 0;
				while ($i < $num) {
					$obj = $dyap_db->fetch_object($resql);
					if ($dyap_user->socid != $obj->fk_soc) {
						$accessallowed = 0;
						break;
					}
					$i++;
				}
			}
		}
	}
}

// Security:
// Limit access if permissions are wrong
if (!$accessallowed) {
	accessforbidden();
}

// Security:
// We refuse directory transversal change and pipes in file names
if (preg_match('/\.\./', $dyap_fullpath_original_file) || preg_match('/[<>|]/', $dyap_fullpath_original_file)) {
	dol_syslog("Refused to deliver file ".$dyap_fullpath_original_file);
	print "ErrorFileNameInvalid: ".dol_escape_htmltag($dyap_original_file);
	exit;
}


clearstatcache();

$dyap_filename = basename($dyap_fullpath_original_file);
$dyap_filename = preg_replace('/\.noexe$/i', '', $dyap_filename);

// Output file on browser
dol_syslog("document.php download $dyap_fullpath_original_file filename=$dyap_filename content-type=$type");
$dyap_fullpath_original_file_osencoded = dol_osencode($dyap_fullpath_original_file); // New file name encoded in OS encoding charset

// This test if file exists should be useless. We keep it to find bug more easily
if (!file_exists($dyap_fullpath_original_file_osencoded)) {
	dol_syslog("ErrorFileDoesNotExists: ".$dyap_fullpath_original_file);
	print "ErrorFileDoesNotExists: ".$dyap_original_file;
	exit;
}

// Hooks
if (!is_object($dyap_hookmanager)) {
	include_once DOL_DOCUMENT_ROOT.'/core/class/hookmanager.class.php';
	$dyap_hookmanager = new HookManager($this->db);
}
$dyap_hookmanager->initHooks(array('document'));
$parameters = array('ecmfile' => $ecmfile, 'modulepart' => $dyap_modulepart, 'original_file' => $dyap_original_file,
	'entity' => $dyap_entity, 'refname' => $refname, 'fullpath_original_file' => $dyap_fullpath_original_file,
	'filename' => $dyap_filename, 'fullpath_original_file_osencoded' => $dyap_fullpath_original_file_osencoded);
$reshook = $dyap_hookmanager->executeHooks('downloadDocument', $parameters); // Note that $action and $object may have been
if ($reshook < 0) {
	$errors = $dyap_hookmanager->error.(is_array($dyap_hookmanager->errors) ? (!empty($dyap_hookmanager->error) ? ', ' : '').join($dyap_separator, $dyap_hookmanager->errors) : '');
	dol_syslog("document.php - Errors when executing the hook 'downloadDocument' : ".$errors);
	print "ErrorDownloadDocumentHooks: ".$errors;
	exit;
}

// Permissions are ok and file found, so we return it
top_httphead($type);
header('Content-Description: File Transfer');
if ($dyap_encoding) {
	header('Content-Encoding: '.$dyap_encoding);
}
// Add MIME Content-Disposition from RFC 2183 (inline=automatically displayed, attachment=need user action to open)
if ($attachment) {
	header('Content-Disposition: attachment; filename="'.$dyap_filename.'"');
} else {
	header('Content-Disposition: inline; filename="'.$dyap_filename.'"');
}
// Ajout directives pour resoudre bug IE
header('Cache-Control: Public, must-revalidate');
header('Pragma: public');
$readfile = true;

// on view document, can output images with good orientation according to exif infos
if (!$attachment && !empty($dyap_conf->global->MAIN_USE_EXIF_ROTATION) && image_format_supported($dyap_fullpath_original_file_osencoded) == 1) {
	$imgres = correctExifImageOrientation($dyap_fullpath_original_file_osencoded, null);
	$readfile = !$imgres;
}

if (is_object($dyap_db)) {
	$dyap_db->close();
}

// Send file now
if ($readfile) {
	header('Content-Length: '.dol_filesize($dyap_fullpath_original_file));

	readfileLowMemory($dyap_fullpath_original_file_osencoded);
}
