<?php
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

WordPress DyaPress plugin :
Copyright (C) 2016-2022      François Pons        <fpons@aytechnet.fr>
 */

namespace DyaPress;

/*
 * This file allows compatibility mode with removed files with Dolibarr code :
 *    filefunc.inc.php
 *    master.inc.php
 *    main.inc.php
 */

require_once( dirname(__FILE__) . '/dyapress-init.php' );

function test_sql_and_script_inject($val, $type)
{
    return 0;
}

function analyseVarsForSqlAndScriptsInjection(&$var, $type)
{
    return true;
}

// If there is a POST parameter to tell to save automatically some POST parameters into cookies, we do it.
// This is used for example by form of boxes to save personalization of some options.
// DOL_AUTOSET_COOKIE=cookiename:val1,val2 and  cookiename_val1=aaa cookiename_val2=bbb will set cookie_name with value json_encode(array('val1'=> , ))
if (! empty($_POST["DOL_AUTOSET_COOKIE"]))
{
	$tmpautoset=explode(':',$_POST["DOL_AUTOSET_COOKIE"],2);
	$tmplist=explode(',',$tmpautoset[1]);
	$cookiearrayvalue=array();
	foreach ($tmplist as $tmpkey)
	{
		$postkey=$tmpautoset[0].'_'.$tmpkey;
		if (! empty($_POST[$postkey])) $cookiearrayvalue[$tmpkey]=$_POST[$postkey];
	}
	$cookiename=$tmpautoset[0];
	$cookievalue=json_encode($cookiearrayvalue);
	//var_dump('setcookie cookiename='.$cookiename.' cookievalue='.$cookievalue);
	setcookie($cookiename, empty($cookievalue)?'':$cookievalue, empty($cookievalue)?0:(time()+(86400*354)), '/', null, false, true);	// keep cookie 1 year and add tag httponly
	if (empty($cookievalue)) unset($_COOKIE[$cookiename]);
}

// Init session. Name of session is specific to Dolibarr instance.
// Note: the function dol_getprefix may have been redefined to return a different key to manage another area to protect.
$prefix=dol_getprefix('');

$sessionname='DOLSESSID_'.$prefix;
$sessiontimeout='DOLSESSTIMEOUT_'.$prefix;
if (! empty($_COOKIE[$sessiontimeout])) ini_set('session.gc_maxlifetime',$_COOKIE[$sessiontimeout]);
session_name($sessionname);
session_set_cookie_params(0, '/', null, false, true);   // Add tag httponly on session cookie (same as setting session.cookie_httponly into php.ini). Must be called before the session_start.
// This create lock, released when session_write_close() or end of page.
// We need this lock as long as we read/write $_SESSION ['vars']. We can remove lock when finished.
if (! defined('NOSESSION'))
{
    session_start();
}

// the following file defines all the various DyaPress globals (from Dolibarr) which need to be protected in WordPress
include DOL_DOCUMENT_ROOT.'/dyapress-globals.php';

// Activate end of page function
register_shutdown_function('\DyaPress\dol_shutdown');

// Detection browser
if (isset($_SERVER["HTTP_USER_AGENT"]))
{
	$tmp=getBrowserInfo($_SERVER["HTTP_USER_AGENT"]);
	$dyap_conf->browser->name=$tmp['browsername'];
	$dyap_conf->browser->os=$tmp['browseros'];
	$dyap_conf->browser->version=$tmp['browserversion'];
	$dyap_conf->browser->layout=$tmp['layout'];     // 'classic', 'phone', 'tablet'
	$dyap_conf->browser->phone=$tmp['phone'];	   // TODO deprecated, use ->layout
	$dyap_conf->browser->tablet=$tmp['tablet'];	   // TODO deprecated, use ->layout
	//var_dump($dyap_conf->browser);

	if ($dyap_conf->browser->layout == 'phone') $dyap_conf->dol_no_mouse_hover=1;
	if ($dyap_conf->browser->layout == 'phone') $dyap_conf->global->MAIN_TESTMENUHIDER=1;
}

// Loading of additional presentation includes
require_once DOL_DOCUMENT_ROOT .'/core/class/html.form.class.php';	    // Need 660ko memory (800ko in 2.2)
if ($dyap_conf->use_javascript_ajax) require_once DOL_DOCUMENT_ROOT.'/core/lib/ajax.lib.php';	// Need 22ko memory

// If install or upgrade process not done or not completely finished, we call the install page.
if (! empty($dyap_conf->global->MAIN_NOT_INSTALLED) || ! empty($dyap_conf->global->MAIN_NOT_UPGRADED))
{
	dol_syslog("main.inc: A previous install or upgrade was not complete. Redirect to install page.", LOG_WARNING);
	header("Location: ". admin_url( "options-general.php?page=dyapress_plugin" ) );
	exit;
}
// If an upgrade process is required, we call the install page.
if ((! empty($dyap_conf->global->MAIN_VERSION_LAST_UPGRADE) && ($dyap_conf->global->MAIN_VERSION_LAST_UPGRADE != DOL_VERSION))
|| (empty($dyap_conf->global->MAIN_VERSION_LAST_UPGRADE) && ! empty($dyap_conf->global->MAIN_VERSION_LAST_INSTALL) && ($dyap_conf->global->MAIN_VERSION_LAST_INSTALL != DOL_VERSION)))
{
	$versiontocompare=empty($dyap_conf->global->MAIN_VERSION_LAST_UPGRADE)?$dyap_conf->global->MAIN_VERSION_LAST_INSTALL:$dyap_conf->global->MAIN_VERSION_LAST_UPGRADE;
	require_once DOL_DOCUMENT_ROOT .'/core/lib/admin.lib.php';
	$dolibarrversionlastupgrade=preg_split('/[.-]/',$versiontocompare);
	$dolibarrversionprogram=preg_split('/[.-]/',DOL_VERSION);
	$rescomp=versioncompare($dolibarrversionprogram,$dolibarrversionlastupgrade);
	if ($rescomp > 0)   // Programs have a version higher than database. We did not add "&& $rescomp < 3" because we want upgrade process for build upgrades
	{
		dol_syslog("main.inc: database version ".$versiontocompare." is lower than programs version ".DOL_VERSION.". Redirect to install page.", LOG_WARNING);
		header("Location: ". admin_url( "options-general.php?page=dyapress_plugin" ) );
		exit;
	}
}

// Creation of a token against CSRF vulnerabilities
// roulement des jetons car cree a chaque appel
if (isset($_SESSION['newtoken'])) $_SESSION['token'] = $_SESSION['newtoken'];

// Save in $_SESSION['newtoken'] what will be next token. Into forms, we will add param token = $_SESSION['newtoken']
$token = dol_hash(uniqid(mt_rand(), true)); // Generates a hash of a random number
$_SESSION['newtoken'] = $token;

if ((! defined('NOCSRFCHECK') && empty($dolibarr_nocsrfcheck) && ! empty($dyap_conf->global->MAIN_SECURITY_CSRF_WITH_TOKEN))
    || defined('CSRFCHECK_WITH_TOKEN'))	// Check validity of token, only if option MAIN_SECURITY_CSRF_WITH_TOKEN enabled or if constant CSRFCHECK_WITH_TOKEN is set
{
    if ($_SERVER['REQUEST_METHOD'] == 'POST' && ! GETPOST('token','alpha')) // Note, offender can still send request by GET
    {
	print "Access refused by CSRF protection in main.inc.php. Token not provided.\n";
	print "If you access your server behind a proxy using url rewriting, you might check that all HTTP header is propagated (or add the line \$dolibarr_nocsrfcheck=1 into your conf.php file).\n";
	die;
    }
    if ($_SERVER['REQUEST_METHOD'] === 'POST')  // This test must be after loading $_SESSION['token'].
    {
	if (GETPOST('token', 'alpha') != $_SESSION['token'])
	{
	    dol_syslog("Invalid token in ".$_SERVER['HTTP_REFERER'].", action=".GETPOST('action','aZ09').", _POST['token']=".GETPOST('token','alpha').", _SESSION['token']=".$_SESSION['token'], LOG_WARNING);
	    //print 'Unset POST by CSRF protection in main.inc.php.';	// Do not output anything because this create problems when using the BACK button on browsers.
	    unset($_POST);
	}
    }
}

// Disable modules (this must be after session_start and after conf has been loaded)
if (GETPOSTISSET('disablemodules')) {
    $_SESSION["disablemodules"] = GETPOST('disablemodules', 'alpha');
}
if (!empty($_SESSION["disablemodules"])) {
    $modulepartkeys = array('css', 'js', 'tabs', 'triggers', 'login', 'substitutions', 'menus', 'theme', 'sms', 'tpl', 'barcode', 'models', 'societe', 'hooks', 'dir', 'syslog', 'tpllinkable', 'contactelement', 'moduleforexternal');

    $disabled_modules = explode(',', $_SESSION["disablemodules"]);
    foreach ($disabled_modules as $module) {
	if ($module) {
	    if (empty($dyap_conf->$module)) {
		$dyap_conf->$module = new stdClass(); // To avoid warnings
	    }
	    $dyap_conf->$module->enabled = false;
	    foreach ($modulepartkeys as $modulepartkey) {
		unset($dyap_conf->modules_parts[$modulepartkey][$module]);
	    }
	    if ($module == 'fournisseur') {		// Special case
		$dyap_conf->supplier_order->enabled = 0;
		$dyap_conf->supplier_invoice->enabled = 0;
	    }
	}
    }
}

// Set current modulepart
$modulepart = explode("/", $_SERVER["PHP_SELF"]);
if (is_array($modulepart) && count($modulepart) > 0) {
	foreach ($dyap_conf->modules as $module) {
		if (in_array($module, $modulepart)) {
			$dyap_conf->modulepart = $module;
			break;
		}
	}
}

/*
 * Phase authentication / login
 */
$entitytotest = (GETPOST('entity', 'int') ? GETPOST('entity', 'int') : (!empty($dyap_conf->entity) ? $dyap_conf->entity : 1));
if (!is_object($dyap_langs)) { // This can occurs when calling page with NOREQUIRETRAN defined, however we need langs for error messages.
    include_once DOL_DOCUMENT_ROOT.'/core/class/translate.class.php';
    $dyap_langs = new Translate("", $dyap_conf);
    $langcode = (GETPOST('lang', 'aZ09', 1) ?GETPOST('lang', 'aZ09', 1) : (empty($dyap_conf->global->MAIN_LANG_DEFAULT) ? 'auto' : $dyap_conf->global->MAIN_LANG_DEFAULT));
    if (defined('MAIN_LANG_DEFAULT')) {
	$langcode = constant('MAIN_LANG_DEFAULT');
    }
    $dyap_langs->setDefaultLang($langcode);
}
$login=\wp_get_current_user()->user_login;
$resultFetchUser = $dyap_user->fetch('', $login, '', 1, ($entitytotest > 0 ? $entitytotest : -1)); // login is the same as wordpress login.
if ($resultFetchUser <= 0) {
    dol_syslog('User not found, connexion refused');
    session_destroy();
    session_set_cookie_params(0, '/', null, (empty($dolibarr_main_force_https) ? false : true), true); // Add tag secure and httponly on session cookie
    session_name($sessionname);
    session_start();

    if ($resultFetchUser == 0) {
	// Load translation files required by page
	$dyap_langs->loadLangs(array('main', 'errors'));

	$_SESSION["dol_loginmesg"] = $dyap_langs->transnoentitiesnoconv("ErrorCantLoadUserFromDolibarrDatabase", $login);

	$dyap_user->trigger_mesg = 'ErrorCantLoadUserFromDolibarrDatabase - login='.$login;
    }
    if ($resultFetchUser < 0) {
	$_SESSION["dol_loginmesg"] = $dyap_user->error;

	$dyap_user->trigger_mesg = $dyap_user->error;
    }

    // Call trigger
    $result = $dyap_user->call_trigger('USER_LOGIN_FAILED', $dyap_user);
    if ($result < 0) {
	$error++;
    }
    // End call triggers


    // Hooks on failed login
    $dyap_action = '';
    $dyap_hookmanager->initHooks(array('login'));
    $parameters = array('dol_authmode'=>$dol_authmode, 'dol_loginmesg'=>$_SESSION["dol_loginmesg"]);
    $reshook = $dyap_hookmanager->executeHooks('afterLoginFailed', $parameters, $dyap_user, $dyap_action); // Note that $action and $object may have been modified by some hooks
    if ($reshook < 0) {
	$error++;
    }

    /*
     * DyaPress following code is not needed, in case of bad login, no DyaPress is available at all.
     * Original code follow :
    $paramsurl = array();
    if (GETPOST('textbrowser', 'int')) {
	$paramsurl[] = 'textbrowser='.GETPOST('textbrowser', 'int');
    }
    if (GETPOST('nojs', 'int')) {
	$paramsurl[] = 'nojs='.GETPOST('nojs', 'int');
    }
    if (GETPOST('lang', 'aZ09')) {
	$paramsurl[] = 'lang='.GETPOST('lang', 'aZ09');
    }
    header('Location: '.DOL_URL_ROOT.'/index.php'.(count($paramsurl) ? '?'.implode('&', $paramsurl) : ''));
    exit;
     */
    return;
} else {
    // User is loaded, we may need to change language for him according to its choice
    if (!empty($dyap_user->conf->MAIN_LANG_DEFAULT)) {
	$dyap_langs->setDefaultLang($dyap_user->conf->MAIN_LANG_DEFAULT);
    }

    // Initialize technical object to manage hooks of page. Note that conf->hooks_modules contains array of hook context
    $dyap_hookmanager->initHooks(array('main'));

    // Code for search criteria persistence.
    if (!empty($_GET['save_lastsearch_values'])) {    // We must use $_GET here
	$relativepathstring = preg_replace('/\?.*$/', '', $_SERVER["HTTP_REFERER"]);
	$relativepathstring = preg_replace('/^https?:\/\/[^\/]*/', '', $relativepathstring); // Get full path except host server
	// Clean $relativepathstring
	if (constant('DOL_URL_ROOT')) {
	    $relativepathstring = preg_replace('/^'.preg_quote(constant('DOL_URL_ROOT'), '/').'/', '', $relativepathstring);
	}
	$relativepathstring = preg_replace('/^\//', '', $relativepathstring);
	$relativepathstring = preg_replace('/^custom\//', '', $relativepathstring);
	//var_dump($relativepathstring);

	// We click on a link that leave a page we have to save search criteria, contextpage, limit and page. We save them from tmp to no tmp
	if (!empty($_SESSION['lastsearch_values_tmp_'.$relativepathstring])) {
	    $_SESSION['lastsearch_values_'.$relativepathstring] = $_SESSION['lastsearch_values_tmp_'.$relativepathstring];
	    unset($_SESSION['lastsearch_values_tmp_'.$relativepathstring]);
	}
	if (!empty($_SESSION['lastsearch_contextpage_tmp_'.$relativepathstring])) {
	    $_SESSION['lastsearch_contextpage_'.$relativepathstring] = $_SESSION['lastsearch_contextpage_tmp_'.$relativepathstring];
	    unset($_SESSION['lastsearch_contextpage_tmp_'.$relativepathstring]);
	}
	if (!empty($_SESSION['lastsearch_page_tmp_'.$relativepathstring]) && $_SESSION['lastsearch_page_tmp_'.$relativepathstring] > 0) {
	    $_SESSION['lastsearch_page_'.$relativepathstring] = $_SESSION['lastsearch_page_tmp_'.$relativepathstring];
	    unset($_SESSION['lastsearch_page_tmp_'.$relativepathstring]);
	}
	if (!empty($_SESSION['lastsearch_limit_tmp_'.$relativepathstring]) && $_SESSION['lastsearch_limit_tmp_'.$relativepathstring] != $dyap_conf->liste_limit) {
	    $_SESSION['lastsearch_limit_'.$relativepathstring] = $_SESSION['lastsearch_limit_tmp_'.$relativepathstring];
	    unset($_SESSION['lastsearch_limit_tmp_'.$relativepathstring]);
	}
    }

    $dyap_action = '';
    $reshook = $dyap_hookmanager->executeHooks('updateSession', array(), $dyap_user, $dyap_action);
    if ($reshook < 0) {
	setEventMessages($dyap_hookmanager->error, $dyap_hookmanager->errors, 'errors');
    }
}

// Is it a new session that has started ?
// If we are here, this means authentication was successfull.
if (!isset($_SESSION["dol_login"])) {
    // New session for this login has started.
    $error = 0;

    // Store value into session (values always stored)
    $_SESSION["dol_login"] = $dyap_user->login;
    $_SESSION["dol_authmode"] = isset($dol_authmode) ? $dol_authmode : '';
    $_SESSION["dol_tz"] = isset($dol_tz) ? $dol_tz : '';
    $_SESSION["dol_tz_string"] = isset($dol_tz_string) ? $dol_tz_string : '';
    $_SESSION["dol_dst"] = isset($dol_dst) ? $dol_dst : '';
    $_SESSION["dol_dst_observed"] = isset($dol_dst_observed) ? $dol_dst_observed : '';
    $_SESSION["dol_dst_first"] = isset($dol_dst_first) ? $dol_dst_first : '';
    $_SESSION["dol_dst_second"] = isset($dol_dst_second) ? $dol_dst_second : '';
    $_SESSION["dol_screenwidth"] = isset($dol_screenwidth) ? $dol_screenwidth : '';
    $_SESSION["dol_screenheight"] = isset($dol_screenheight) ? $dol_screenheight : '';
    $_SESSION["dol_company"] = getDolGlobalString("MAIN_INFO_SOCIETE_NOM");
    $_SESSION["dol_entity"] = $dyap_conf->entity;
    // Store value into session (values stored only if defined)
    if (!empty($dol_hide_topmenu)) {
	$_SESSION['dol_hide_topmenu'] = $dol_hide_topmenu;
    }
    if (!empty($dol_hide_leftmenu)) {
	$_SESSION['dol_hide_leftmenu'] = $dol_hide_leftmenu;
    }
    if (!empty($dol_optimize_smallscreen)) {
	$_SESSION['dol_optimize_smallscreen'] = $dol_optimize_smallscreen;
    }
    if (!empty($dol_no_mouse_hover)) {
	$_SESSION['dol_no_mouse_hover'] = $dol_no_mouse_hover;
    }
    if (!empty($dol_use_jmobile)) {
	$_SESSION['dol_use_jmobile'] = $dol_use_jmobile;
    }

    dol_syslog("This is a new started user session. _SESSION['dol_login']=".$_SESSION["dol_login"]." Session id=".session_id());

    $dyap_db->begin();

    $dyap_user->update_last_login_date();

    $loginfo = 'TZ='.$_SESSION["dol_tz"].';TZString='.$_SESSION["dol_tz_string"].';Screen='.$_SESSION["dol_screenwidth"].'x'.$_SESSION["dol_screenheight"];

    // Call triggers for the "security events" log
    $dyap_user->trigger_mesg = $loginfo;

    // Call trigger
    $result = $dyap_user->call_trigger('USER_LOGIN', $dyap_user);
    if ($result < 0) {
	$error++;
    }
    // End call triggers

    // Hooks on successfull login
    $dyap_action = '';
    $dyap_hookmanager->initHooks(array('login'));
    $parameters = array('dol_authmode'=>$dol_authmode, 'dol_loginfo'=>$loginfo);
    $reshook = $dyap_hookmanager->executeHooks('afterLogin', $parameters, $dyap_user, $dyap_action); // Note that $action and $object may have been modified by some hooks
    if ($reshook < 0) {
	$error++;
    }

    if ($error) {
	$dyap_db->rollback();
	session_destroy();
	dol_print_error($dyap_db, 'Error in some triggers USER_LOGIN or in some hooks afterLogin');
	exit;
    } else {
	$dyap_db->commit();
    }

    // Change landing page if defined.
    $landingpage = (empty($dyap_user->conf->MAIN_LANDING_PAGE) ? (empty($dyap_conf->global->MAIN_LANDING_PAGE) ? '' : $dyap_conf->global->MAIN_LANDING_PAGE) : $dyap_user->conf->MAIN_LANDING_PAGE);
    if (!empty($landingpage)) {    // Example: /index.php
	$newpath = dol_buildpath($landingpage, 1);
	if ($_SERVER["PHP_SELF"] != $newpath) {   // not already on landing page (avoid infinite loop)
	    header('Location: '.$newpath);
	    exit;
	}
    }
}

/* DyaPress: admin user on WordPress is now an admin user on DyaPress */
if ( wp_get_current_user()->user_email == get_option( 'admin_email' ) )
    $dyap_user->admin = 1; // Force admin user
dol_syslog("DyaPress from WordPress logged session=".$login, LOG_DEBUG);

// If user admin, we force the rights-based modules
if ($dyap_user->admin) {
    $dyap_user->rights->user->user->lire = 1;
    $dyap_user->rights->user->user->creer = 1;
    $dyap_user->rights->user->user->password = 1;
    $dyap_user->rights->user->user->supprimer = 1;
    $dyap_user->rights->user->self->creer = 1;
    $dyap_user->rights->user->self->password = 1;

    //Required if advanced permissions are used with MAIN_USE_ADVANCED_PERMS
    if (!empty($dyap_conf->global->MAIN_USE_ADVANCED_PERMS)) {
	if (empty($dyap_user->rights->user->user_advance)) {
	    $dyap_user->rights->user->user_advance = new stdClass(); // To avoid warnings
	}
	if (empty($dyap_user->rights->user->self_advance)) {
	    $dyap_user->rights->user->self_advance = new stdClass(); // To avoid warnings
	}
	if (empty($dyap_user->rights->user->group_advance)) {
	    $dyap_user->rights->user->group_advance = new stdClass(); // To avoid warnings
	}

	$dyap_user->rights->user->user_advance->readperms = 1;
	$dyap_user->rights->user->user_advance->write = 1;
	$dyap_user->rights->user->self_advance->readperms = 1;
	$dyap_user->rights->user->self_advance->writeperms = 1;
	$dyap_user->rights->user->group_advance->read = 1;
	$dyap_user->rights->user->group_advance->readperms = 1;
	$dyap_user->rights->user->group_advance->write = 1;
	$dyap_user->rights->user->group_advance->delete = 1;
    }
}

/*
 * Overwrite some configs globals (try to avoid this and have code to use instead $dyap_user->conf->xxx)
 */

// Set liste_limit
if (isset($dyap_user->conf->MAIN_SIZE_LISTE_LIMIT)) {
    $dyap_conf->liste_limit = $dyap_user->conf->MAIN_SIZE_LISTE_LIMIT; // Can be 0
}
if (isset($dyap_user->conf->PRODUIT_LIMIT_SIZE)) {
    $dyap_conf->product->limit_size = $dyap_user->conf->PRODUIT_LIMIT_SIZE; // Can be 0
}

// Replace conf->css by personalized value if theme not forced
if (empty($dyap_conf->global->MAIN_FORCETHEME) && !empty($dyap_user->conf->MAIN_THEME)) {
    $dyap_conf->theme = $dyap_user->conf->MAIN_THEME;
    $dyap_conf->css = "/theme/".$dyap_conf->theme."/style.css.php";
}

// Case forcing style from url
if (GETPOST('theme', 'aZ09')) {
	$dyap_conf->theme = GETPOST('theme', 'aZ09', 1);
	$dyap_conf->css = "/theme/".$dyap_conf->theme."/style.css.php";
}

// Set javascript option
if (GETPOST('nojs', 'int')) {  // If javascript was not disabled on URL
	$dyap_conf->use_javascript_ajax = 0;
} else {
	if (!empty($dyap_user->conf->MAIN_DISABLE_JAVASCRIPT)) {
		$dyap_conf->use_javascript_ajax = !$dyap_user->conf->MAIN_DISABLE_JAVASCRIPT;
	}
}

// Set MAIN_OPTIMIZEFORTEXTBROWSER for user (must be after login part)
if (empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER) && !empty($dyap_user->conf->MAIN_OPTIMIZEFORTEXTBROWSER)) {
	$dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER = $dyap_user->conf->MAIN_OPTIMIZEFORTEXTBROWSER;
}

// set MAIN_OPTIMIZEFORCOLORBLIND for user
$dyap_conf->global->MAIN_OPTIMIZEFORCOLORBLIND = empty($dyap_user->conf->MAIN_OPTIMIZEFORCOLORBLIND) ? '' : $dyap_user->conf->MAIN_OPTIMIZEFORCOLORBLIND;

// Set terminal output option according to conf->browser.
if (GETPOST('dol_hide_leftmenu', 'int') || !empty($_SESSION['dol_hide_leftmenu'])) {
	$dyap_conf->dol_hide_leftmenu = 1;
}
if (GETPOST('dol_hide_topmenu', 'int') || !empty($_SESSION['dol_hide_topmenu'])) {
	$dyap_conf->dol_hide_topmenu = 1;
}
if (GETPOST('dol_optimize_smallscreen', 'int') || !empty($_SESSION['dol_optimize_smallscreen'])) {
	$dyap_conf->dol_optimize_smallscreen = 1;
}
if (GETPOST('dol_no_mouse_hover', 'int') || !empty($_SESSION['dol_no_mouse_hover'])) {
	$dyap_conf->dol_no_mouse_hover = 1;
}
if (GETPOST('dol_use_jmobile', 'int') || !empty($_SESSION['dol_use_jmobile'])) {
	$dyap_conf->dol_use_jmobile = 1;
}
if (!empty($dyap_conf->browser->layout) && $dyap_conf->browser->layout != 'classic') {
	$dyap_conf->dol_no_mouse_hover = 1;
}
if ((!empty($dyap_conf->browser->layout) && $dyap_conf->browser->layout == 'phone')
	|| (!empty($_SESSION['dol_screenwidth']) && $_SESSION['dol_screenwidth'] < 400)
	|| (!empty($_SESSION['dol_screenheight']) && $_SESSION['dol_screenheight'] < 400)
) {
	$dyap_conf->dol_optimize_smallscreen = 1;
}
// Replace themes bugged with jmobile with eldy
if (!empty($dyap_conf->dol_use_jmobile) && in_array($dyap_conf->theme, array('bureau2crea', 'cameleo', 'amarok'))) {
	$dyap_conf->theme = 'eldy';
	$dyap_conf->css = "/theme/".$dyap_conf->theme."/style.css.php";
}

if (!defined('NOREQUIRETRAN')) {
    if (!GETPOST('lang', 'aZ09')) {	// If language was not forced on URL
	// If user has chosen its own language
	if (!empty($dyap_user->conf->MAIN_LANG_DEFAULT)) {
	    // If different than current language
	    //print ">>>".$dyap_langs->getDefaultLang()."-".$dyap_user->conf->MAIN_LANG_DEFAULT;
	    if ($dyap_langs->getDefaultLang() != $dyap_user->conf->MAIN_LANG_DEFAULT) {
		$dyap_langs->setDefaultLang($dyap_user->conf->MAIN_LANG_DEFAULT);
	    }
	}
    }
}

// If the login is not recovered, it is identified with an account that does not exist.
// Hacking attempt?
if (!$dyap_user->login) {
    accessforbidden();
}

// Check if user is active
if ($dyap_user->statut < 1) {
    // If not active, we refuse the user
    $dyap_langs->loadLangs(array("errors", "other"));
    dol_syslog("Authentication KO as login is disabled", LOG_NOTICE);
    accessforbidden($dyap_langs->trans("ErrorLoginDisabled"));
    exit;
}

// Load permissions
$dyap_user->getrights();


dol_syslog("--- Access to ".(empty($_SERVER["REQUEST_METHOD"]) ? '' : $_SERVER["REQUEST_METHOD"].' ').$_SERVER["PHP_SELF"].' - action='.GETPOST('action', 'aZ09').', massaction='.GETPOST('massaction', 'aZ09').(defined('NOTOKENRENEWAL') ? ' NOTOKENRENEWAL='.constant('NOTOKENRENEWAL') : ''), LOG_NOTICE);
//Another call for easy debugg
//dol_syslog("Access to ".$_SERVER["PHP_SELF"].' '.$_SERVER["HTTP_REFERER"].' GET='.join(',',array_keys($_GET)).'->'.join(',',$_GET).' POST:'.join(',',array_keys($_POST)).'->'.join(',',$_POST));

// Load main languages files
if (!defined('NOREQUIRETRAN')) {
	// Load translation files required by page
	$dyap_langs->loadLangs(array('main', 'dict'));
}

// Define some constants used for style of arrays
$dyap_bc = array(0=>'class="impair"', 1=>'class="pair"');
$dyap_bcdd = array(0=>'class="drag drop oddeven"', 1=>'class="drag drop oddeven"');
$dyap_bcnd = array(0=>'class="nodrag nodrop nohover"', 1=>'class="nodrag nodrop nohoverpair"'); // Used for tr to add new lines
$dyap_bctag = array(0=>'class="impair tagtr"', 1=>'class="pair tagtr"');

// FIXME DYAPRESS : add the following globals ?

// Define messages variables
$mesg = ''; $warning = ''; $error = 0;
// deprecated, see setEventMessages() and dol_htmloutput_events()
$mesgs = array(); $warnings = array(); $errors = array();

// Constants used to defined number of lines in textarea
if (empty($dyap_conf->browser->firefox)) {
	define('ROWS_1', 1);
	define('ROWS_2', 2);
	define('ROWS_3', 3);
	define('ROWS_4', 4);
	define('ROWS_5', 5);
	define('ROWS_6', 6);
	define('ROWS_7', 7);
	define('ROWS_8', 8);
	define('ROWS_9', 9);
} else {
	define('ROWS_1', 0);
	define('ROWS_2', 1);
	define('ROWS_3', 2);
	define('ROWS_4', 3);
	define('ROWS_5', 4);
	define('ROWS_6', 5);
	define('ROWS_7', 6);
	define('ROWS_8', 7);
	define('ROWS_9', 8);
}

$heightforframes = 50;

// Init menu manager
if (!defined('NOREQUIREMENU')) {
	if (empty($dyap_user->socid)) {    // If internal user or not defined
		$dyap_conf->standard_menu = (empty($dyap_conf->global->MAIN_MENU_STANDARD_FORCED) ? (empty($dyap_conf->global->MAIN_MENU_STANDARD) ? 'eldy_menu.php' : $dyap_conf->global->MAIN_MENU_STANDARD) : $dyap_conf->global->MAIN_MENU_STANDARD_FORCED);
	} else {
		// If external user
		$dyap_conf->standard_menu = (empty($dyap_conf->global->MAIN_MENUFRONT_STANDARD_FORCED) ? (empty($dyap_conf->global->MAIN_MENUFRONT_STANDARD) ? 'eldy_menu.php' : $dyap_conf->global->MAIN_MENUFRONT_STANDARD) : $dyap_conf->global->MAIN_MENUFRONT_STANDARD_FORCED);
	}

	// Load the menu manager (only if not already done)
	$file_menu = $dyap_conf->standard_menu;
	if (GETPOST('menu', 'alpha')) {
		$file_menu = GETPOST('menu', 'alpha'); // example: menu=eldy_menu.php
	}
	if (!class_exists('\DyaPress\MenuManager')) {
		$menufound = 0;
		$dirmenus = array_merge(array("/core/menus/"), (array) $dyap_conf->modules_parts['menus']);
		foreach ($dirmenus as $dirmenu) {
			$menufound = dol_include_once($dirmenu."standard/".$file_menu);
			if (class_exists('\DyaPress\MenuManager')) {
				break;
			}
		}
		if (!class_exists('\DyaPress\MenuManager')) {	// If failed to include, we try with standard eldy_menu.php
			dol_syslog("You define a menu manager '".$file_menu."' that can not be loaded.", LOG_WARNING);
			$file_menu = 'eldy_menu.php';
			include_once DOL_DOCUMENT_ROOT."/core/menus/standard/".$file_menu;
		}
	}
	$dyap_menumanager = new MenuManager($dyap_db, empty($dyap_user->socid) ? 0 : 1);
	$dyap_menumanager->loadMenu();
}


// Functions

if (!function_exists("llxHeader")) {
    /**
     *	Show HTML header HTML + BODY + Top menu + left menu + DIV
     *
     * @param 	string 			$head				Optionnal head lines
     * @param 	string 			$title				HTML title
     * @param	string			$help_url			Url links to help page
     * 		                            			Syntax is: For a wiki page: EN:EnglishPage|FR:FrenchPage|ES:SpanishPage
     *                                  			For other external page: http://server/url
     * @param	string			$target				Target to use on links
     * @param 	int    			$disablejs			More content into html header
     * @param 	int    			$disablehead		More content into html header
     * @param 	array|string  	$arrayofjs			Array of complementary js files
     * @param 	array|string  	$arrayofcss			Array of complementary css files
     * @param	string			$morequerystring	Query string to add to the link "print" to get same parameters (use only if autodetect fails)
     * @param   string  		$morecssonbody      More CSS on body tag. For example 'classforhorizontalscrolloftabs'.
     * @param	string			$replacemainareaby	Replace call to main_area() by a print of this string
     * @param	int				$disablenofollow	Disable the "nofollow" on page
     * @return	void
     */
    function llxHeader($head = '', $title = '', $help_url = '', $target = '', $disablejs = 0, $disablehead = 0, $arrayofjs = '', $arrayofcss = '', $morequerystring = '', $morecssonbody = '', $replacemainareaby = '', $disablenofollow = 0)
    {
	global $dyap_conf;

	// FIXME: the following code replace top_htmlhead() original call
	global $dyap_ob_content, $dyap_output_llx_header;
	if ( !$dyap_output_llx_header ) {
	    $dyap_head = '';

	    if (is_array($arrayofcss)) {
		foreach ($arrayofcss as $css) {
		    $dyap_head .= '<link rel="stylesheet" href="'.dol_buildpath($css, 1).'" type="text/css" media="all" />'."\n";
		}
	    }
	    if (is_array($arrayofjs)) {
		foreach ($arrayofjs as $js) {
		    $dyap_head .= '<script type="text/javascript" src="'.dol_buildpath($js, 1).'"></script>'."\n";
		}
	    }
	    // FIXME: add $arrayofjs and $arrayofcss just before </head>
	    if (!empty($dyap_head)) {
		$dyap_ob_content = preg_replace('#</head>#i', $dyap_head.'</head>', $dyap_ob_content);
	    }

	    // FIXME: admin_title filter has already been launched
	    if (!empty($title)) {
		$dyap_ob_content = preg_replace('#<title>#i', '<title>'.$title.' &lsaquo; ', $dyap_ob_content);
	    }

	    echo $dyap_ob_content; // FIXME: transform title before output

	    $dyap_output_llx_header = 1;
	}

	$tmpcsstouse = 'sidebar-collapse'.($morecssonbody ? ' '.$morecssonbody : '');
	// If theme MD and classic layer, we open the menulayer by default.
	if ($dyap_conf->theme == 'md' && !in_array($dyap_conf->browser->layout, array('phone', 'tablet')) && empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) {
	    global $mainmenu;
	    if ($mainmenu != 'website') {
		$tmpcsstouse = $morecssonbody; // We do not use sidebar-collpase by default to have menuhider open by default.
	    }
	}

	if (!empty($dyap_conf->global->MAIN_OPTIMIZEFORCOLORBLIND)) {
	    $tmpcsstouse .= ' colorblind-'.strip_tags($dyap_conf->global->MAIN_OPTIMIZEFORCOLORBLIND);
	}

	print '<div id="mainbody" class="'.$tmpcsstouse.'">'."\n";

	// top menu and left menu area
	if (empty($dyap_conf->dol_hide_topmenu) || GETPOST('dol_invisible_topmenu','int')) {
	    top_menu($head, $title, $target, $disablejs, $disablehead, $arrayofjs, $arrayofcss, $morequerystring, $help_url);
	}

	if (empty($dyap_conf->dol_hide_leftmenu)) {
	    left_menu('', $help_url, '', '', 1, $title, 1);
	}

	// main area
	if ($replacemainareaby) {
	    print $replacemainareaby;
	    return;
	}
	main_area($title);
    }
}


/**
 *  Show HTTP header
 *
 *  @param  string  $contenttype    Content type. For example, 'text/html'
 *  @param	int		$forcenocache	Force disabling of cache for the page
 *  @return	void
 */
function top_httphead($contenttype='text/html', $forcenocache=0)
{
	global $dyap_conf, $dyap_hookmanager;

	if ($contenttype == 'text/html') {
		header("Content-Type: text/html; charset=".$dyap_conf->file->character_set_client);
	} else {
		header("Content-Type: ".$contenttype);
	}

	// Security options
	header("X-Content-Type-Options: nosniff"); // With the nosniff option, if the server says the content is text/html, the browser will render it as text/html (note that most browsers now force this option to on)
	if (!defined('XFRAMEOPTIONS_ALLOWALL')) {
		header("X-Frame-Options: SAMEORIGIN"); // Frames allowed only if on same domain (stop some XSS attacks)
	} else {
		header("X-Frame-Options: ALLOWALL");
	}
	//header("X-XSS-Protection: 1");      		// XSS filtering protection of some browsers (note: use of Content-Security-Policy is more efficient). Disabled as deprecated.
	if (!defined('FORCECSP')) {
		//if (! isset($dyap_conf->global->MAIN_HTTP_CONTENT_SECURITY_POLICY))
		//{
		//	// A default security policy that keep usage of js external component like ckeditor, stripe, google, working
		//	$contentsecuritypolicy = "font-src *; img-src *; style-src * 'unsafe-inline' 'unsafe-eval'; default-src 'self' *.stripe.com 'unsafe-inline' 'unsafe-eval'; script-src 'self' *.stripe.com 'unsafe-inline' 'unsafe-eval'; frame-src 'self' *.stripe.com; connect-src 'self';";
		//}
		//else
		$contentsecuritypolicy = empty($dyap_conf->global->MAIN_HTTP_CONTENT_SECURITY_POLICY) ? '' : $dyap_conf->global->MAIN_HTTP_CONTENT_SECURITY_POLICY;

		if (!is_object($dyap_hookmanager)) {
			$dyap_hookmanager = new HookManager($db);
		}
		$dyap_hookmanager->initHooks(array("main"));

		$parameters = array('contentsecuritypolicy'=>$contentsecuritypolicy);
		$result = $dyap_hookmanager->executeHooks('setContentSecurityPolicy', $parameters); // Note that $action and $object may have been modified by some hooks
		if ($result > 0) {
			$contentsecuritypolicy = $dyap_hookmanager->resPrint; // Replace CSP
		} else {
			$contentsecuritypolicy .= $dyap_hookmanager->resPrint; // Concat CSP
		}

		if (!empty($contentsecuritypolicy)) {
			// For example, to restrict 'script', 'object', 'frames' or 'img' to some domains:
			// script-src https://api.google.com https://anotherhost.com; object-src https://youtube.com; frame-src https://youtube.com; img-src: https://static.example.com
			// For example, to restrict everything to one domain, except 'object', ...:
			// default-src https://cdn.example.net; object-src 'none'
			// For example, to restrict everything to itself except img that can be on other servers:
			// default-src 'self'; img-src *;
			// Pre-existing site that uses too much inline code to fix but wants to ensure resources are loaded only over https and disable plugins:
			// default-src http: https: 'unsafe-eval' 'unsafe-inline'; object-src 'none'
			header("Content-Security-Policy: ".$contentsecuritypolicy);
		}
	} elseif (constant('FORCECSP')) {
		header("Content-Security-Policy: ".constant('FORCECSP'));
	}
	if ($forcenocache) {
		header("Cache-Control: no-cache, no-store, must-revalidate, max-age=0");
	}
}

/**
 * Ouput html header of a page.
 * This code is also duplicated into security2.lib.php::dol_loginfunction
 *
 * @param 	string 	$head			 Optionnal head lines
 * @param 	string 	$title			 HTML title
 * @param 	int    	$disablejs		 Disable js output
 * @param 	int    	$disablehead	 Disable head output
 * @param 	array  	$arrayofjs		 Array of complementary js files
 * @param 	array  	$arrayofcss		 Array of complementary css files
 * @param 	int    	$disablejmobile	 Disable jmobile (No more used)
 * @param   int     $disablenofollow Disable no follow tag
 * @return	void
 */
function top_htmlhead($head, $title='', $disablejs=0, $disablehead=0, $arrayofjs='', $arrayofcss='', $disablejmobile=0, $disablenofollow=0)
{
    global $dyap_output_llx_header;

    if ( empty( $dyap_output_llx_header ) ) {
	global $dyap_db, $dyap_conf, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

	top_httphead();

	if (empty($dyap_conf->css)) {
		$dyap_conf->css = '/theme/eldy/style.css.php'; // If not defined, eldy by default
	}

	print '<!doctype html>'."\n";

	print '<html lang="'.substr($dyap_langs->defaultlang, 0, 2).'">'."\n";

	//print '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="fr">'."\n";
	if (empty($disablehead)) {
		if (!is_object($dyap_hookmanager)) {
			$dyap_hookmanager = new HookManager($dyap_db);
		}
		$dyap_hookmanager->initHooks(array("main"));

		$ext = 'layout='.$dyap_conf->browser->layout.'&amp;version='.urlencode(DOL_VERSION);

		print "<head>\n";

		if (GETPOST('dol_basehref', 'alpha')) {
			print '<base href="'.dol_escape_htmltag(GETPOST('dol_basehref', 'alpha')).'">'."\n";
		}

		// Displays meta
		print '<meta charset="utf-8">'."\n";
		print '<meta name="robots" content="noindex'.($disablenofollow ? '' : ',nofollow').'">'."\n"; // Do not index
		print '<meta name="viewport" content="width=device-width, initial-scale=1.0">'."\n"; // Scale for mobile device
		print '<meta name="author" content="Dolibarr Development Team">'."\n";
		if (getDolGlobalInt('MAIN_FEATURES_LEVEL')) {
			print '<meta name="MAIN_FEATURES_LEVEL" content="'.getDolGlobalInt('MAIN_FEATURES_LEVEL').'">'."\n";
		}
		// Favicon
		$favicon = DOL_URL_ROOT.'/theme/dolibarr_256x256_color.png';
		if (!empty($dyap_mysoc->logo_squarred_mini)) {
			$favicon = DOL_URL_ROOT.'/viewimage.php?cache=1&modulepart=mycompany&file='.urlencode('logos/thumbs/'.$dyap_mysoc->logo_squarred_mini);
		}
		if (!empty($dyap_conf->global->MAIN_FAVICON_URL)) {
			$favicon = $dyap_conf->global->MAIN_FAVICON_URL;
		}
		if (empty($dyap_conf->dol_use_jmobile)) {
			print '<link rel="shortcut icon" type="image/x-icon" href="'.$favicon.'"/>'."\n"; // Not required into an Android webview
		}

		//if (empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) print '<link rel="top" title="'.$dyap_langs->trans("Home").'" href="'.(DOL_URL_ROOT?DOL_URL_ROOT:'/').'">'."\n";
		//if (empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) print '<link rel="copyright" title="GNU General Public License" href="https://www.gnu.org/copyleft/gpl.html#SEC1">'."\n";
		//if (empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) print '<link rel="author" title="Dolibarr Development Team" href="https://www.dolibarr.org">'."\n";

		// Mobile appli like icon
		$manifest = DOL_URL_ROOT.'/theme/'.$dyap_conf->theme.'/manifest.json.php';
		if (!empty($manifest)) {
			print '<link rel="manifest" href="'.$manifest.'" />'."\n";
		}

		if (!empty($dyap_conf->global->THEME_ELDY_TOPMENU_BACK1)) {
			// TODO: use auto theme color switch
			print '<meta name="theme-color" content="rgb('.$dyap_conf->global->THEME_ELDY_TOPMENU_BACK1.')">'."\n";
		}

		// Auto refresh page
		if (GETPOST('autorefresh', 'int') > 0) {
			print '<meta http-equiv="refresh" content="'.GETPOST('autorefresh', 'int').'">';
		}

		// Displays title
		$appli = constant('DOL_APPLICATION_TITLE');
		if (!empty($dyap_conf->global->MAIN_APPLICATION_TITLE)) {
			$appli = $dyap_conf->global->MAIN_APPLICATION_TITLE;
		}

		print '<title>';
		$titletoshow = '';
		if ($title && !empty($dyap_conf->global->MAIN_HTML_TITLE) && preg_match('/noapp/', $dyap_conf->global->MAIN_HTML_TITLE)) {
			$titletoshow = dol_htmlentities($title);
		} elseif ($title) {
			$titletoshow = dol_htmlentities($appli.' - '.$title);
		} else {
			$titletoshow = dol_htmlentities($appli);
		}

		$parameters = array('title'=>$titletoshow);
		$result = $dyap_hookmanager->executeHooks('setHtmlTitle', $parameters); // Note that $action and $object may have been modified by some hooks
		if ($result > 0) {
			$titletoshow = $dyap_hookmanager->resPrint; // Replace Title to show
		} else {
			$titletoshow .= $dyap_hookmanager->resPrint; // Concat to Title to show
		}

		print $titletoshow;
		print '</title>';

		print "\n";

		if (GETPOST('version', 'int')) {
			$ext = 'version='.GETPOST('version', 'int'); // usefull to force no cache on css/js
		}

		$themeparam = '?lang='.$dyap_langs->defaultlang.'&amp;theme='.$dyap_conf->theme.(GETPOST('optioncss', 'aZ09') ? '&amp;optioncss='.GETPOST('optioncss', 'aZ09', 1) : '').'&amp;userid='.$dyap_user->id.'&amp;entity='.$dyap_conf->entity;
		$themeparam .= ($ext ? '&amp;'.$ext : '').'&amp;revision='.getDolGlobalInt("MAIN_IHM_PARAMS_REV");
		if (!empty($_SESSION['dol_resetcache'])) {
			$themeparam .= '&amp;dol_resetcache='.$_SESSION['dol_resetcache'];
		}
		if (GETPOSTISSET('dol_hide_topmenu')) {
			$themeparam .= '&amp;dol_hide_topmenu='.GETPOST('dol_hide_topmenu', 'int');
		}
		if (GETPOSTISSET('dol_hide_leftmenu')) {
			$themeparam .= '&amp;dol_hide_leftmenu='.GETPOST('dol_hide_leftmenu', 'int');
		}
		if (GETPOSTISSET('dol_optimize_smallscreen')) {
			$themeparam .= '&amp;dol_optimize_smallscreen='.GETPOST('dol_optimize_smallscreen', 'int');
		}
		if (GETPOSTISSET('dol_no_mouse_hover')) {
			$themeparam .= '&amp;dol_no_mouse_hover='.GETPOST('dol_no_mouse_hover', 'int');
		}
		if (GETPOSTISSET('dol_use_jmobile')) {
			$themeparam .= '&amp;dol_use_jmobile='.GETPOST('dol_use_jmobile', 'int'); $dyap_conf->dol_use_jmobile = GETPOST('dol_use_jmobile', 'int');
		}
		if (GETPOSTISSET('THEME_DARKMODEENABLED')) {
			$themeparam .= '&amp;THEME_DARKMODEENABLED='.GETPOST('THEME_DARKMODEENABLED', 'int');
		}
		if (GETPOSTISSET('THEME_SATURATE_RATIO')) {
			$themeparam .= '&amp;THEME_SATURATE_RATIO='.GETPOST('THEME_SATURATE_RATIO', 'int');
		}

		if (!empty($dyap_conf->global->MAIN_ENABLE_FONT_ROBOTO)) {
			print '<link rel="preconnect" href="https://fonts.gstatic.com">'."\n";
			print '<link href="https://fonts.googleapis.com/css2?family=Roboto:wght@200;300;400;500;600&display=swap" rel="stylesheet">'."\n";
		}

		if (!defined('DISABLE_JQUERY') && !$disablejs && $dyap_conf->use_javascript_ajax) {
			print '<!-- Includes CSS for JQuery (Ajax library) -->'."\n";
			$jquerytheme = 'base';
			if (!empty($dyap_conf->global->MAIN_USE_JQUERY_THEME)) {
				$jquerytheme = $dyap_conf->global->MAIN_USE_JQUERY_THEME;
			}
			if (constant('JS_JQUERY_UI')) {
				print '<link rel="stylesheet" type="text/css" href="'.JS_JQUERY_UI.'css/'.$jquerytheme.'/jquery-ui.min.css'.($ext ? '?'.$ext : '').'">'."\n"; // Forced JQuery
			} else {
				print '<link rel="stylesheet" type="text/css" href="'.DOL_URL_ROOT.'/includes/jquery/css/'.$jquerytheme.'/jquery-ui.css'.($ext ? '?'.$ext : '').'">'."\n"; // JQuery
			}
			if (!defined('DISABLE_JQUERY_JNOTIFY')) {
				print '<link rel="stylesheet" type="text/css" href="'.DOL_URL_ROOT.'/includes/jquery/plugins/jnotify/jquery.jnotify-alt.min.css'.($ext ? '?'.$ext : '').'">'."\n"; // JNotify
			}
			if (!defined('DISABLE_SELECT2') && (!empty($dyap_conf->global->MAIN_USE_JQUERY_MULTISELECT) || defined('REQUIRE_JQUERY_MULTISELECT'))) {     // jQuery plugin "mutiselect", "multiple-select", "select2"...
				$tmpplugin = empty($dyap_conf->global->MAIN_USE_JQUERY_MULTISELECT) ?constant('REQUIRE_JQUERY_MULTISELECT') : $dyap_conf->global->MAIN_USE_JQUERY_MULTISELECT;
				print '<link rel="stylesheet" type="text/css" href="'.DOL_URL_ROOT.'/includes/jquery/plugins/'.$tmpplugin.'/dist/css/'.$tmpplugin.'.css'.($ext ? '?'.$ext : '').'">'."\n";
			}
		}

		if (!defined('DISABLE_FONT_AWSOME')) {
			print '<!-- Includes CSS for font awesome -->'."\n";
			print '<link rel="stylesheet" type="text/css" href="'.DOL_URL_ROOT.'/theme/common/fontawesome-5/css/all.min.css'.($ext ? '?'.$ext : '').'">'."\n";
			print '<link rel="stylesheet" type="text/css" href="'.DOL_URL_ROOT.'/theme/common/fontawesome-5/css/v4-shims.min.css'.($ext ? '?'.$ext : '').'">'."\n";
		}

		print '<!-- Includes CSS for Dolibarr theme -->'."\n";
		// Output style sheets (optioncss='print' or ''). Note: $dyap_conf->css looks like '/theme/eldy/style.css.php'
		$themepath = dol_buildpath($dyap_conf->css, 1);
		$themesubdir = '';
		if (!empty($dyap_conf->modules_parts['theme'])) {	// This slow down
			foreach ($dyap_conf->modules_parts['theme'] as $reldir) {
				if (file_exists(dol_buildpath($reldir.$dyap_conf->css, 0))) {
					$themepath = dol_buildpath($reldir.$dyap_conf->css, 1);
					$themesubdir = $reldir;
					break;
				}
			}
		}

		//print 'themepath='.$themepath.' themeparam='.$themeparam;exit;
		print '<link rel="stylesheet" type="text/css" href="'.$themepath.$themeparam.'">'."\n";
		if (!empty($dyap_conf->global->MAIN_FIX_FLASH_ON_CHROME)) {
			print '<!-- Includes CSS that does not exists as a workaround of flash bug of chrome -->'."\n".'<link rel="stylesheet" type="text/css" href="filethatdoesnotexiststosolvechromeflashbug">'."\n";
		}

		// CSS forced by modules (relative url starting with /)
		if (!empty($dyap_conf->modules_parts['css'])) {
			$arraycss = (array) $dyap_conf->modules_parts['css'];
			foreach ($arraycss as $modcss => $filescss) {
				$filescss = (array) $filescss; // To be sure filecss is an array
				foreach ($filescss as $cssfile) {
					if (empty($cssfile)) {
						dol_syslog("Warning: module ".$modcss." declared a css path file into its descriptor that is empty.", LOG_WARNING);
					}
					// cssfile is a relative path
					$urlforcss = dol_buildpath($cssfile, 1);
					if ($urlforcss && $urlforcss != '/') {
						print '<!-- Includes CSS added by module '.$modcss.' -->'."\n".'<link rel="stylesheet" type="text/css" href="'.$urlforcss;
						// We add params only if page is not static, because some web server setup does not return content type text/css if url has parameters, so browser cache is not used.
						if (!preg_match('/\.css$/i', $cssfile)) {
							print $themeparam;
						}
						print '">'."\n";
					} else {
						dol_syslog("Warning: module ".$modcss." declared a css path file for a file we can't find.", LOG_WARNING);
					}
				}
			}
		}
		// CSS forced by page in top_htmlhead call (relative url starting with /)
		if (is_array($arrayofcss)) {
			foreach ($arrayofcss as $cssfile) {
				if (preg_match('/^(http|\/\/)/i', $cssfile)) {
					$urltofile = $cssfile;
				} else {
					$urltofile = dol_buildpath($cssfile, 1);
				}
				print '<!-- Includes CSS added by page -->'."\n".'<link rel="stylesheet" type="text/css" title="default" href="'.$urltofile;
				// We add params only if page is not static, because some web server setup does not return content type text/css if url has parameters and browser cache is not used.
				if (!preg_match('/\.css$/i', $cssfile)) {
					print $themeparam;
				}
				print '">'."\n";
			}
		}

		// Output standard javascript links
		if (!defined('DISABLE_JQUERY') && !$disablejs && !empty($dyap_conf->use_javascript_ajax)) {
			// JQuery. Must be before other includes
			print '<!-- Includes JS for JQuery -->'."\n";
			if (defined('JS_JQUERY') && constant('JS_JQUERY')) {
				print '<script src="'.JS_JQUERY.'jquery.min.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
			} else {
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/js/jquery.min.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
			}
			if (defined('JS_JQUERY_UI') && constant('JS_JQUERY_UI')) {
				print '<script src="'.JS_JQUERY_UI.'jquery-ui.min.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
			} else {
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/js/jquery-ui.min.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
			}
			if (!defined('DISABLE_JQUERY_TABLEDND')) {
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/tablednd/jquery.tablednd.min.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
			}
			// jQuery jnotify
			if (empty($dyap_conf->global->MAIN_DISABLE_JQUERY_JNOTIFY) && !defined('DISABLE_JQUERY_JNOTIFY')) {
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/jnotify/jquery.jnotify.min.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
			}
			// Chart
			if ((empty($dyap_conf->global->MAIN_JS_GRAPH) || $dyap_conf->global->MAIN_JS_GRAPH == 'chart') && !defined('DISABLE_JS_GRAPH')) {
				print '<script src="'.DOL_URL_ROOT.'/includes/nnnick/chartjs/dist/chart.min.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
			}

			// jQuery jeditable for Edit In Place features
			if (!empty($dyap_conf->global->MAIN_USE_JQUERY_JEDITABLE) && !defined('DISABLE_JQUERY_JEDITABLE')) {
				print '<!-- JS to manage editInPlace feature -->'."\n";
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/jeditable/jquery.jeditable.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/jeditable/jquery.jeditable.ui-datepicker.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/jeditable/jquery.jeditable.ui-autocomplete.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
				print '<script>'."\n";
				print 'var urlSaveInPlace = \''.DOL_URL_ROOT.'/core/ajax/saveinplace.php\';'."\n";
				print 'var urlLoadInPlace = \''.DOL_URL_ROOT.'/core/ajax/loadinplace.php\';'."\n";
				print 'var tooltipInPlace = \''.$dyap_langs->transnoentities('ClickToEdit').'\';'."\n"; // Added in title attribute of span
				print 'var placeholderInPlace = \'&nbsp;\';'."\n"; // If we put another string than $dyap_langs->trans("ClickToEdit") here, nothing is shown. If we put empty string, there is error, Why ?
				print 'var cancelInPlace = \''.$dyap_langs->trans("Cancel").'\';'."\n";
				print 'var submitInPlace = \''.$dyap_langs->trans('Ok').'\';'."\n";
				print 'var indicatorInPlace = \'<img src="'.DOL_URL_ROOT."/theme/".$dyap_conf->theme."/img/working.gif".'">\';'."\n";
				print 'var withInPlace = 300;'; // width in pixel for default string edit
				print '</script>'."\n";
				print '<script src="'.DOL_URL_ROOT.'/core/js/editinplace.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/jeditable/jquery.jeditable.ckeditor.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
			}
			// jQuery Timepicker
			if (!empty($dyap_conf->global->MAIN_USE_JQUERY_TIMEPICKER) || defined('REQUIRE_JQUERY_TIMEPICKER')) {
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/timepicker/jquery-ui-timepicker-addon.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
				print '<script src="'.\dyap_url('/core/js/timepicker.js.php?lang='.$dyap_langs->defaultlang.($ext ? '&amp;'.$ext : '')).'"></script>'."\n";
			}
			if (!defined('DISABLE_SELECT2') && (!empty($dyap_conf->global->MAIN_USE_JQUERY_MULTISELECT) || defined('REQUIRE_JQUERY_MULTISELECT'))) {
				// jQuery plugin "mutiselect", "multiple-select", "select2", ...
				$tmpplugin = empty($dyap_conf->global->MAIN_USE_JQUERY_MULTISELECT) ?constant('REQUIRE_JQUERY_MULTISELECT') : $dyap_conf->global->MAIN_USE_JQUERY_MULTISELECT;
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/'.$tmpplugin.'/dist/js/'.$tmpplugin.'.full.min.js'.($ext ? '?'.$ext : '').'"></script>'."\n"; // We include full because we need the support of containerCssClass
			}
			if (!defined('DISABLE_MULTISELECT')) {     // jQuery plugin "mutiselect" to select with checkboxes. Can be removed once we have an enhanced search tool
				print '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/multiselect/jquery.multi-select.js'.($ext ? '?'.$ext : '').'"></script>'."\n";
			}
		}

		if (!$disablejs && !empty($dyap_conf->use_javascript_ajax)) {
			// CKEditor
			if ((!empty($dyap_conf->fckeditor->enabled) && (empty($dyap_conf->global->FCKEDITOR_EDITORNAME) || $dyap_conf->global->FCKEDITOR_EDITORNAME == 'ckeditor') && !defined('DISABLE_CKEDITOR')) || defined('FORCE_CKEDITOR')) {
				print '<!-- Includes JS for CKEditor -->'."\n";
				$pathckeditor = DOL_URL_ROOT.'/includes/ckeditor/ckeditor/';
				$jsckeditor = 'ckeditor.js';
				if (constant('JS_CKEDITOR')) {
					// To use external ckeditor 4 js lib
					$pathckeditor = constant('JS_CKEDITOR');
				}
				print '<script>';
				print '/* enable ckeditor by main.inc.php */';
				print 'var CKEDITOR_BASEPATH = \''.dol_escape_js($pathckeditor).'\';'."\n";
				print 'var ckeditorConfig = \''.dol_escape_js(dol_buildpath($themesubdir.'/theme/'.$dyap_conf->theme.'/ckeditor/config.js'.($ext ? '?'.$ext : ''), 1)).'\';'."\n"; // $themesubdir='' in standard usage
				print 'var ckeditorFilebrowserBrowseUrl = \''.DOL_URL_ROOT.'/core/filemanagerdol/browser/default/browser.php?Connector='.DOL_URL_ROOT.'/core/filemanagerdol/connectors/php/connector.php\';'."\n";
				print 'var ckeditorFilebrowserImageBrowseUrl = \''.DOL_URL_ROOT.'/core/filemanagerdol/browser/default/browser.php?Type=Image&Connector='.DOL_URL_ROOT.'/core/filemanagerdol/connectors/php/connector.php\';'."\n";
				print '</script>'."\n";
				print '<script src="'.$pathckeditor.$jsckeditor.($ext ? '?'.$ext : '').'"></script>'."\n";
				print '<script>';
				if (GETPOST('mode', 'aZ09') == 'Full_inline') {
					print 'CKEDITOR.disableAutoInline = false;'."\n";
				} else {
					print 'CKEDITOR.disableAutoInline = true;'."\n";
				}
				print '</script>'."\n";
			}

			// Browser notifications (if NOREQUIREMENU is on, it is mostly a page for popup, so we do not enable notif too. We hide also for public pages).
			if (!defined('NOBROWSERNOTIF') && !defined('NOREQUIREMENU') && !defined('NOLOGIN')) {
				$enablebrowsernotif = false;
				if (!empty($dyap_conf->agenda->enabled) && !empty($dyap_conf->global->AGENDA_REMINDER_BROWSER)) {
					$enablebrowsernotif = true;
				}
				if ($dyap_conf->browser->layout == 'phone') {
					$enablebrowsernotif = false;
				}
				if ($enablebrowsernotif) {
					print '<!-- Includes JS of Dolibarr (browser layout = '.$dyap_conf->browser->layout.')-->'."\n";
					print '<script src="'.\dyap_url('/core/js/lib_notification.js.php').($ext ? '?'.$ext : '').'"></script>'."\n";
				}
			}

			// Global js function
			print '<!-- Includes JS of Dolibarr -->'."\n";
			print '<script src="'.\dyap_url('/core/js/lib_head.js.php?lang='.$dyap_langs->defaultlang.($ext ? '&amp;'.$ext : '')).'"></script>'."\n";

			// JS forced by modules (relative url starting with /)
			if (!empty($dyap_conf->modules_parts['js'])) {		// $dyap_conf->modules_parts['js'] is array('module'=>array('file1','file2'))
				$arrayjs = (array) $dyap_conf->modules_parts['js'];
				foreach ($arrayjs as $modjs => $filesjs) {
					$filesjs = (array) $filesjs; // To be sure filejs is an array
					foreach ($filesjs as $jsfile) {
						// jsfile is a relative path
						$urlforjs = dol_buildpath($jsfile, 1);
						if ($urlforjs && $urlforjs != '/') {
							print '<!-- Include JS added by module '.$modjs.'-->'."\n".'<script src="'.$urlforjs.((strpos($jsfile, '?') === false) ? '?' : '&amp;').'lang='.$dyap_langs->defaultlang.'"></script>'."\n";
						} else {
							dol_syslog("Warning: module ".$modjs." declared a js path file for a file we can't find.", LOG_WARNING);
						}
					}
				}
			}
			// JS forced by page in top_htmlhead (relative url starting with /)
			if (is_array($arrayofjs)) {
				print '<!-- Includes JS added by page -->'."\n";
				foreach ($arrayofjs as $jsfile) {
					if (preg_match('/^(http|\/\/)/i', $jsfile)) {
						print '<script src="'.$jsfile.((strpos($jsfile, '?') === false) ? '?' : '&amp;').'lang='.$dyap_langs->defaultlang.'"></script>'."\n";
					} else {
						print '<script src="'.dol_buildpath($jsfile, 1).((strpos($jsfile, '?') === false) ? '?' : '&amp;').'lang='.$dyap_langs->defaultlang.'"></script>'."\n";
					}
				}
			}
		}

		if (!empty($head)) {
			print $head."\n";
		}
		if (!empty($dyap_conf->global->MAIN_HTML_HEADER)) {
			print $dyap_conf->global->MAIN_HTML_HEADER."\n";
		}

		$parameters = array();
		$result = $dyap_hookmanager->executeHooks('addHtmlHeader', $parameters); // Note that $action and $object may have been modified by some hooks
		print $dyap_hookmanager->resPrint; // Replace Title to show

		print "</head>\n\n";
	}

	$dyap_conf->headerdone = 1; // To tell header was output

        print '<div id="dyapress" class="frontend">'."\n";
    } else {
        print '<div id="dyapress" class="backend">'."\n";
    }
}


/**
 *  Show an HTML header + a BODY + The top menu bar
 *
 *  @param      string	$head    			Lines in the HEAD
 *  @param      string	$title   			Title of web page
 *  @param      string	$target  			Target to use in menu links (Example: '' or '_top')
 *	@param		int		$disablejs			Do not output links to js (Ex: qd fonction utilisee par sous formulaire Ajax)
 *	@param		int		$disablehead		Do not output head section
 *	@param		array	$arrayofjs			Array of js files to add in header
 *	@param		array	$arrayofcss			Array of css files to add in header
 *  @param		string	$morequerystring	Query string to add to the link "print" to get same parameters (use only if autodetect fails)
 *  @param      string	$helppagename    	Name of wiki page for help ('' by default).
 * 				     		                Syntax is: For a wiki page: EN:EnglishPage|FR:FrenchPage|ES:SpanishPage
 * 						                    For other external page: http://server/url
 *  @return		void
 */
function top_menu($head, $title = '', $target = '', $disablejs = 0, $disablehead = 0, $arrayofjs = '', $arrayofcss = '', $morequerystring = '', $helppagename = '')
{
    global $dyap_user, $dyap_conf, $dyap_langs, $dyap_db;
    global $dolibarr_main_authentication, $dolibarr_main_demo;
    global $dyap_hookmanager, $dyap_menumanager;

    $searchform = '';

    // Instantiate hooks for external modules
    $dyap_hookmanager->initHooks(array('toprightmenu'));

    $toprightmenu = '';

    // For backward compatibility with old modules
    if (empty($dyap_conf->headerdone)) {
	$disablenofollow = 0;
	top_htmlhead($head, $title, $disablejs, $disablehead, $arrayofjs, $arrayofcss, 0, $disablenofollow);
	print '<div id="mainbody">';
    }

    /*
     * Top menu
     */
    if ((empty($dyap_conf->dol_hide_topmenu) || GETPOST('dol_invisible_topmenu', 'int')) && (!defined('NOREQUIREMENU') || !constant('NOREQUIREMENU'))) {
	$options = get_option( 'dyapress_options' );
	if ( !empty( $options['show_menutop'] ) ) {
	    if (!isset($dyap_form) || !is_object($dyap_form)) {
		include_once DOL_DOCUMENT_ROOT.'/core/class/html.form.class.php';
		$dyap_form = new Form($dyap_db);
	    }

	    print "\n".'<!-- Start top horizontal -->'."\n";

	    print '<header id="id-top" class="side-nav-vert'.(GETPOST('dol_invisible_topmenu', 'int') ? ' hidden' : '').'">'; // dol_invisible_topmenu differs from dol_hide_topmenu: dol_invisible_topmenu means we output menu but we make it invisible.

	    // Show menu entries
	    print '<div id="tmenu_tooltip'.(empty($dyap_conf->global->MAIN_MENU_INVERT) ? '' : 'invert').'" class="tmenu">'."\n";
	    $dyap_menumanager->atarget = $target;
	    $dyap_menumanager->showmenu('top', array('searchform'=>$searchform)); // This contains a \n
	    print "</div>\n";

	    // Define link to login card
	    $appli = constant('DOL_APPLICATION_TITLE');
	    if (!empty($dyap_conf->global->MAIN_APPLICATION_TITLE)) {
		$appli = $dyap_conf->global->MAIN_APPLICATION_TITLE;
		if (preg_match('/\d\.\d/', $appli)) {
		    if (!preg_match('/'.preg_quote(DOL_VERSION).'/', $appli)) {
			$appli .= " (".DOL_VERSION.")"; // If new title contains a version that is different than core
		    }
		} else {
		    $appli .= " ".DOL_VERSION;
		}
	    } else {
		$appli .= " ".DOL_VERSION;
	    }

	    if (getDolGlobalInt('MAIN_FEATURES_LEVEL')) {
		$appli .= "<br>".$dyap_langs->trans("LevelOfFeature").': '.getDolGlobalInt('MAIN_FEATURES_LEVEL');
	    }

	    $logouttext = '';
	    $logouthtmltext = '';
	    if (empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) {
		//$logouthtmltext=$appli.'<br>';
		if ($_SESSION["dol_authmode"] != 'forceuser' && $_SESSION["dol_authmode"] != 'http') {
		    $logouthtmltext .= $dyap_langs->trans("Logout").'<br>';

		    $logouttext .= '<a accesskey="l" href="'.DOL_URL_ROOT.'/user/logout.php?token='.newToken().'">';
		    $logouttext .= img_picto($dyap_langs->trans('Logout'), 'sign-out', '', false, 0, 0, '', 'atoplogin');
		    $logouttext .= '</a>';
		} else {
		    $logouthtmltext .= $dyap_langs->trans("NoLogoutProcessWithAuthMode", $_SESSION["dol_authmode"]);
		    $logouttext .= img_picto($dyap_langs->trans('Logout'), 'sign-out', '', false, 0, 0, '', 'atoplogin opacitymedium');
		}
	    }

	    print '<div class="login_block usedropdown">'."\n";

	    $toprightmenu .= '<div class="login_block_other">';

	    // Execute hook printTopRightMenu (hooks should output string like '<div class="login"><a href="">mylink</a></div>')
	    $parameters = array();
	    $result = $dyap_hookmanager->executeHooks('printTopRightMenu', $parameters); // Note that $action and $object may have been modified by some hooks
	    if (is_numeric($result)) {
		if ($result == 0) {
		    $toprightmenu .= $dyap_hookmanager->resPrint; // add
		} else {
		    $toprightmenu = $dyap_hookmanager->resPrint; // replace
		}
	    } else {
		$toprightmenu .= $result; // For backward compatibility
	    }

	    // Link to module builder
	    if (!empty($dyap_conf->modulebuilder->enabled)) {
		$text = '<a href="'.DOL_URL_ROOT.'/modulebuilder/index.php?mainmenu=home&leftmenu=admintools" target="modulebuilder">';
		//$text.= img_picto(":".$dyap_langs->trans("ModuleBuilder"), 'printer_top.png', 'class="printer"');
		$text .= '<span class="fa fa-bug atoplogin valignmiddle"></span>';
		$text .= '</a>';
		$toprightmenu .= $dyap_form->textwithtooltip('', $dyap_langs->trans("ModuleBuilder"), 2, 1, $text, 'login_block_elem', 2);
	    }

	    // Link to print main content area
	    if (empty($dyap_conf->global->MAIN_PRINT_DISABLELINK) && empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) {
		$qs = dol_escape_htmltag($_SERVER["QUERY_STRING"]);

		if (isset($_POST) && is_array($_POST)) {
		    foreach ($_POST as $key => $value) {
			if ($key !== 'action' && $key !== 'password' && !is_array($value)) {
			    $qs .= '&'.$key.'='.urlencode($value);
			}
		    }
		}
		$qs .= (($qs && $morequerystring) ? '&' : '').$morequerystring;
		$text = '<a href="'.dol_escape_htmltag($_SERVER["PHP_SELF"]).'?'.$qs.($qs ? '&' : '').'optioncss=print" target="_blank" rel="noopener noreferrer">';
		//$text.= img_picto(":".$dyap_langs->trans("PrintContentArea"), 'printer_top.png', 'class="printer"');
		$text .= '<span class="fa fa-print atoplogin valignmiddle"></span>';
		$text .= '</a>';
		$toprightmenu .= $dyap_form->textwithtooltip('', $dyap_langs->trans("PrintContentArea"), 2, 1, $text, 'login_block_elem', 2);
	    }

	    // Link to Dolibarr wiki pages
	    if (empty($dyap_conf->global->MAIN_HELP_DISABLELINK) && empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) {
		$dyap_langs->load("help");

		$helpbaseurl = '';
		$helppage = '';
		$mode = '';
		$helppresent = '';

		if (empty($helppagename)) {
		    $helppagename = 'EN:User_documentation|FR:Documentation_utilisateur|ES:Documentación_usuarios';
		} else {
		    $helppresent = 'helppresent';
		}

		// Get helpbaseurl, helppage and mode from helppagename and langs
		$arrayres = getHelpParamFor($helppagename, $dyap_langs);
		$helpbaseurl = $arrayres['helpbaseurl'];
		$helppage = $arrayres['helppage'];
		$mode = $arrayres['mode'];

		// Link to help pages
		if ($helpbaseurl && $helppage) {
		    $text = '';
		    $title = $dyap_langs->trans($mode == 'wiki' ? 'GoToWikiHelpPage' : 'GoToHelpPage').', ';
		    if ($mode == 'wiki') {
			$title .= '<br>'.img_picto('', 'globe', 'class="pictofixedwidth"').$dyap_langs->trans("PageWiki").' '.dol_escape_htmltag('"'.strtr($helppage, '_', ' ').'"');
			if ($helppresent) {
			    $title .= ' <span class="opacitymedium">('.$dyap_langs->trans("DedicatedPageAvailable").')</span>';
			} else {
			    $title .= ' <span class="opacitymedium">('.$dyap_langs->trans("HomePage").')</span>';
			}
		    }
		    $text .= '<a class="help" target="_blank" rel="noopener noreferrer" href="';
		    if ($mode == 'wiki') {
			$text .= sprintf($helpbaseurl, urlencode(html_entity_decode($helppage)));
		    } else {
			$text .= sprintf($helpbaseurl, $helppage);
		    }
		    $text .= '">';
		    $text .= '<span class="fa fa-question-circle atoplogin valignmiddle'.($helppresent ? ' '.$helppresent : '').'"></span>';
		    $text .= '<span class="fa fa-long-arrow-alt-up helppresentcircle'.($helppresent ? '' : ' unvisible').'"></span>';
		    $text .= '</a>';
		    $toprightmenu .= $dyap_form->textwithtooltip('', $title, 2, 1, $text, 'login_block_elem', 2);
		}

		// Version
		if (!empty($dyap_conf->global->MAIN_SHOWDATABASENAMEINHELPPAGESLINK)) {
		    $dyap_langs->load('admin');
		    $appli .= '<br>'.$dyap_langs->trans("Database").': '.$dyap_db->database_name;
		}
	    }

	    if (empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) {
		$text = '<span class="aversion"><span class="hideonsmartphone small">'.DOL_VERSION.'</span></span>';
		$toprightmenu .= $dyap_form->textwithtooltip('', $appli, 2, 1, $text, 'login_block_elem', 2);
	    }

	    // Logout link
	    $toprightmenu .= $dyap_form->textwithtooltip('', $logouthtmltext, 2, 1, $logouttext, 'login_block_elem logout-btn', 2);

	    $toprightmenu .= '</div>'; // end div class="login_block_other"


	    // Add login user link
	    $toprightmenu .= '<div class="login_block_user">';

	    // Login name with photo and tooltip
	    $mode = -1;
	    $toprightmenu .= '<div class="inline-block nowrap"><div class="inline-block login_block_elem login_block_elem_name" style="padding: 0px;">';

	    if (!empty($dyap_conf->global->MAIN_USE_TOP_MENU_SEARCH_DROPDOWN)) {
		// Add search dropdown
		$toprightmenu .= top_menu_search();
	    }

	    if (!empty($dyap_conf->global->MAIN_USE_TOP_MENU_QUICKADD_DROPDOWN)) {
		// Add search dropdown
		$toprightmenu .= top_menu_quickadd();
	    }

	    // Add bookmark dropdown
	    $toprightmenu .= top_menu_bookmark();

	    // Add user dropdown
	    $toprightmenu .= top_menu_user();

	    $toprightmenu .= '</div></div>';

	    $toprightmenu .= '</div>'."\n";


	    print $toprightmenu;

	    print "</div>\n"; // end div class="login_block"

	    print '</header>';

	    print '<div style="clear: both;"></div>';
	    print "<!-- End top horizontal menu -->\n\n";
	}
    }

    if (empty($dyap_conf->dol_hide_leftmenu) && empty($dyap_conf->dol_use_jmobile)) {
	print '<!-- Begin div id-container --><div id="id-container" class="id-container">';
    }
}

/**
 * Build the tooltip on user login
 *
 * @param	int			$hideloginname		Hide login name. Show only the image.
 * @param	string		$urllogout			URL for logout (Will use DOL_URL_ROOT.'/user/logout.php?token=...' if empty)
 * @return  string                  		HTML content
 */
function top_menu_user($hideloginname = 0, $urllogout = '')
{
	global $dyap_langs, $dyap_conf, $dyap_db, $dyap_hookmanager, $dyap_user, $dyap_mysoc;
	global $dolibarr_main_authentication, $dolibarr_main_demo;
	global $dyap_menumanager;

	$dyap_langs->load('companies');

	$userImage = $userDropDownImage = '';
	if (!empty($dyap_user->photo)) {
		$userImage          = Form::showphoto('userphoto', $dyap_user, 0, 0, 0, 'photouserphoto userphoto', 'small', 0, 1);
		$userDropDownImage  = Form::showphoto('userphoto', $dyap_user, 0, 0, 0, 'dropdown-user-image', 'small', 0, 1);
	} else {
		$nophoto = '/public/theme/common/user_anonymous.png';
		if ($dyap_user->gender == 'man') {
			$nophoto = '/public/theme/common/user_man.png';
		}
		if ($dyap_user->gender == 'woman') {
			$nophoto = '/public/theme/common/user_woman.png';
		}

		$userImage = '<img class="photo photouserphoto userphoto" alt="No photo" src="'.DOL_URL_ROOT.$nophoto.'">';
		$userDropDownImage = '<img class="photo dropdown-user-image" alt="No photo" src="'.DOL_URL_ROOT.$nophoto.'">';
	}

	$dropdownBody = '';
	$dropdownBody .= '<span id="topmenulogincompanyinfo-btn"><i class="fa fa-caret-right"></i> '.$dyap_langs->trans("ShowCompanyInfos").'</span>';
	$dropdownBody .= '<div id="topmenulogincompanyinfo" >';

	if ($dyap_langs->transcountry("ProfId1", $dyap_mysoc->country_code) != '-') {
		$dropdownBody .= '<br><b>'.$dyap_langs->transcountry("ProfId1", $dyap_mysoc->country_code).'</b>: <span>'.showValueWithClipboardCPButton(getDolGlobalString("MAIN_INFO_SIREN")).'</span>';
	}
	if ($dyap_langs->transcountry("ProfId2", $dyap_mysoc->country_code) != '-') {
		$dropdownBody .= '<br><b>'.$dyap_langs->transcountry("ProfId2", $dyap_mysoc->country_code).'</b>: <span>'.showValueWithClipboardCPButton(getDolGlobalString("MAIN_INFO_SIRET")).'</span>';
	}
	if ($dyap_langs->transcountry("ProfId3", $dyap_mysoc->country_code) != '-') {
		$dropdownBody .= '<br><b>'.$dyap_langs->transcountry("ProfId3", $dyap_mysoc->country_code).'</b>: <span>'.showValueWithClipboardCPButton(getDolGlobalString("MAIN_INFO_APE")).'</span>';
	}
	if ($dyap_langs->transcountry("ProfId4", $dyap_mysoc->country_code) != '-') {
		$dropdownBody .= '<br><b>'.$dyap_langs->transcountry("ProfId4", $dyap_mysoc->country_code).'</b>: <span>'.showValueWithClipboardCPButton(getDolGlobalString("MAIN_INFO_RCS")).'</span>';
	}
	if ($dyap_langs->transcountry("ProfId5", $dyap_mysoc->country_code) != '-') {
		$dropdownBody .= '<br><b>'.$dyap_langs->transcountry("ProfId5", $dyap_mysoc->country_code).'</b>: <span>'.showValueWithClipboardCPButton(getDolGlobalString("MAIN_INFO_PROFID5")).'</span>';
	}
	if ($dyap_langs->transcountry("ProfId6", $dyap_mysoc->country_code) != '-') {
		$dropdownBody .= '<br><b>'.$dyap_langs->transcountry("ProfId6", $dyap_mysoc->country_code).'</b>: <span>'.showValueWithClipboardCPButton(getDolGlobalString("MAIN_INFO_PROFID6")).'</span>';
	}
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("VATIntraShort").'</b>: <span>'.showValueWithClipboardCPButton(getDolGlobalString("MAIN_INFO_TVAINTRA")).'</span>';

	$dropdownBody .= '</div>';

	$dropdownBody .= '<br>';
	$dropdownBody .= '<span id="topmenuloginmoreinfo-btn"><i class="fa fa-caret-right"></i> '.$dyap_langs->trans("ShowMoreInfos").'</span>';
	$dropdownBody .= '<div id="topmenuloginmoreinfo" >';

	// login infos
	if (!empty($dyap_user->admin)) {
		$dropdownBody .= '<br><b>'.$dyap_langs->trans("Administrator").'</b>: '.yn($dyap_user->admin);
	}
	if (!empty($dyap_user->socid)) {	// Add thirdparty for external users
		$thirdpartystatic = new Societe($dyap_db);
		$thirdpartystatic->fetch($dyap_user->socid);
		$companylink = ' '.$thirdpartystatic->getNomUrl(2); // picto only of company
		$company = ' ('.$dyap_langs->trans("Company").': '.$thirdpartystatic->name.')';
	}
	$type = ($dyap_user->socid ? $dyap_langs->trans("External").$company : $dyap_langs->trans("Internal"));
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("Type").':</b> '.$type;
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("Status").'</b>: '.$dyap_user->getLibStatut(0);
	$dropdownBody .= '<br>';

	$dropdownBody .= '<br><u>'.$dyap_langs->trans("Session").'</u>';
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("IPAddress").'</b>: '.dol_escape_htmltag($_SERVER["REMOTE_ADDR"]);
	if (!empty($dyap_conf->global->MAIN_MODULE_MULTICOMPANY)) {
		$dropdownBody .= '<br><b>'.$dyap_langs->trans("ConnectedOnMultiCompany").':</b> '.$dyap_conf->entity.' (user entity '.$dyap_user->entity.')';
	}
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("AuthenticationMode").':</b> '.$_SESSION["dol_authmode"].(empty($dolibarr_main_demo) ? '' : ' (demo)');
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("ConnectedSince").':</b> '.dol_print_date($dyap_user->datelastlogin, "dayhour", 'tzuser');
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("PreviousConnexion").':</b> '.dol_print_date($dyap_user->datepreviouslogin, "dayhour", 'tzuser');
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("CurrentTheme").':</b> '.$dyap_conf->theme;
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("CurrentMenuManager").':</b> '.(isset($dyap_menumanager) ? $dyap_menumanager->name : 'unknown');
	$langFlag = picto_from_langcode($dyap_langs->getDefaultLang());
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("CurrentUserLanguage").':</b> '.($langFlag ? $langFlag.' ' : '').$dyap_langs->getDefaultLang();

	$tz = (int) $_SESSION['dol_tz'] + (int) $_SESSION['dol_dst'];
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("ClientTZ").':</b> '.($tz ? ($tz >= 0 ? '+' : '').$tz : '');
	$dropdownBody .= ' ('.$_SESSION['dol_tz_string'].')';
	//$dropdownBody .= ' &nbsp; &nbsp; &nbsp; '.$dyap_langs->trans("DaylingSavingTime").': ';
	//if ($_SESSION['dol_dst'] > 0) $dropdownBody .= yn(1);
	//else $dropdownBody .= yn(0);

	$dropdownBody .= '<br><b>'.$dyap_langs->trans("Browser").':</b> '.$dyap_conf->browser->name.($dyap_conf->browser->version ? ' '.$dyap_conf->browser->version : '').' ('.dol_escape_htmltag($_SERVER['HTTP_USER_AGENT']).')';
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("Layout").':</b> '.$dyap_conf->browser->layout;
	$dropdownBody .= '<br><b>'.$dyap_langs->trans("Screen").':</b> '.$_SESSION['dol_screenwidth'].' x '.$_SESSION['dol_screenheight'];
	if ($dyap_conf->browser->layout == 'phone') {
		$dropdownBody .= '<br><b>'.$dyap_langs->trans("Phone").':</b> '.$dyap_langs->trans("Yes");
	}
	if (!empty($_SESSION["disablemodules"])) {
		$dropdownBody .= '<br><b>'.$dyap_langs->trans("DisabledModules").':</b> <br>'.join(', ', explode(',', $_SESSION["disablemodules"]));
	}
	$dropdownBody .= '</div>';

	// Execute hook
	$parameters = array('user'=>$dyap_user, 'langs' => $dyap_langs);
	$result = $dyap_hookmanager->executeHooks('printTopRightMenuLoginDropdownBody', $parameters); // Note that $action and $object may have been modified by some hooks
	if (is_numeric($result)) {
		if ($result == 0) {
			$dropdownBody .= $dyap_hookmanager->resPrint; // add
		} else {
			$dropdownBody = $dyap_hookmanager->resPrint; // replace
		}
	}

	if (empty($urllogout)) {
		$urllogout = DOL_URL_ROOT.'/user/logout.php?token='.newToken();
	}
	$logoutLink = '<a accesskey="l" href="'.$urllogout.'" class="button-top-menu-dropdown" ><i class="fa fa-sign-out-alt"></i> '.$dyap_langs->trans("Logout").'</a>';
	$profilLink = '<a accesskey="l" href="'.DOL_URL_ROOT.'/user/card.php?id='.$dyap_user->id.'" class="button-top-menu-dropdown" ><i class="fa fa-user"></i>  '.$dyap_langs->trans("Card").'</a>';


	$profilName = $dyap_user->getFullName($dyap_langs).' ('.$dyap_user->login.')';

	if (!empty($dyap_user->admin)) {
		$profilName = '<i class="far fa-star classfortooltip" title="'.$dyap_langs->trans("Administrator").'" ></i> '.$profilName;
	}

	// Define version to show
	$appli = constant('DOL_APPLICATION_TITLE');
	if (!empty($dyap_conf->global->MAIN_APPLICATION_TITLE)) {
		$appli = $dyap_conf->global->MAIN_APPLICATION_TITLE;
		if (preg_match('/\d\.\d/', $appli)) {
			if (!preg_match('/'.preg_quote(DOL_VERSION).'/', $appli)) {
				$appli .= " (".DOL_VERSION.")"; // If new title contains a version that is different than core
			}
		} else {
			$appli .= " ".DOL_VERSION;
		}
	} else {
		$appli .= " ".DOL_VERSION;
	}

	if (empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) {
		$btnUser = '<!-- div for user link -->
	    <div id="topmenu-login-dropdown" class="userimg atoplogin dropdown user user-menu inline-block">
	        <a href="'.DOL_URL_ROOT.'/user/card.php?id='.$dyap_user->id.'" class="dropdown-toggle login-dropdown-a" data-toggle="dropdown">
	            '.$userImage.'<span class="hidden-xs maxwidth200 atoploginusername hideonsmartphone paddingleft">'.dol_trunc($dyap_user->firstname ? $dyap_user->firstname : $dyap_user->login, 10).'</span>
	        </a>
	        <div class="dropdown-menu">
	            <!-- User image -->
	            <div class="user-header">
	                '.$userDropDownImage.'
	                <p>
	                    '.$profilName.'<br>';
		if ($dyap_user->datelastlogin) {
			$title = $dyap_langs->trans("ConnectedSince").' : '.dol_print_date($dyap_user->datelastlogin, "dayhour", 'tzuser');
			if ($dyap_user->datepreviouslogin) {
				$title .= '<br>'.$dyap_langs->trans("PreviousConnexion").' : '.dol_print_date($dyap_user->datepreviouslogin, "dayhour", 'tzuser');
			}
		}
		$btnUser .= '<small class="classfortooltip" title="'.dol_escape_htmltag($title).'" ><i class="fa fa-user-clock"></i> '.dol_print_date($dyap_user->datelastlogin, "dayhour", 'tzuser').'</small><br>';
		if ($dyap_user->datepreviouslogin) {
			$btnUser .= '<small class="classfortooltip" title="'.dol_escape_htmltag($title).'" ><i class="fa fa-user-clock opacitymedium"></i> '.dol_print_date($dyap_user->datepreviouslogin, "dayhour", 'tzuser').'</small><br>';
		}

		//$btnUser .= '<small class="classfortooltip"><i class="fa fa-cog"></i> '.$dyap_langs->trans("Version").' '.$appli.'</small>';
		$btnUser .= '
	                </p>
	            </div>

	            <!-- Menu Body -->
	            <div class="user-body">'.$dropdownBody.'</div>

	            <!-- Menu Footer-->
	            <div class="user-footer">
	                <div class="pull-left">
	                    '.$profilLink.'
	                </div>
	                <div class="pull-right">
	                    '.$logoutLink.'
	                </div>
	                <div style="clear:both;"></div>
	            </div>

	        </div>
	    </div>';
	} else {
		$btnUser = '<!-- div for user link -->
	    <div id="topmenu-login-dropdown" class="userimg atoplogin dropdown user user-menu  inline-block">
	    	<a href="'.DOL_URL_ROOT.'/user/card.php?id='.$dyap_user->id.'">
	    	'.$userImage.'
	    		<span class="hidden-xs maxwidth200 atoploginusername hideonsmartphone">'.dol_trunc($dyap_user->firstname ? $dyap_user->firstname : $dyap_user->login, 10).'</span>
	    		</a>
		</div>';
	}

	if (!defined('JS_JQUERY_DISABLE_DROPDOWN') && !empty($dyap_conf->use_javascript_ajax)) {    // This may be set by some pages that use different jquery version to avoid errors
		$btnUser .= '
        <!-- Code to show/hide the user drop-down -->
        <script>
        jQuery(document).ready(function($) {
            $(document).on("click", function(event) {
                if (!$(event.target).closest("#topmenu-login-dropdown").length) {
					//console.log("close login dropdown");
					// Hide the menus.
                    $("#topmenu-login-dropdown").removeClass("open");
                }
            });
			';

		if ($dyap_conf->theme != 'md') {
			$btnUser .= '
	            $("#topmenu-login-dropdown .dropdown-toggle").on("click", function(event) {
					console.log("toggle login dropdown");
					event.preventDefault();
	                $("#topmenu-login-dropdown").toggleClass("open");
	            });

	            $("#topmenulogincompanyinfo-btn").on("click", function() {
	                $("#topmenulogincompanyinfo").slideToggle();
	            });

	            $("#topmenuloginmoreinfo-btn").on("click", function() {
	                $("#topmenuloginmoreinfo").slideToggle();
	            });';
		}

		$btnUser .= '
        });
        </script>
        ';
	}

	return $btnUser;
}

/**
 * Build the tooltip on top menu quick add
 *
 * @return  string                  HTML content
 */
function top_menu_quickadd()
{
	global $dyap_langs, $dyap_conf, $dyap_db, $dyap_hookmanager, $dyap_user;
	global $dyap_menumanager;

	$html = '';
	// Define $dropDownQuickAddHtml
	$dropDownQuickAddHtml = '<div class="dropdown-header bookmark-header center">';
	$dropDownQuickAddHtml .= $dyap_langs->trans('QuickAdd');
	$dropDownQuickAddHtml .= '</div>';

	$dropDownQuickAddHtml .= '<div class="quickadd-body dropdown-body">';
	$dropDownQuickAddHtml .= '<div class="quickadd">';
	if (!empty($dyap_conf->societe->enabled) && $dyap_user->rights->societe->creer) {
		$dyap_langs->load("companies");
		$dropDownQuickAddHtml .= '
                <!-- Thirdparty link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/societe/card.php?action=create" title="'.$dyap_langs->trans("MenuNewThirdParty").'">
                    '. img_picto('', 'object_company').'<br>'.$dyap_langs->trans("ThirdParty").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->societe->enabled) && $dyap_user->rights->societe->contact->creer) {
		$dyap_langs->load("companies");
		$dropDownQuickAddHtml .= '
                <!-- Contact link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/contact/card.php?action=create" title="'.$dyap_langs->trans("NewContactAddress").'">
                    '. img_picto('', 'object_contact').'<br>'.$dyap_langs->trans("Contact").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->propal->enabled) && $dyap_user->rights->propale->creer) {
		$dyap_langs->load("propal");
		$dropDownQuickAddHtml .= '
                <!-- Propal link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/comm/propal/card.php?action=create" title="'.$dyap_langs->trans("NewPropal").'">
                    '. img_picto('', 'object_propal').'<br>'.$dyap_langs->trans("Proposal").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->commande->enabled) && $dyap_user->rights->commande->creer) {
		$dyap_langs->load("orders");
		$dropDownQuickAddHtml .= '
                <!-- Order link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/commande/card.php?action=create" title="'.$dyap_langs->trans("NewOrder").'">
                    '. img_picto('', 'object_order').'<br>'.$dyap_langs->trans("Order").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->facture->enabled) && $dyap_user->rights->facture->creer) {
		$dyap_langs->load("bills");
		$dropDownQuickAddHtml .= '
                <!-- Invoice link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/compta/facture/card.php?action=create" title="'.$dyap_langs->trans("NewBill").'">
                    '. img_picto('', 'object_bill').'<br>'.$dyap_langs->trans("Bill").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->contrat->enabled) && $dyap_user->rights->contrat->creer) {
		$dyap_langs->load("contracts");
		$dropDownQuickAddHtml .= '
                <!-- Contract link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/compta/facture/card.php?action=create" title="'.$dyap_langs->trans("NewContractSubscription").'">
                    '. img_picto('', 'object_contract').'<br>'.$dyap_langs->trans("Contract").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->supplier_proposal->enabled) && $dyap_user->rights->supplier_proposal->creer) {
		$dyap_langs->load("supplier_proposal");
		$dropDownQuickAddHtml .= '
                <!-- Supplier proposal link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/supplier_proposal/card.php?action=create" title="'.$dyap_langs->trans("NewAskPrice").'">
                    '. img_picto('', 'object_propal').'<br>'.$dyap_langs->trans("AskPrice").'</a>
                </div>
                ';
	}

	if ((!empty($dyap_conf->fournisseur->enabled) && empty($dyap_conf->global->MAIN_USE_NEW_SUPPLIERMOD) && $dyap_user->rights->fournisseur->commande->creer) || (!empty($dyap_conf->supplier_order->enabled) && $dyap_user->rights->supplier_order->creer)) {
		$dyap_langs->load("orders");
		$dropDownQuickAddHtml .= '
                <!-- Supplier order link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/fourn/commande/card.php?action=create" title="'.$dyap_langs->trans("NewSupplierOrderShort").'">
                    '. img_picto('', 'object_order').'<br>'.$dyap_langs->trans("SupplierOrder").'</a>
                </div>
                ';
	}

	if ((!empty($dyap_conf->fournisseur->enabled) && empty($dyap_conf->global->MAIN_USE_NEW_SUPPLIERMOD) && $dyap_user->rights->fournisseur->facture->creer) || (!empty($dyap_conf->supplier_invoice->enabled) && $dyap_user->rights->supplier_invoice->creer)) {
		$dyap_langs->load("bills");
		$dropDownQuickAddHtml .= '
                <!-- Supplier invoice link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/fourn/facture/card.php?action=create" title="'.$dyap_langs->trans("NewBill").'">
                    '. img_picto('', 'object_bill').'<br>'.$dyap_langs->trans("SupplierBill").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->product->enabled) && $dyap_user->rights->produit->creer) {
		$dyap_langs->load("products");
		$dropDownQuickAddHtml .= '
                <!-- Product link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/product/card.php?action=create&amp;type=0" title="'.$dyap_langs->trans("NewProduct").'">
                    '. img_picto('', 'object_product').'<br>'.$dyap_langs->trans("Product").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->service->enabled) && $dyap_user->rights->service->creer) {
		$dyap_langs->load("products");
		$dropDownQuickAddHtml .= '
                <!-- Service link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/product/card.php?action=create&amp;type=1" title="'.$dyap_langs->trans("NewService").'">
                    '. img_picto('', 'object_service').'<br>'.$dyap_langs->trans("Service").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->expensereport->enabled) && $dyap_user->rights->expensereport->creer) {
		$dyap_langs->load("trips");
		$dropDownQuickAddHtml .= '
                <!-- Expense report link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/expensereport/card.php?action=create&fk_user_author='.$dyap_user->id.'" title="'.$dyap_langs->trans("AddTrip").'">
                    '. img_picto('', 'object_trip').'<br>'.$dyap_langs->trans("ExpenseReport").'</a>
                </div>
                ';
	}

	if (!empty($dyap_conf->holiday->enabled) && $dyap_user->rights->holiday->write) {
		$dyap_langs->load("holiday");
		$dropDownQuickAddHtml .= '
                <!-- Holiday link -->
                <div class="quickaddblock center">
                    <a class="quickadddropdown-icon-link" href="'.DOL_URL_ROOT.'/holiday/card.php?action=create&fuserid='.$dyap_user->id.'" title="'.$dyap_langs->trans("AddCP").'">
                    '. img_picto('', 'object_holiday').'<br>'.$dyap_langs->trans("Holidays").'</a>
                </div>
                ';
	}

	// Execute hook printTopRightMenu (hooks should output string like '<div class="login"><a href="">mylink</a></div>')
	$parameters = array();
	$result = $dyap_hookmanager->executeHooks('printQuickAddBlock', $parameters); // Note that $action and $object may have been modified by some hooks
	if (is_numeric($result)) {
		if ($result == 0) {
			$dropDownQuickAddHtml .= $dyap_hookmanager->resPrint; // add
		} else {
			$dropDownQuickAddHtml = $dyap_hookmanager->resPrint; // replace
		}
	} else {
		$dropDownQuickAddHtml .= $result; // For backward compatibility
	}

	$dropDownQuickAddHtml .= '</div>';
	$dropDownQuickAddHtml .= '</div>';

	$html .= '<!-- div for quick add link -->
    <div id="topmenu-quickadd-dropdown" class="atoplogin dropdown inline-block">
        <a class="dropdown-toggle login-dropdown-a" data-toggle="dropdown" href="#" title="'.$dyap_langs->trans('QuickAdd').' ('.$dyap_langs->trans('QuickAddMenuShortCut').')">
            <i class="fa fa-plus-circle" ></i>
        </a>

        <div class="dropdown-menu">
            '.$dropDownQuickAddHtml.'
        </div>
    </div>';
	$html .= '
        <!-- Code to show/hide the user drop-down -->
        <script>
        jQuery(document).ready(function($) {
            $(document).on("click", function(event) {
                if (!$(event.target).closest("#topmenu-quickadd-dropdown").length) {
                    // Hide the menus.
                    $("#topmenu-quickadd-dropdown").removeClass("open");
                }
            });
            $("#topmenu-quickadd-dropdown .dropdown-toggle").on("click", function(event) {
                openQuickAddDropDown();
            });
            // Key map shortcut
            $(document).keydown(function(e){
                  if( e.which === 76 && e.ctrlKey && e.shiftKey ){
                     console.log(\'control + shift + l : trigger open quick add dropdown\');
                     openQuickAddDropDown();
                  }
            });


            var openQuickAddDropDown = function() {
                event.preventDefault();
                $("#topmenu-quickadd-dropdown").toggleClass("open");
                //$("#top-quickadd-search-input").focus();
            }
        });
        </script>
        ';
	return $html;
}

/**
 * Build the tooltip on top menu bookmark
 *
 * @return  string                  HTML content
 */
function top_menu_bookmark()
{
	global $dyap_langs, $dyap_conf, $dyap_db, $dyap_user;

	$html = '';

	// Define $bookmarks
	if (empty($dyap_conf->bookmark->enabled) || empty($dyap_user->rights->bookmark->lire)) {
		return $html;
	}

	if (!defined('JS_JQUERY_DISABLE_DROPDOWN') && !empty($dyap_conf->use_javascript_ajax)) {	    // This may be set by some pages that use different jquery version to avoid errors
		include_once DOL_DOCUMENT_ROOT.'/bookmarks/bookmarks.lib.php';
		$dyap_langs->load("bookmarks");

		if (!empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) {
			$html .= '<div id="topmenu-bookmark-dropdown" class="dropdown inline-block">';
			$html .= printDropdownBookmarksList();
			$html .= '</div>';
		} else {
			$html .= '<!-- div for bookmark link -->
	        <div id="topmenu-bookmark-dropdown" class="dropdown inline-block">
	            <a class="dropdown-toggle login-dropdown-a" data-toggle="dropdown" href="#" title="'.$dyap_langs->trans('Bookmarks').' ('.$dyap_langs->trans('BookmarksMenuShortCut').')">
	                <i class="fa fa-star" ></i>
	            </a>
	            <div class="dropdown-menu">
	                '.printDropdownBookmarksList().'
	            </div>
	        </div>';

			$html .= '
	        <!-- Code to show/hide the bookmark drop-down -->
	        <script>
	        jQuery(document).ready(function($) {
	            $(document).on("click", function(event) {
	                if (!$(event.target).closest("#topmenu-bookmark-dropdown").length) {
						//console.log("close bookmark dropdown - we click outside");
	                    // Hide the menus.
	                    $("#topmenu-bookmark-dropdown").removeClass("open");
	                }
	            });

	            $("#topmenu-bookmark-dropdown .dropdown-toggle").on("click", function(event) {
					console.log("toggle bookmark dropdown");
					openBookMarkDropDown();
	            });

	            // Key map shortcut
	            $(document).keydown(function(e){
	                  if( e.which === 77 && e.ctrlKey && e.shiftKey ){
	                     console.log(\'control + shift + m : trigger open bookmark dropdown\');
	                     openBookMarkDropDown();
	                  }
	            });


	            var openBookMarkDropDown = function() {
	                event.preventDefault();
	                $("#topmenu-bookmark-dropdown").toggleClass("open");
	                $("#top-bookmark-search-input").focus();
	            }

	        });
	        </script>
	        ';
		}
	}
	return $html;
}

/**
 * Build the tooltip on top menu tsearch
 *
 * @return  string                  HTML content
 */
function top_menu_search()
{
	global $dyap_langs, $dyap_conf, $dyap_db, $dyap_user;

	$html = '';

	global $dyap_arrayresult, $dyap_usedbyinclude;
	$dyap_usedbyinclude = 1;
	$dyap_arrayresult = null;
	include DOL_DOCUMENT_ROOT.'/core/ajax/selectsearchbox.php'; // This set $dyap_arrayresult

	$defaultAction = '';
	$buttonList = '<div class="dropdown-global-search-button-list" >';
	// Menu with all searchable items
	foreach ($dyap_arrayresult as $keyItem => $item) {
		if (empty($defaultAction)) {
			$defaultAction = $item['url'];
		}
		$buttonList .= '<button class="dropdown-item global-search-item" data-target="'.dol_escape_htmltag($item['url']).'" >';
		$buttonList .= $item['text'];
		$buttonList .= '</button>';
	}
	$buttonList .= '</div>';


	$searchInput = '<input name="sall" id="top-global-search-input" class="dropdown-search-input" placeholder="'.$dyap_langs->trans('Search').'" autocomplete="off" >';

	$dropDownHtml = '<form id="top-menu-action-search" name="actionsearch" method="GET" action="'.$defaultAction.'" >';

	$dropDownHtml .= '
        <!-- search input -->
        <div class="dropdown-header search-dropdown-header">
            ' . $searchInput.'
        </div>
    ';

	$dropDownHtml .= '
        <!-- Menu Body -->
        <div class="dropdown-body search-dropdown-body">
        '.$buttonList.'
        </div>
        ';

	$dropDownHtml .= '</form>';


	$html .= '<!-- div for Global Search -->
    <div id="topmenu-global-search-dropdown" class="atoplogin dropdown inline-block">
        <a class="dropdown-toggle login-dropdown-a" data-toggle="dropdown" href="#" title="'.$dyap_langs->trans('Search').' ('.$dyap_langs->trans('SearchMenuShortCut').')">
            <i class="fa fa-search" ></i>
        </a>
        <div class="dropdown-menu dropdown-search">
            '.$dropDownHtml.'
        </div>
    </div>';

	$html .= '
    <!-- Code to show/hide the user drop-down -->
    <script>
    jQuery(document).ready(function($) {

        // prevent submiting form on press ENTER
        $("#top-global-search-input").keydown(function (e) {
            if (e.keyCode == 13) {
                var inputs = $(this).parents("form").eq(0).find(":button");
                if (inputs[inputs.index(this) + 1] != null) {
                    inputs[inputs.index(this) + 1].focus();
                }
                e.preventDefault();
                return false;
            }
        });

        // arrow key nav
        $(document).keydown(function(e) {
			// Get the focused element:
			var $focused = $(":focus");
			if($focused.length && $focused.hasClass("global-search-item")){

           		// UP - move to the previous line
				if (e.keyCode == 38) {
				    e.preventDefault();
					$focused.prev().focus();
				}

				// DOWN - move to the next line
				if (e.keyCode == 40) {
				    e.preventDefault();
					$focused.next().focus();
				}
			}
        });


        // submit form action
        $(".dropdown-global-search-button-list .global-search-item").on("click", function(event) {
            $("#top-menu-action-search").attr("action", $(this).data("target"));
            $("#top-menu-action-search").submit();
        });

        // close drop down
        $(document).on("click", function(event) {
			if (!$(event.target).closest("#topmenu-global-search-dropdown").length) {
				console.log("click close search - we click outside");
                // Hide the menus.
                $("#topmenu-global-search-dropdown").removeClass("open");
            }
        });

        // Open drop down
        $("#topmenu-global-search-dropdown .dropdown-toggle").on("click", function(event) {
			console.log("toggle search dropdown");
            openGlobalSearchDropDown();
        });

        // Key map shortcut
        $(document).keydown(function(e){
              if( e.which === 70 && e.ctrlKey && e.shiftKey ){
                 console.log(\'control + shift + f : trigger open global-search dropdown\');
                 openGlobalSearchDropDown();
              }
        });


        var openGlobalSearchDropDown = function() {
            $("#topmenu-global-search-dropdown").toggleClass("open");
            $("#top-global-search-input").focus();
        }

    });
    </script>
    ';

	return $html;
}


/**
 *  Show left menu bar
 *
 *  @param  array	$menu_array_before 	       	Table of menu entries to show before entries of menu handler. This param is deprectaed and must be provided to ''.
 *  @param  string	$helppagename    	       	Name of wiki page for help ('' by default).
 * 				     		                   	Syntax is: For a wiki page: EN:EnglishPage|FR:FrenchPage|ES:SpanishPage
 * 									         	For other external page: http://server/url
 *  @param  string	$notused             		Deprecated. Used in past to add content into left menu. Hooks can be used now.
 *  @param  array	$menu_array_after           Table of menu entries to show after entries of menu handler
 *  @param  int		$leftmenuwithoutmainarea    Must be set to 1. 0 by default for backward compatibility with old modules.
 *  @param  string	$title                      Title of web page
 *  @param  string  $acceptdelayedhtml          1 if caller request to have html delayed content not returned but saved into global $delayedhtmlcontent (so caller can show it at end of page to avoid flash FOUC effect)
 *  @return	void
 */
function left_menu($menu_array_before, $helppagename = '', $notused = '', $menu_array_after = '', $leftmenuwithoutmainarea = 0, $title = '', $acceptdelayedhtml = 0)
{
	global $dyap_user, $dyap_conf, $dyap_langs, $dyap_db, $dyap_form;
	global $dyap_hookmanager, $dyap_menumanager;

	$searchform = '';

	if (!empty($menu_array_before)) {
		dol_syslog("Deprecated parameter menu_array_before was used when calling main::left_menu function. Menu entries of module should now be defined into module descriptor and not provided when calling left_menu.", LOG_WARNING);
	}

	if (empty($dyap_conf->dol_hide_leftmenu) && (!defined('NOREQUIREMENU') || !constant('NOREQUIREMENU'))) {
		// Instantiate hooks for external modules
		$dyap_hookmanager->initHooks(array('searchform', 'leftblock'));

		print "\n".'<!-- Begin side-nav id-left -->'."\n".'<div class="side-nav"><div id="id-left">'."\n";

		if ($dyap_conf->browser->layout == 'phone') {
			$dyap_conf->global->MAIN_USE_OLD_SEARCH_FORM = 1; // Select into select2 is awfull on smartphone. TODO Is this still true with select2 v4 ?
		}

		print "\n";

		if (!is_object($dyap_form)) {
			$dyap_form = new Form($dyap_db);
		}
		$selected = -1;
		if (empty($dyap_conf->global->MAIN_USE_TOP_MENU_SEARCH_DROPDOWN)) {
			global $dyap_arrayresult, $dyap_usedbyinclude;

			$dyap_usedbyinclude = 1;
			$dyap_arrayresult = null;
			include DOL_DOCUMENT_ROOT.'/core/ajax/selectsearchbox.php'; // This set $dyap_arrayresult

			if ($dyap_conf->use_javascript_ajax && empty($dyap_conf->global->MAIN_USE_OLD_SEARCH_FORM)) {
				$searchform .= $dyap_form->selectArrayFilter('searchselectcombo', $dyap_arrayresult, $selected, '', 1, 0, (empty($dyap_conf->global->MAIN_SEARCHBOX_CONTENT_LOADED_BEFORE_KEY) ? 1 : 0), 'vmenusearchselectcombo', 1, $dyap_langs->trans("Search"), 1);
			} else {
				if (is_array($dyap_arrayresult)) {
					foreach ($dyap_arrayresult as $key => $val) {
						$searchform .= printSearchForm($val['url'], $val['url'], $val['label'], 'maxwidth125', 'sall', $val['shortcut'], 'searchleft'.$key, $val['img']);
					}
				}
			}

			// Execute hook printSearchForm
			$parameters = array('searchform' => $searchform);
			$reshook = $dyap_hookmanager->executeHooks('printSearchForm', $parameters); // Note that $action and $object may have been modified by some hooks
			if (empty($reshook)) {
				$searchform .= $dyap_hookmanager->resPrint;
			} else {
				$searchform = $dyap_hookmanager->resPrint;
			}

			// Force special value for $searchform
			if (!empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER) || empty($dyap_conf->use_javascript_ajax)) {
				$urltosearch = DOL_URL_ROOT.'/core/search_page.php?showtitlebefore=1';
				$searchform = '<div class="blockvmenuimpair blockvmenusearchphone"><div id="divsearchforms1"><a href="'.$urltosearch.'" accesskey="s" alt="'.dol_escape_htmltag($dyap_langs->trans("ShowSearchFields")).'">'.$dyap_langs->trans("Search").'...</a></div></div>';
			} elseif ($dyap_conf->use_javascript_ajax && !empty($dyap_conf->global->MAIN_USE_OLD_SEARCH_FORM)) {
				$searchform = '<div class="blockvmenuimpair blockvmenusearchphone"><div id="divsearchforms1"><a href="#" alt="'.dol_escape_htmltag($dyap_langs->trans("ShowSearchFields")).'">'.$dyap_langs->trans("Search").'...</a></div><div id="divsearchforms2" style="display: none">'.$searchform.'</div>';
				$searchform .= '<script>
            	jQuery(document).ready(function($) {
            		jQuery("#divsearchforms1").click(function(){
	                   jQuery("#divsearchforms2").toggle();
	               });
            	});
                </script>' . "\n";
				$searchform .= '</div>';
			}
		}

		// Left column
		print '<!-- Begin left menu -->'."\n";

		print '<div class="vmenu"'.(empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER) ? '' : ' title="Left menu"').'>'."\n\n";

		// Show left menu with other forms
		$dyap_menumanager->menu_array = $menu_array_before;
		$dyap_menumanager->menu_array_after = $menu_array_after;
		$dyap_menumanager->showmenu('left', array('searchform'=>$searchform)); // output menu_array and menu found in database

		// Dolibarr version + help + bug report link
		print "\n";
		print "<!-- Begin Help Block-->\n";
		print '<div id="blockvmenuhelp" class="blockvmenuhelp">'."\n";

		// Version
		if (!empty($dyap_conf->global->MAIN_SHOW_VERSION)) {    // Version is already on help picto and on login page.
		    $doliurl='https://dyapress.com/';
			//local communities
			if (preg_match('/fr/i', $dyap_langs->defaultlang)) {
			    $doliurl='https://dyapress.fr/';
				$doliurl = 'https://www.dolibarr.fr';
			}

			$appli = constant('DOL_APPLICATION_TITLE');
			if (!empty($conf->global->MAIN_APPLICATION_TITLE)) {
				$appli = $dyap_conf->global->MAIN_APPLICATION_TITLE; $doliurl = '';
				if (preg_match('/\d\.\d/', $appli)) {
					if (!preg_match('/'.preg_quote(DOL_VERSION).'/', $appli)) {
						$appli .= " (".DOL_VERSION.")"; // If new title contains a version that is different than core
					}
				} else {
					$appli .= " ".DOL_VERSION;
				}
			} else {
				$appli .= " ".DOL_VERSION;
			}
			print '<div id="blockvmenuhelpapp" class="blockvmenuhelp">';
			if ($doliurl) {
				print '<a class="help" target="_blank" rel="noopener noreferrer" href="'.$doliurl.'">';
			} else {
				print '<span class="help">';
			}
			print $appli;
			if ($doliurl) {
				print '</a>';
			} else {
				print '</span>';
			}
			print '</div>'."\n";
		}

		// Link to bugtrack
		if (!empty($dyap_conf->global->MAIN_BUGTRACK_ENABLELINK)) {
			require_once DOL_DOCUMENT_ROOT.'/core/lib/functions2.lib.php';

			if ($dyap_conf->global->MAIN_BUGTRACK_ENABLELINK == 'github') {
				$bugbaseurl = 'https://github.com/Dolibarr/dolibarr/issues/new?labels=Bug';
				$bugbaseurl .= '&title=';
				$bugbaseurl .= urlencode("Bug: ");
				$bugbaseurl .= '&body=';
				$bugbaseurl .= urlencode("# Instructions\n");
				$bugbaseurl .= urlencode("*This is a template to help you report good issues. You may use [Github Markdown](https://help.github.com/articles/getting-started-with-writing-and-formatting-on-github/) syntax to format your issue report.*\n");
				$bugbaseurl .= urlencode("*Please:*\n");
				$bugbaseurl .= urlencode("- *replace the bracket enclosed texts with meaningful information*\n");
				$bugbaseurl .= urlencode("- *remove any unused sub-section*\n");
				$bugbaseurl .= urlencode("\n");
				$bugbaseurl .= urlencode("\n");
				$bugbaseurl .= urlencode("# Bug\n");
				$bugbaseurl .= urlencode("[*Short description*]\n");
				$bugbaseurl .= urlencode("\n");
				$bugbaseurl .= urlencode("## Environment\n");
				$bugbaseurl .= urlencode("- **Version**: ".DOL_VERSION."\n");
				$bugbaseurl .= urlencode("- **OS**: ".php_uname('s')."\n");
				$bugbaseurl .= urlencode("- **Web server**: ".$_SERVER["SERVER_SOFTWARE"]."\n");
				$bugbaseurl .= urlencode("- **PHP**: ".php_sapi_name().' '.phpversion()."\n");
				$bugbaseurl .= urlencode("- **Database**: ".$db::LABEL.' '.$db->getVersion()."\n");
				$bugbaseurl .= urlencode("- **URL(s)**: ".$_SERVER["REQUEST_URI"]."\n");
				$bugbaseurl .= urlencode("\n");
				$bugbaseurl .= urlencode("## Expected and actual behavior\n");
				$bugbaseurl .= urlencode("[*Verbose description*]\n");
				$bugbaseurl .= urlencode("\n");
				$bugbaseurl .= urlencode("## Steps to reproduce the behavior\n");
				$bugbaseurl .= urlencode("[*Verbose description*]\n");
				$bugbaseurl .= urlencode("\n");
				$bugbaseurl .= urlencode("## [Attached files](https://help.github.com/articles/issue-attachments) (Screenshots, screencasts, dolibarr.log, debugging informations…)\n");
				$bugbaseurl .= urlencode("[*Files*]\n");
				$bugbaseurl .= urlencode("\n");

				$bugbaseurl .= urlencode("\n");
				$bugbaseurl .= urlencode("## Report\n");
			} elseif (!empty($dyap_conf->global->MAIN_BUGTRACK_ENABLELINK)) {
				$bugbaseurl = $dyap_conf->global->MAIN_BUGTRACK_ENABLELINK;
			} else {
				$bugbaseurl = "";
			}

			// Execute hook printBugtrackInfo
			$parameters = array('bugbaseurl' => $bugbaseurl);
			$reshook = $dyap_hookmanager->executeHooks('printBugtrackInfo', $parameters); // Note that $action and $object may have been modified by some hooks
			if (empty($reshook)) {
				$bugbaseurl .= $dyap_hookmanager->resPrint;
			} else {
				$bugbaseurl = $dyap_hookmanager->resPrint;
			}

			print '<div id="blockvmenuhelpbugreport" class="blockvmenuhelp">';
			print '<a class="help" target="_blank" rel="noopener noreferrer" href="'.$bugbaseurl.'">'.$dyap_langs->trans("FindBug").'</a>';
			print '</div>';
		}

		print "</div>\n";
		print "<!-- End Help Block-->\n";
		print "\n";

		print "</div>\n";
		print "<!-- End left menu -->\n";
		print "\n";

		// Execute hook printLeftBlock
		$parameters = array();
		$reshook = $dyap_hookmanager->executeHooks('printLeftBlock', $parameters); // Note that $action and $object may have been modified by some hooks
		print $dyap_hookmanager->resPrint;

		print '</div></div> <!-- End side-nav id-left -->'; // End div id="side-nav" div id="id-left"
	}

	print "\n";
	print '<!-- Begin right area -->'."\n";

	if (empty($leftmenuwithoutmainarea)) {
		main_area($title);
	}
}


/**
 *  Begin main area
 *
 *  @param	string	$title		Title
 *  @return	void
 */
function main_area($title='')
{
	global $dyap_conf, $dyap_langs, $dyap_hookmanager;

	if (empty($dyap_conf->dol_hide_leftmenu)) {
		print '<div id="id-right">';
	}

	print "\n";

	print '<!-- Begin div class="fiche" -->'."\n".'<div class="fiche">'."\n";

	$dyap_hookmanager->initHooks(array('main'));
	$parameters = array();
	$reshook = $dyap_hookmanager->executeHooks('printMainArea', $parameters); // Note that $action and $object may have been modified by some hooks
	print $dyap_hookmanager->resPrint;

	if (!empty($dyap_conf->global->MAIN_ONLY_LOGIN_ALLOWED)) {
		print info_admin($dyap_langs->trans("WarningYouAreInMaintenanceMode", $dyap_conf->global->MAIN_ONLY_LOGIN_ALLOWED), 0, 0, 1, 'warning maintenancemode');
	}

	// Permit to add user company information on each printed document by setting SHOW_SOCINFO_ON_PRINT
	if (!empty($dyap_conf->global->SHOW_SOCINFO_ON_PRINT) && GETPOST('optioncss', 'aZ09') == 'print' && empty(GETPOST('disable_show_socinfo_on_print', 'az09'))) {
		$parameters = array();
		$reshook = $dyap_hookmanager->executeHooks('showSocinfoOnPrint', $parameters);
		if (empty($reshook)) {
			print '<!-- Begin show mysoc info header -->'."\n";
			print '<div id="mysoc-info-header">'."\n";
			print '<table class="centpercent div-table-responsive">'."\n";
			print '<tbody>';
			print '<tr><td rowspan="0" class="width20p">';
			if ($dyap_conf->global->MAIN_SHOW_LOGO && empty($dyap_conf->global->MAIN_OPTIMIZEFORTEXTBROWSER) && !empty($dyap_conf->global->MAIN_INFO_SOCIETE_LOGO)) {
				print '<img id="mysoc-info-header-logo" style="max-width:100%" alt="" src="'.DOL_URL_ROOT.'/viewimage.php?cache=1&amp;modulepart=mycompany&amp;file='.urlencode('logos/'.dol_escape_htmltag($conf->global->MAIN_INFO_SOCIETE_LOGO)).'">';
			}
			print '</td><td  rowspan="0" class="width50p"></td></tr>'."\n";
			print '<tr><td class="titre bold">'.dol_escape_htmltag($dyap_conf->global->MAIN_INFO_SOCIETE_NOM).'</td></tr>'."\n";
			print '<tr><td>'.dol_escape_htmltag($dyap_conf->global->MAIN_INFO_SOCIETE_ADDRESS).'<br>'.dol_escape_htmltag($dyap_conf->global->MAIN_INFO_SOCIETE_ZIP).' '.dol_escape_htmltag($dyap_conf->global->MAIN_INFO_SOCIETE_TOWN).'</td></tr>'."\n";
			if (!empty($dyap_conf->global->MAIN_INFO_SOCIETE_TEL)) {
				print '<tr><td style="padding-left: 1em" class="small">'.$dyap_langs->trans("Phone").' : '.dol_escape_htmltag($dyap_conf->global->MAIN_INFO_SOCIETE_TEL).'</td></tr>';
			}
			if (!empty($dyap_conf->global->MAIN_INFO_SOCIETE_MAIL)) {
				print '<tr><td style="padding-left: 1em" class="small">'.$dyap_langs->trans("Email").' : '.dol_escape_htmltag($dyap_conf->global->MAIN_INFO_SOCIETE_MAIL).'</td></tr>';
			}
			if (!empty($dyap_conf->global->MAIN_INFO_SOCIETE_WEB)) {
				print '<tr><td style="padding-left: 1em" class="small">'.$dyap_langs->trans("Web").' : '.dol_escape_htmltag($dyap_conf->global->MAIN_INFO_SOCIETE_WEB).'</td></tr>';
			}
			print '</tbody>';
			print '</table>'."\n";
			print '</div>'."\n";
			print '<!-- End show mysoc info header -->'."\n";
		}
	}
}


/**
 *  Return helpbaseurl, helppage and mode
 *
 *  @param	string		$helppagename		Page name ('EN:xxx,ES:eee,FR:fff...' or 'http://localpage')
 *  @param  Translate	$dyap_langs				Language
 *  @return	array		Array of help urls
 */
function getHelpParamFor($helppagename,$dyap_langs)
{
	$helpbaseurl='';
	$helppage='';
	$mode='';

	if (preg_match('/^http/i',$helppagename)) {
		// If complete URL
		$helpbaseurl='%s';
		$helppage=$helppagename;
		$mode='local';
	} else {
		// If WIKI URL
		if (preg_match('/^es/i',$dyap_langs->defaultlang)) {
			$helpbaseurl='http://wiki.dolibarr.org/index.php/%s';
			if (preg_match('/ES:([^|]+)/i',$helppagename,$reg)) $helppage=$reg[1];
		}
		if (preg_match('/^fr/i',$dyap_langs->defaultlang)) {
			$helpbaseurl='http://wiki.dolibarr.org/index.php/%s';
			if (preg_match('/FR:([^|]+)/i',$helppagename,$reg)) $helppage=$reg[1];
		}
		if (empty($helppage)) { // If help page not already found
			$helpbaseurl='http://wiki.dolibarr.org/index.php/%s';
			if (preg_match('/EN:([^|]+)/i',$helppagename,$reg)) $helppage=$reg[1];
		}
		$mode='wiki';
	}
	return array('helpbaseurl'=>$helpbaseurl,'helppage'=>$helppage,'mode'=>$mode);
}


/**
 *  Show a search area.
 *  Used when the javascript quick search is not used.
 *
 *  @param  string	$urlaction          Url post
 *  @param  string	$urlobject          Url of the link under the search box
 *  @param  string	$title              Title search area
 *  @param  string	$htmlmorecss        Add more css
 *  @param  string	$htmlinputname      Field Name input form
 *  @param	string	$accesskey			Accesskey
 *  @param  string  $prefhtmlinputname  Complement for id to avoid multiple same id in the page
 *  @param	string	$img				Image to use
 *  @param	string	$showtitlebefore	Show title before input text instead of into placeholder. This can be set when output is dedicated for text browsers.
 *  @param	string	$autofocus			Set autofocus on field
 *  @return	string
 */
function printSearchForm($urlaction, $urlobject, $title, $htmlmorecss, $htmlinputname, $accesskey = '', $prefhtmlinputname = '', $img = '', $showtitlebefore = 0, $autofocus = 0)
{
	global $dyap_conf,$dyap_langs,$dyap_user;

	$ret = '';
	$ret .= '<form action="'.$urlaction.'" method="post" class="searchform nowraponall tagtr">';
	$ret .= '<input type="hidden" name="token" value="'.newToken().'">';
	$ret .= '<input type="hidden" name="mode" value="search">';
	$ret .= '<input type="hidden" name="savelogin" value="'.dol_escape_htmltag($dyap_user->login).'">';
	if ($showtitlebefore) {
		$ret .= '<div class="tagtd left">'.$title.'</div> ';
	}
	$ret .= '<div class="tagtd">';
	$ret .= img_picto('', $img, '', false, 0, 0, '', 'paddingright width20');
	$ret .= '<input type="text" class="flat '.$htmlmorecss.'"';
	$ret .= ' style="background-repeat: no-repeat; background-position: 3px;"';
	$ret .= ($accesskey ? ' accesskey="'.$accesskey.'"' : '');
	$ret .= ' placeholder="'.strip_tags($title).'"';
	$ret .= ($autofocus ? ' autofocus' : '');
	$ret .= ' name="'.$htmlinputname.'" id="'.$prefhtmlinputname.$htmlinputname.'" />';
	$ret .= '<button type="submit" class="button bordertransp" style="padding-top: 4px; padding-bottom: 4px; padding-left: 6px; padding-right: 6px">';
	$ret .= '<span class="fa fa-search"></span>';
	$ret .= '</button>';
	$ret .= '</div>';
	$ret .= "</form>\n";
	return $ret;
}


if (! function_exists("llxFooter")) {
	/**
	 * Show HTML footer
	 * Close div /DIV class=fiche + /DIV id-right + /DIV id-container + /BODY + /HTML.
	 * If global var $delayedhtmlcontent was filled, we output it just before closing the body.
	 *
	 * @param	string	$comment    				A text to add as HTML comment into HTML generated page
	 * @param	string	$zone						'private' (for private pages) or 'public' (for public pages)
	 * @param	int		$disabledoutputofmessages	Clear all messages stored into session without diplaying them
	 * @return	void
	 */
	function llxFooter($comment = '', $zone = 'private', $disabledoutputofmessages = 0)
	{
		global $dyap_conf, $dyap_langs, $dyap_user, $dyap_object, $dyap_hookmanager;
		global $dyap_delayedhtmlcontent;
		global $dyap_contextpage, $dyap_page, $dyap_limit;
		global $dolibarr_distrib;

		// keep track of original llxFooter() called
		global $dyap_output_llx_footer;
		if ( !$dyap_output_llx_footer )
		    $dyap_output_llx_footer = 1;

		$ext = 'layout='.$dyap_conf->browser->layout.'&version='.urlencode(DOL_VERSION);

		// Global html output events ($mesgs, $errors, $warnings)
		dol_htmloutput_events($disabledoutputofmessages);

		// Code for search criteria persistence.
		// $dyap_user->lastsearch_values was set by the GETPOST when form field search_xxx exists
		if (is_object($dyap_user) && !empty($dyap_user->lastsearch_values_tmp) && is_array($dyap_user->lastsearch_values_tmp)) {
			// Clean and save data
			foreach ($dyap_user->lastsearch_values_tmp as $key => $val) {
				unset($_SESSION['lastsearch_values_tmp_'.$key]); // Clean array to rebuild it just after
				if (count($val) && empty($_POST['button_removefilter'])) {	// If there is search criteria to save and we did not click on 'Clear filter' button
					if (empty($val['sortfield'])) {
						unset($val['sortfield']);
					}
					if (empty($val['sortorder'])) {
						unset($val['sortorder']);
					}
					dol_syslog('Save lastsearch_values_tmp_'.$key.'='.json_encode($val, 0)." (systematic recording of last search criterias)");
					$_SESSION['lastsearch_values_tmp_'.$key] = json_encode($val);
					unset($_SESSION['lastsearch_values_'.$key]);
				}
			}
		}


		$relativepathstring = $_SERVER["PHP_SELF"];
		// Clean $relativepathstring
		if (constant('DOL_URL_ROOT')) {
			$relativepathstring = preg_replace('/^'.preg_quote(constant('DOL_URL_ROOT'), '/').'/', '', $relativepathstring);
		}
		$relativepathstring = preg_replace('/^\//', '', $relativepathstring);
		$relativepathstring = preg_replace('/^custom\//', '', $relativepathstring);
		if (preg_match('/list\.php$/', $relativepathstring)) {
			unset($_SESSION['lastsearch_contextpage_tmp_'.$relativepathstring]);
			unset($_SESSION['lastsearch_page_tmp_'.$relativepathstring]);
			unset($_SESSION['lastsearch_limit_tmp_'.$relativepathstring]);

			if (!empty($contextpage)) {
				$_SESSION['lastsearch_contextpage_tmp_'.$relativepathstring] = $contextpage;
			}
			if (!empty($page) && $page > 0) {
				$_SESSION['lastsearch_page_tmp_'.$relativepathstring] = $page;
			}
			if (!empty($limit) && $limit != $dyap_conf->liste_limit) {
				$_SESSION['lastsearch_limit_tmp_'.$relativepathstring] = $limit;
			}

			unset($_SESSION['lastsearch_contextpage_'.$relativepathstring]);
			unset($_SESSION['lastsearch_page_'.$relativepathstring]);
			unset($_SESSION['lastsearch_limit_'.$relativepathstring]);
		}

		// Core error message
		if (!empty($dyap_conf->global->MAIN_CORE_ERROR)) {
			// Ajax version
			if ($dyap_conf->use_javascript_ajax) {
				$title = img_warning().' '.$dyap_langs->trans('CoreErrorTitle');
				print ajax_dialog($title, $dyap_langs->trans('CoreErrorMessage'));
			} else {
				// html version
				$msg = img_warning().' '.$dyap_langs->trans('CoreErrorMessage');
				print '<div class="error">'.$msg.'</div>';
			}

			//define("MAIN_CORE_ERROR",0);      // Constant was defined and we can't change value of a constant
		}

		print "\n\n";

		print '</div> <!-- End div class="fiche" -->'."\n"; // End div fiche

		if (empty($dyap_conf->dol_hide_leftmenu)) {
			print '</div> <!-- End div id-right -->'."\n"; // End div id-right
		}

		if (empty($dyap_conf->dol_hide_leftmenu) && empty($dyap_conf->dol_use_jmobile)) {
			print '</div> <!-- End div id-container -->'."\n"; // End div container
		}

		print "\n";
		if ($comment) {
			print '<!-- '.$comment.' -->'."\n";
		}

		printCommonFooter($zone);

		if (!empty($dyap_delayedhtmlcontent)) {
			print $dyap_delayedhtmlcontent;
		}

		if (!empty($dyap_conf->use_javascript_ajax)) {
			print "\n".'<!-- Includes JS Footer of Dolibarr -->'."\n";
			print '<script type="text/javascript" src="'.\dyap_url('/core/js/lib_foot.js.php?lang='.$dyap_langs->defaultlang.($ext?'&'.$ext:'')).'"></script>'."\n";
		}

		// Wrapper to add log when clicking on download or preview
		if (!empty($dyap_conf->blockedlog->enabled) && is_object($dyap_object) && !empty($dyap_object->id) && $dyap_object->id > 0 && $dyap_object->statut > 0) {
			if (in_array($dyap_object->element, array('facture'))) {       // Restrict for the moment to element 'facture'
				print "\n<!-- JS CODE TO ENABLE log when making a download or a preview of a document -->\n";
				?>
				<script>
				jQuery(document).ready(function($) {
					$('a.documentpreview').click(function() {
						$.post('<?php echo DOL_URL_ROOT."/blockedlog/ajax/block-add.php" ?>'
								, {
									id:<?php echo $dyap_object->id; ?>
									, element:'<?php echo $dyap_object->element ?>'
									, action:'DOC_PREVIEW'
									, token: '<?php echo currentToken(); ?>'
								}
						);
					});
					$('a.documentdownload').click(function() {
						$.post('<?php echo DOL_URL_ROOT."/blockedlog/ajax/block-add.php" ?>'
								, {
									id:<?php echo $dyap_object->id; ?>
									, element:'<?php echo $dyap_object->element ?>'
									, action:'DOC_DOWNLOAD'
									, token: '<?php echo currentToken(); ?>'
								}
						);
					});
				});
				</script>
				<?php
			}
		}

		// A div for the address popup
		print "\n<!-- A div to allow dialog popup by jQuery('#dialogforpopup').dialog() -->\n";
		print '<div id="dialogforpopup" style="display: none;"></div>'."\n";

		$reshook = $dyap_hookmanager->executeHooks('beforeBodyClose'); // Note that $action and $object may have been modified by some hooks
		if ($reshook > 0) {
			print $dyap_hookmanager->resPrint;
		}

		print "</div><!-- #dyapress -->\n";

		global $dyap_output_llx_header;
		if ( empty( $dyap_output_llx_header) ) {
		    print "</body>\n";
		    print "</html>\n";
		}
	}
}
