<?php
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

WordPress DyaPress plugin :
Copyright (C) 2016-2022      François Pons        <fpons@aytechnet.fr>
 */

namespace DyaPress;

/*
 * This file allows compatibility mode with removed files with Dolibarr code :
 *    filefunc.inc.php
 *    master.inc.php
 *    main.inc.php
 */

if (function_exists("stripslashes_deep")) {
    $_GET = stripslashes_deep($_GET);
    $_POST = stripslashes_deep($_POST);
    $_REQUEST = stripslashes_deep($_REQUEST);
    $_COOKIE = stripslashes_deep($_COOKIE);
}

if (!defined('DOL_APPLICATION_TITLE')) {
	define('DOL_APPLICATION_TITLE', 'Dolibarr');
}
if (!defined('DOL_VERSION')) {
	define('DOL_VERSION', '16.0.5'); // a.b.c-alpha, a.b.c-beta, a.b.c-rcX or a.b.c
}

if (!defined('NOCSRFCHECK')) {
        define('NOCSRFCHECK', 1);
}

if (!defined('EURO')) {
	define('EURO', chr(128));
}

// Define syslog constants
if (!defined('LOG_DEBUG')) {
	if (!function_exists("syslog")) {
		// For PHP versions without syslog (like running on Windows OS)
		define('LOG_EMERG', 0);
		define('LOG_ALERT', 1);
		define('LOG_CRIT', 2);
		define('LOG_ERR', 3);
		define('LOG_WARNING', 4);
		define('LOG_NOTICE', 5);
		define('LOG_INFO', 6);
		define('LOG_DEBUG', 7);
	}
}

// End of common declaration part
if (defined('DOL_INC_FOR_VERSION_ERROR')) {
	return;
}

// DyaPress : use WordPress confguration by default
if (empty($dolibarr_main_db_host))
	$dolibarr_main_db_host = DB_HOST;
if (empty($dolibarr_main_db_name))
	$dolibarr_main_db_name = DB_NAME;
if (empty($dolibarr_main_db_user))
	$dolibarr_main_db_user = DB_USER;
if (empty($dolibarr_main_db_pass))
	$dolibarr_main_db_pass = DB_PASSWORD;
if (empty($dolibarr_main_db_character_set))
	$dolibarr_main_db_character_set = DB_CHARSET;
if (empty($dolibarr_character_set_client))
	$dolibarr_character_set_client = preg_match('/utf8/', $dolibarr_main_db_character_set) ? 'UTF-8' : 'ISO-8859-1';
if (empty($dolibarr_main_db_collation))
	$dolibarr_main_db_collation = DB_COLLATE;
if (empty($dolibarr_main_document_root))
	$dolibarr_main_document_root = dirname(__FILE__);
if (empty($dolibarr_main_data_root))
	$dolibarr_main_data_root = \wp_get_upload_dir()['basedir'] . DIRECTORY_SEPARATOR . 'dyapress';
	// $dolibarr_main_data_root = dirname(dirname(dirname(__FILE__))) . '/uploads/dyapress';
if (empty($dolibarr_main_url_root))
	$dolibarr_main_url_root = plugin_dir_url( __FILE__ );

// special case for custom Dolibarr extensions
global $dolibarr_main_url_root_alt, $dolibarr_main_document_root_alt;
if (empty($dolibarr_main_url_root_alt))
	$dolibarr_main_url_root_alt = '/custom,'.\wp_get_upload_dir()['baseurl'] . DIRECTORY_SEPARATOR . 'dyapress' . DIRECTORY_SEPARATOR . 'custom';
if (empty($dolibarr_main_document_root_alt))
	$dolibarr_main_document_root_alt = \wp_get_upload_dir()['basedir'] . DIRECTORY_SEPARATOR . 'dyapress' . DIRECTORY_SEPARATOR . 'custom';

// Force PHP error_reporting setup (Dolibarr may report warning without this)
if (!empty($dolibarr_strict_mode)) {
	error_reporting(E_ALL | E_STRICT);
} else {
	error_reporting(E_ALL & ~(E_STRICT | E_NOTICE | E_DEPRECATED));
}

// Disable php display errors
if (!empty($dolibarr_main_prod)) {
	ini_set('display_errors', 'Off');
}

// Clean parameters
$dolibarr_main_data_root = trim($dolibarr_main_data_root);
$dolibarr_main_url_root = trim(preg_replace('/\/+$/', '', $dolibarr_main_url_root));
$dolibarr_main_url_root_alt = (empty($dolibarr_main_url_root_alt) ? '' : trim($dolibarr_main_url_root_alt));
$dolibarr_main_document_root = trim($dolibarr_main_document_root);
$dolibarr_main_document_root_alt = (empty($dolibarr_main_document_root_alt) ? '' : trim($dolibarr_main_document_root_alt));

if (empty($dolibarr_main_db_port)) {
	$dolibarr_main_db_port = 3306; // For compatibility with old configs, if not defined, we take 'mysql' type
}
if (empty($dolibarr_main_db_type)) {
	$dolibarr_main_db_type = 'mysqli'; // For compatibility with old configs, if not defined, we take 'mysql' type
}

// Mysql driver support has been removed in favor of mysqli
if ($dolibarr_main_db_type == 'mysql') {
	$dolibarr_main_db_type = 'mysqli';
}
if (empty($dolibarr_main_db_prefix)) {
	$dolibarr_main_db_prefix = 'llx_';
}
if (empty($dolibarr_main_db_character_set)) {
	$dolibarr_main_db_character_set = ($dolibarr_main_db_type == 'mysqli' ? 'utf8' : ''); // Old installation
}
if (empty($dolibarr_main_db_collation)) {
	$dolibarr_main_db_collation = ($dolibarr_main_db_type == 'mysqli' ? 'utf8_unicode_ci' : ''); // Old installation
}
if (empty($dolibarr_main_db_encryption)) {
	$dolibarr_main_db_encryption = 0;
}
if (empty($dolibarr_main_db_cryptkey)) {
	$dolibarr_main_db_cryptkey = '';
}
if (empty($dolibarr_main_limit_users)) {
	$dolibarr_main_limit_users = 0;
}
if (empty($dolibarr_mailing_limit_sendbyweb)) {
	$dolibarr_mailing_limit_sendbyweb = 0;
}
if (empty($dolibarr_mailing_limit_sendbycli)) {
	$dolibarr_mailing_limit_sendbycli = 0;
}
if (empty($dolibarr_strict_mode)) {
	$dolibarr_strict_mode = 0; // For debug in php strict mode
}

// Define some constants
define('DOL_CLASS_PATH', 'class/');					// Filesystem path to class dir (defined only for some code that want to be compatible with old versions without this parameter)
define('DOL_DATA_ROOT', $dolibarr_main_data_root);			// Filesystem data (documents)
define('DOL_DOCUMENT_ROOT', $dolibarr_main_document_root);		// Filesystem core php (htdocs)
define('DOL_MAIN_URL_ROOT', plugins_url( '/', __FILE__ ));		// URL absolute root (https://sss/dolibarr, ...)
define('DOL_URL_ROOT', preg_replace('/\/$/', '', parse_url(DOL_MAIN_URL_ROOT, PHP_URL_PATH)));	// URL relative root ('', '/dolibarr', ...)

// Define prefix MAIN_DB_PREFIX
define('MAIN_DB_PREFIX',$dolibarr_main_db_prefix);

/*
 * Define PATH to external libraries
 * To use other version than embeded libraries, define here constant to path. Use '' to use include class path autodetect.
 */
// Path to root libraries
if (!defined('ADODB_PATH')) {
	define('ADODB_PATH', (!isset($dolibarr_lib_ADODB_PATH)) ?DOL_DOCUMENT_ROOT.'/includes/adodbtime/' : (empty($dolibarr_lib_ADODB_PATH) ? '' : $dolibarr_lib_ADODB_PATH.'/'));
}
if (!defined('TCPDF_PATH')) {
	define('TCPDF_PATH', (empty($dolibarr_lib_TCPDF_PATH)) ?DOL_DOCUMENT_ROOT.'/includes/tecnickcom/tcpdf/' : $dolibarr_lib_TCPDF_PATH.'/');
}
if (!defined('TCPDI_PATH')) {
	define('TCPDI_PATH', (empty($dolibarr_lib_TCPDI_PATH)) ?DOL_DOCUMENT_ROOT.'/includes/tcpdi/' : $dolibarr_lib_TCPDI_PATH.'/');
}
if (!defined('NUSOAP_PATH')) {
	define('NUSOAP_PATH', (!isset($dolibarr_lib_NUSOAP_PATH)) ?DOL_DOCUMENT_ROOT.'/includes/nusoap/lib/' : (empty($dolibarr_lib_NUSOAP_PATH) ? '' : $dolibarr_lib_NUSOAP_PATH.'/'));
}
if (!defined('PHPEXCELNEW_PATH')) {
	define('PHPEXCELNEW_PATH', (!isset($dolibarr_lib_PHPEXCELNEW_PATH)) ?DOL_DOCUMENT_ROOT.'/includes/phpoffice/phpspreadsheet/src/PhpSpreadsheet/' : (empty($dolibarr_lib_PHPEXCELNEW_PATH) ? '' : $dolibarr_lib_PHPEXCELNEW_PATH.'/'));
}
if (!defined('ODTPHP_PATH')) {
	define('ODTPHP_PATH', (!isset($dolibarr_lib_ODTPHP_PATH)) ?DOL_DOCUMENT_ROOT.'/includes/odtphp/' : (empty($dolibarr_lib_ODTPHP_PATH) ? '' : $dolibarr_lib_ODTPHP_PATH.'/'));
}
if (!defined('ODTPHP_PATHTOPCLZIP')) {
	define('ODTPHP_PATHTOPCLZIP', (!isset($dolibarr_lib_ODTPHP_PATHTOPCLZIP)) ?DOL_DOCUMENT_ROOT.'/includes/odtphp/zip/pclzip/' : (empty($dolibarr_lib_ODTPHP_PATHTOPCLZIP) ? '' : $dolibarr_lib_ODTPHP_PATHTOPCLZIP.'/'));
}
if (!defined('JS_CKEDITOR')) {
	define('JS_CKEDITOR', (!isset($dolibarr_js_CKEDITOR)) ? '' : (empty($dolibarr_js_CKEDITOR) ? '' : $dolibarr_js_CKEDITOR.'/'));
}
if (!defined('JS_JQUERY')) {
	define('JS_JQUERY', (!isset($dolibarr_js_JQUERY)) ? '' : (empty($dolibarr_js_JQUERY) ? '' : $dolibarr_js_JQUERY.'/'));
}
if (!defined('JS_JQUERY_UI')) {
	define('JS_JQUERY_UI', (!isset($dolibarr_js_JQUERY_UI)) ? '' : (empty($dolibarr_js_JQUERY_UI) ? '' : $dolibarr_js_JQUERY_UI.'/'));
}
// Other required path
if (!defined('DOL_DEFAULT_TTF')) {
	define('DOL_DEFAULT_TTF', (!isset($dolibarr_font_DOL_DEFAULT_TTF)) ?DOL_DOCUMENT_ROOT.'/includes/fonts/Aerial.ttf' : (empty($dolibarr_font_DOL_DEFAULT_TTF) ? '' : $dolibarr_font_DOL_DEFAULT_TTF));
}
if (!defined('DOL_DEFAULT_TTF_BOLD')) {
	define('DOL_DEFAULT_TTF_BOLD', (!isset($dolibarr_font_DOL_DEFAULT_TTF_BOLD)) ?DOL_DOCUMENT_ROOT.'/includes/fonts/AerialBd.ttf' : (empty($dolibarr_font_DOL_DEFAULT_TTF_BOLD) ? '' : $dolibarr_font_DOL_DEFAULT_TTF_BOLD));
}


/*
 * Include functions
 */

if (!defined('ADODB_DATE_VERSION') && file_exists(ADODB_PATH.'adodb-time.inc.php')) {
	include_once ADODB_PATH.'adodb-time.inc.php';
}

// the following file defines all the various DyaPress globals (from Dolibarr) which need to be protected in WordPress
include DOL_DOCUMENT_ROOT.'/dyapress-globals.php';

if (file_exists(DOL_DOCUMENT_ROOT ."/core/lib/functions.lib.php")) {
	// Included by default
	include_once DOL_DOCUMENT_ROOT .'/core/lib/functions.lib.php';
	include_once DOL_DOCUMENT_ROOT .'/core/lib/security.lib.php';

	/*
	 * Create $dyap_conf object
	 */

	require_once DOL_DOCUMENT_ROOT.'/core/class/conf.class.php';

	$dyap_conf = new Conf();
} else {
	// degraded compatibility mode without DyaPress fully loaded
	$dyap_conf = new \stdClass();
	$dyap_conf->db = new \stdClass();
	$dyap_conf->file = new \stdClass();
}

if (!function_exists('is_countable')) {
	/**
	 * function is_countable (to remove when php version supported will be >= 7.3)
	 * @param mixed $c data to check if countable
	 * @return bool
	 */
	function is_countable($c)
	{
		return is_array($c) || $c instanceof Countable;
	}
}

// Set properties specific to database
$dyap_conf->db->host = $dolibarr_main_db_host;
$dyap_conf->db->port = $dolibarr_main_db_port;
$dyap_conf->db->name = $dolibarr_main_db_name;
$dyap_conf->db->user = $dolibarr_main_db_user;
$dyap_conf->db->pass = empty($dolibarr_main_db_pass) ? '' : $dolibarr_main_db_pass;
$dyap_conf->db->type = $dolibarr_main_db_type;
$dyap_conf->db->prefix = $dolibarr_main_db_prefix;
$dyap_conf->db->character_set = $dolibarr_main_db_character_set;
$dyap_conf->db->dolibarr_main_db_collation = $dolibarr_main_db_collation;
$dyap_conf->db->dolibarr_main_db_encryption = $dolibarr_main_db_encryption;
$dyap_conf->db->dolibarr_main_db_cryptkey = $dolibarr_main_db_cryptkey;
if (defined('TEST_DB_FORCE_TYPE')) {
	$dyap_conf->db->type = constant('TEST_DB_FORCE_TYPE'); // Force db type (for test purpose, by PHP unit for example)
}

// Set properties specific to conf file
$dyap_conf->file->main_limit_users = $dolibarr_main_limit_users;
$dyap_conf->file->mailing_limit_sendbyweb = $dolibarr_mailing_limit_sendbyweb;
$dyap_conf->file->mailing_limit_sendbycli = $dolibarr_mailing_limit_sendbycli;
$dyap_conf->file->main_authentication = empty($dolibarr_main_authentication) ? '' : $dolibarr_main_authentication; // Identification mode
$dyap_conf->file->main_force_https = empty($dolibarr_main_force_https) ? '' : $dolibarr_main_force_https; // Force https
$dyap_conf->file->strict_mode = empty($dolibarr_strict_mode) ? '' : $dolibarr_strict_mode; // Force php strict mode (for debug)
$dyap_conf->file->instance_unique_id = empty($dolibarr_main_instance_unique_id) ? (empty($dolibarr_main_cookie_cryptkey) ? '' : $dolibarr_main_cookie_cryptkey) : $dolibarr_main_instance_unique_id; // Unique id of instance
$dyap_conf->file->dol_document_root = array('main' => (string) DOL_DOCUMENT_ROOT); // Define array of document root directories ('/home/htdocs')
$dyap_conf->file->dol_url_root = array('main' => (string) DOL_URL_ROOT); // Define array of url root path ('' or '/dolibarr')

// Chargement des includes principaux de librairies communes
if (!defined('NOREQUIREUSER') && file_exists(DOL_DOCUMENT_ROOT.'/user/class/user.class.php')) {
	require_once DOL_DOCUMENT_ROOT.'/user/class/user.class.php'; // Need 500ko memory
}
if (!defined('NOREQUIRETRAN') && file_exists(DOL_DOCUMENT_ROOT.'/core/class/translate.class.php')) {
	require_once DOL_DOCUMENT_ROOT.'/core/class/translate.class.php';
}
if (!defined('NOREQUIRESOC') && file_exists(DOL_DOCUMENT_ROOT.'/societe/class/societe.class.php')) {
	require_once DOL_DOCUMENT_ROOT.'/societe/class/societe.class.php';
}


/*
 * Creation objet $langs (must be before all other code)
 */
if (!defined('NOREQUIRETRAN') && class_exists('\DyaPress\Translate')) {
	$dyap_langs = new Translate('', $dyap_conf); // Must be after reading conf
}

/*
 * Object $dyap_db
 */
$dyap_db = null;
if (!defined('NOREQUIREDB') && function_exists('\DyaPress\getDoliDBInstance') ) {
	$dyap_db = getDoliDBInstance($dyap_conf->db->type, $dyap_conf->db->host, $dyap_conf->db->user, $dyap_conf->db->pass, $dyap_conf->db->name, $dyap_conf->db->port);

	if ($dyap_db->error) {
		dol_print_error($dyap_db, "host=".$dyap_conf->db->host.", port=".$dyap_conf->db->port.", user=".$dyap_conf->db->user.", databasename=".$dyap_conf->db->name.", ".$dyap_db->error);
	}
}

// Now database connexion is known, so we can forget password
//unset($dolibarr_main_db_pass); 	// We comment this because this constant is used in a lot of pages
unset($dyap_conf->db->pass);			// This is to avoid password to be shown in memory/swap dump


/*
 * Object $dyap_user
 */
if (!defined('NOREQUIREUSER') && class_exists('\DyaPress\User')) {
	$dyap_user = new User($dyap_db);
}


/*
 * Load object $dyap_conf
 * After this, all parameters conf->global->CONSTANTS are loaded
 */

// By default conf->entity is 1, but we change this if we ask another value.
if (function_exists('\DyaPress\GETPOSTISSET') && function_exists('\DyaPress\GETPOST')) {
	if (session_id() && !empty($_SESSION["dol_entity"])) {
		// Entity inside an opened session
		$dyap_conf->entity = $_SESSION["dol_entity"];
	} elseif (!empty($_ENV["dol_entity"])) {
		// Entity inside a CLI script
		$dyap_conf->entity = $_ENV["dol_entity"];
	} elseif (GETPOSTISSET("loginfunction") && GETPOST("entity", 'int')) {
		// Just after a login page
		$dyap_conf->entity = GETPOST("entity", 'int');
	} elseif (defined('DOLENTITY') && is_numeric(constant('DOLENTITY'))) {
		// For public page with MultiCompany module
		$dyap_conf->entity = constant('DOLENTITY');
	}
}
// Sanitize entity
if (!is_numeric($dyap_conf->entity)) {
	$dyap_conf->entity = 1;
}
// Here we read database (llx_const table) and define $dyap_conf->global->XXX var.
//print "We work with data into entity instance number '".$dyap_conf->entity."'";
if (method_exists($dyap_conf, 'setValues')) {
	$dyap_conf->setValues($dyap_db);
} else {
    global $wpdb;

    $result = $wpdb->get_results("SELECT name,value,entity FROM ".MAIN_DB_PREFIX."const WHERE entity IN(0, ".$dyap_conf->entity.") ORDER BY entity");

    $dyap_conf->global = new \stdClass();

    foreach ($result as $row) {
	if (isset( $row->name ) )
	    $dyap_conf->global->{$row->name} = $row->value;
    }
}

// Create object $dyap_mysoc (A thirdparty object that contains properties of companies managed by Dolibarr.
if (!defined('NOREQUIREDB') && !defined('NOREQUIRESOC') && file_exists(DOL_DOCUMENT_ROOT.'/societe/class/societe.class.php')) {
	require_once DOL_DOCUMENT_ROOT.'/societe/class/societe.class.php';

	$dyap_mysoc = new Societe($dyap_db);
	$dyap_mysoc->setMysoc($dyap_conf);

	// For some countries, we need to invert our address with customer address
	if ($dyap_mysoc->country_code == 'DE' && !isset($dyap_conf->global->MAIN_INVERT_SENDER_RECIPIENT)) {
		$dyap_conf->global->MAIN_INVERT_SENDER_RECIPIENT = 1;
	}
}


// Set default language (must be after the setValues setting global $dyap_conf->global->MAIN_LANG_DEFAULT. Page main.inc.php will overwrite dyap_langs->defaultlang with user value later)
if (!defined('NOREQUIRETRAN') && function_exists('\DyaPress\GETPOST')) {
	$langcode = (GETPOST('lang', 'aZ09') ? GETPOST('lang', 'aZ09', 1) : (empty($dyap_conf->global->MAIN_LANG_DEFAULT) ? 'auto' : $dyap_conf->global->MAIN_LANG_DEFAULT));
	if (defined('MAIN_LANG_DEFAULT')) {	// So a page can force the language whatever is setup and parameters in URL
		$langcode = constant('MAIN_LANG_DEFAULT');
	}
	if (isset($dyap_langs))
		$dyap_langs->setDefaultLang($langcode);
}


// Create the global $dyap_hookmanager object
if (file_exists(DOL_DOCUMENT_ROOT.'/core/class/hookmanager.class.php')) {
	include_once DOL_DOCUMENT_ROOT.'/core/class/hookmanager.class.php';
	$dyap_hookmanager = new HookManager($dyap_db);
}


if (!defined('MAIN_LABEL_MENTION_NPR')) {
	define('MAIN_LABEL_MENTION_NPR', 'NPR');
}

