<?php
/* Copyright (C) 2004-2005 Rodolphe Quiedeville <rodolphe@quiedeville.org>
 * Copyright (C) 2005-2016 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2005-2016 Regis Houssin        <regis.houssin@inodbox.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 * or see https://www.gnu.org/
 */

/**
 *		\file       htdocs/viewimage.php
 *		\brief      Wrapper to show images into Dolibarr screens.
 *		\remarks    Call to wrapper is :
 *					DOL_URL_ROOT.'/viewimage.php?modulepart=diroffile&file=relativepathofofile&cache=0
 *					DOL_URL_ROOT.'/viewimage.php?hashp=sharekey
 */

/* DyaPress plugin for WordPress
 *
 * Copyright (c) 2019   François Pons   <fpons@aytechnet.fr>
 *
 * the DyaPress namespace is added in order to avoid conflicts
 * with all the various WordPress extensions.
 */
namespace DyaPress;

/**
 * Header empty
 *
 * @ignore
 * @return	void
 */
/* function llxHeader()
{
}
 */
/**
 * Footer empty
 *
 * @ignore
 * @return	void
 */
/*
function llxFooter()
{
}
 */

// For multicompany
$dyap_entity = (!empty($_GET['entity']) ? (int) $_GET['entity'] : (!empty($_POST['entity']) ? (int) $_POST['entity'] : 1));
if (is_numeric($dyap_entity)) {
	define("DOLENTITY", $dyap_entity);
}

require 'main.inc.php'; // Load $user and permissions
require_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';

$dyap_action = GETPOST('action', 'aZ09');
$dyap_original_file = GETPOST('file', 'alphanohtml'); // Do not use urldecode here ($_GET are already decoded by PHP).
$dyap_hashp = GETPOST('hashp', 'aZ09');
$dyap_modulepart = GETPOST('modulepart', 'alpha');
$dyap_urlsource = GETPOST('urlsource', 'alpha');
$dyap_entity = GETPOST('entity', 'int') ?GETPOST('entity', 'int') : $dyap_conf->entity;

// Security check
if (empty($dyap_modulepart) && empty($dyap_hashp)) {
	accessforbidden('Bad link. Bad value for parameter modulepart', 0, 0, 1);
}
if (empty($dyap_original_file) && empty($dyap_hashp) && $dyap_modulepart != 'barcode') {
	accessforbidden('Bad link. Missing identification to find file (param file or hashp)', 0, 0, 1);
}
if ($dyap_modulepart == 'fckeditor') {
	$dyap_modulepart = 'medias'; // For backward compatibility
}



/*
 * Actions
 */

// None



/*
 * View
 */

if (GETPOST("cache", 'alpha')) {
	// Important: Following code is to avoid page request by browser and PHP CPU at
	// each Dolibarr page access.
	if (empty($dolibarr_nocache)) {
		header('Cache-Control: max-age=3600, public, must-revalidate');
		header('Pragma: cache'); // This is to avoid having Pragma: no-cache
	} else {
		header('Cache-Control: no-cache');
	}
	//print $dolibarr_nocache; exit;
}

// If we have a hash public (hashp), we guess the original_file.
if (!empty($dyap_hashp)) {
	include_once DOL_DOCUMENT_ROOT.'/ecm/class/ecmfiles.class.php';
	$ecmfile = new EcmFiles($dyap_db);
	$result = $ecmfile->fetch(0, '', '', '', $dyap_hashp);
	if ($result > 0) {
		$tmp = explode('/', $ecmfile->filepath, 2); // $ecmfile->filepath is relative to document directory
		// filepath can be 'users/X' or 'X/propale/PR11111'
		if (is_numeric($tmp[0])) { // If first tmp is numeric, it is subdir of company for multicompany, we take next part.
			$tmp = explode('/', $tmp[1], 2);
		}
		$dyap_moduleparttocheck = $tmp[0]; // moduleparttocheck is first part of path

		if ($dyap_modulepart) {	// Not required, so often not defined, for link using public hashp parameter.
			if ($dyap_moduleparttocheck == $dyap_modulepart) {
				// We remove first level of directory
				$dyap_original_file = (($tmp[1] ? $tmp[1].'/' : '').$ecmfile->filename); // this is relative to module dir
				//var_dump($original_file); exit;
			} else {
				accessforbidden('Bad link. File is from another module part.', 0, 0, 1);
			}
		} else {
			$dyap_modulepart = $dyap_moduleparttocheck;
			$dyap_original_file = (($tmp[1] ? $tmp[1].'/' : '').$ecmfile->filename); // this is relative to module dir
		}
	} else {
		$dyap_langs->load("errors");
		accessforbidden($dyap_langs->trans("ErrorFileNotFoundWithSharedLink"), 0, 0, 1);
	}
}

// Define mime type
$type = 'application/octet-stream';
if (GETPOST('type', 'alpha')) {
	$type = GETPOST('type', 'alpha');
} else {
	$type = dol_mimetype($dyap_original_file);
}

// Security: This wrapper is for images. We do not allow type/html
if (preg_match('/html/i', $type)) {
	accessforbidden('Error: Using the image wrapper to output a file with a mime type HTML is not possible.', 0, 0, 1);
}
// Security: This wrapper is for images. We do not allow files ending with .noexe
if (preg_match('/\.noexe$/i', $dyap_original_file)) {
	accessforbidden('Error: Using the image wrapper to output a file ending with .noexe is not allowed.', 0, 0, 1);
}

// Security: Delete string ../ or ..\ into $original_file
$dyap_original_file = preg_replace('/\.\.+/', '..', $dyap_original_file);	// Replace '... or more' with '..'
$dyap_original_file = str_replace('../', '/', $dyap_original_file);
$dyap_original_file = str_replace('..\\', '/', $dyap_original_file);

// Find the subdirectory name as the reference
$refname = basename(dirname($dyap_original_file)."/");
if ($refname == 'thumbs') {
	// If we get the thumbs directory, we must go one step higher. For example original_file='10/thumbs/myfile_small.jpg' -> refname='10'
	$refname = basename(dirname(dirname($dyap_original_file))."/");
}

// Check that file is allowed for view with viewimage.php
if (!empty($dyap_original_file) && !dolIsAllowedForPreview($dyap_original_file)) {
	accessforbidden('This file is not qualified for preview', 0, 0, 1);
}

// Security check
if (empty($dyap_modulepart)) {
	accessforbidden('Bad value for parameter modulepart', 0, 0, 1);
}

// When logged in a different entity, medias cannot be accessed because $conf->$module->multidir_output
// is not set on the requested entity, but they are public documents, so reset entity
if ($dyap_modulepart === 'medias' && $dyap_entity != $dyap_conf->entity) {
	$dyap_conf->entity = $dyap_entity;
	$dyap_conf->setValues($dyap_db);
}

$check_access = dol_check_secure_access_document($dyap_modulepart, $dyap_original_file, $dyap_entity, $dyap_user, $refname);
$accessallowed              = $check_access['accessallowed'];
$sqlprotectagainstexternals = $check_access['sqlprotectagainstexternals'];
$dyap_fullpath_original_file     = $check_access['original_file']; // $fullpath_original_file is now a full path name

if (!empty($dyap_hashp)) {
	$accessallowed = 1; // When using hashp, link is public so we force $accessallowed
	$sqlprotectagainstexternals = '';
} elseif (isset($_GET["publictakepos"])) {
	if (!empty($dyap_conf->global->TAKEPOS_AUTO_ORDER)) {
		$accessallowed = 1; // Only if TakePOS Public Auto Order is enabled and received publictakepos variable
	}
} else {
	// Basic protection (against external users only)
	if ($dyap_user->socid > 0) {
		if ($sqlprotectagainstexternals) {
			$resql = $dyap_db->query($sqlprotectagainstexternals);
			if ($resql) {
				$num = $dyap_db->num_rows($resql);
				$i = 0;
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);
					if ($dyap_user->socid != $dyap_obj->fk_soc) {
						$accessallowed = 0;
						break;
					}
					$i++;
				}
			}
		}
	}
}

// Security:
// Limit access if permissions are wrong
if (!$accessallowed) {
	accessforbidden();
}

// Security:
// On interdit les remontees de repertoire ainsi que les pipe dans les noms de fichiers.
if (preg_match('/\.\./', $dyap_fullpath_original_file) || preg_match('/[<>|]/', $dyap_fullpath_original_file)) {
	dol_syslog("Refused to deliver file ".$dyap_fullpath_original_file);
	print "ErrorFileNameInvalid: ".dol_escape_htmltag($dyap_original_file);
	exit;
}



if ($dyap_modulepart == 'barcode') {
	$generator = GETPOST("generator", "aZ09");
	$encoding = GETPOST("encoding", "aZ09");
	$readable = GETPOST("readable", 'aZ09') ? GETPOST("readable", "aZ09") : "Y";
	if (in_array($encoding, array('EAN8', 'EAN13'))) {
		$code = GETPOST("code", 'alphanohtml');
	} else {
		$code = GETPOST("code", 'restricthtml'); // This can be rich content (qrcode, datamatrix, ...)
	}

	if (empty($generator) || empty($encoding)) {
		print 'Error: Parameter "generator" or "encoding" not defined';
		exit;
	}

	$dirbarcode = array_merge(array("/core/modules/barcode/doc/"), $dyap_conf->modules_parts['barcode']);

	$result = 0;

	foreach ($dirbarcode as $reldir) {
		$dir = dol_buildpath($reldir, 0);
		$newdir = dol_osencode($dir);

		// Check if directory exists (we do not use dol_is_dir to avoid loading files.lib.php)
		if (!is_dir($newdir)) {
			continue;
		}

		$result = @include_once $newdir.$generator.'.modules.php';
		if ($result) {
			break;
		}
	}

	// Load barcode class
	$classname = "mod".ucfirst($generator);
	$dyap_classname_classname="\\DyaPress\\".$classname;$module = new $dyap_classname_classname($dyap_db);
	if ($module->encodingIsSupported($encoding)) {
		$result = $module->buildBarCode($code, $encoding, $readable);
	}
} else {
	// Open and return file
	clearstatcache();

	$filename = basename($dyap_fullpath_original_file);

	// Output files on browser
	dol_syslog("viewimage.php return file $dyap_fullpath_original_file filename=$filename content-type=$type");

	// This test is to avoid error images when image is not available (for example thumbs).
	if (!dol_is_file($dyap_fullpath_original_file) && empty($_GET["noalt"])) {
		$dyap_fullpath_original_file = DOL_DOCUMENT_ROOT.'/public/theme/common/nophoto.png';
		/*$error='Error: File '.$_GET["file"].' does not exists or filesystems permissions are not allowed';
		print $error;
		exit;*/
	}

	// Permissions are ok and file found, so we return it
	if ($type) {
		top_httphead($type);
		header('Content-Disposition: inline; filename="'.basename($dyap_fullpath_original_file).'"');
	} else {
		top_httphead('image/png');
		header('Content-Disposition: inline; filename="'.basename($dyap_fullpath_original_file).'"');
	}

	$fullpath_original_file_osencoded = dol_osencode($dyap_fullpath_original_file);

	readfile($fullpath_original_file_osencoded);
}


if (is_object($dyap_db)) {
	$dyap_db->close();
}
