<?php
/* Copyright (C) 2005      Marc Barilley / Ocebo <marc@ocebo.com>
 * Copyright (C) 2005-2018 Laurent Destailleur   <eldy@users.sourceforge.net>
 * Copyright (C) 2005-2011 Regis Houssin         <regis.houssin@capnetworks.com>
 * Copyright (C) 2010      Juanjo Menent         <jmenent@2byte.es>
 * Copyright (C) 2015      Raphaël Doursenaud    <rdoursenaud@gpcsolutions.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WordPress DyaPress plugin :
 * Copyright (C) 2016-2019      François Pons        <fpons@aytechnet.fr>
 */

/* DyaPress plugin for WordPress
 *
 * Copyright (c) 2019   François Pons   <fpons@aytechnet.fr>
 *
 * the DyaPress namespace is added in order to avoid conflicts
 * with all the various WordPress extensions.
 */
namespace DyaPress;

defined( 'DYAPRESS_INSTALL_VERSION' ) or die( 'Oops' );

// Needed for DyaPress bootstrap just after installation of files not available previously

require( dirname(__FILE__) . '/dyapress-init.php' );

/*
 * Create $dyap_conf object
 */

require_once DOL_DOCUMENT_ROOT.'/core/class/conf.class.php';

require_once DOL_DOCUMENT_ROOT . '/compta/facture/class/facture.class.php';
require_once DOL_DOCUMENT_ROOT . '/comm/propal/class/propal.class.php';
require_once DOL_DOCUMENT_ROOT . '/contrat/class/contrat.class.php';
require_once DOL_DOCUMENT_ROOT . '/commande/class/commande.class.php';
require_once DOL_DOCUMENT_ROOT . '/fourn/class/fournisseur.commande.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/lib/price.lib.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/menubase.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/lib/files.lib.php';
require_once DOL_DOCUMENT_ROOT . '/core/lib/admin.lib.php';

function dyapress_check_environment() {
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    $dyap_langs->setDefaultLang(get_locale());
    $dyap_langs->load('install');
    $dyap_langs->load("admin");
    $dyap_langs->load("errors");

    $checksok = 2;

    // Add specific CSS
    print '<style type="text/css">
	.dyapress-environment .dashicons-yes:before { color: #00a32a !important }
	.dyapress-environment .dashicons-no:before { color: #d63638 !important }
    </style><div class="dyapress-environment">';

    // Check browser
    $useragent = $_SERVER['HTTP_USER_AGENT'];
    if (!empty($useragent)) {
	$tmp = getBrowserInfo($_SERVER["HTTP_USER_AGENT"]);
	$browserversion = $tmp['browserversion'];
	$browsername = $tmp['browsername'];
	if ($browsername == 'ie' && $browserversion < 7) {
	    print '<b class="dashicons dashicons-no warning">&nbsp;</b>'.$dyap_langs->trans("WarningBrowserTooOld")."<br>\n";
	}
    }


    // Check PHP version
    $arrayphpminversionerror = array(5, 5, 0);
    $arrayphpminversionwarning = array(5, 6, 0);
    if (versioncompare(versionphparray(), $arrayphpminversionerror) < 0) {        // Minimum to use (error if lower)
	print '<p><span class="dashicons dashicons-no error"></span>'.$dyap_langs->trans("ErrorPHPVersionTooLow", versiontostring($arrayphpminversionerror));
	$checksok = 0; // 0=error, 1=warning
    } elseif (versioncompare(versionphparray(), $arrayphpminversionwarning) < 0) {    // Minimum supported (warning if lower)
	print '<p><span class="dashicons dashicons-no error"></span>'.$dyap_langs->trans("ErrorPHPVersionTooLow", versiontostring($arrayphpminversionwarning));
	$checksok = 0; // 0=error, 1=warning
    } else {
	print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPVersion")." ".versiontostring(versionphparray());
    }
    print "</p>\n";


    // Check if session_id is enabled
    if (!function_exists("session_id")) {
	print '<p><span class="dashicons dashicons-no"></span>'.$dyap_langs->trans("ErrorPHPDoesNotSupportSessions")."</p>\n";
	$checksok = 0;
    } else {
	print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPSupportSessions")."</p>\n";
    }


    // Check if GD is supported (we need GD for image conversion)
    if (!function_exists("imagecreate")) {
	$dyap_langs->load("errors");
	print '<p><span class="dashicons dashicons-no"></span>'.$dyap_langs->trans("ErrorPHPDoesNotSupportGD")."</p>\n";
	// $checksok = 0;		// If ko, just warning. So check must still be 1 (otherwise no way to install)
    } else {
	print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPSupport", "GD")."</p>\n";
    }


    // Check if Curl is supported
    if (!function_exists("curl_init")) {
	$dyap_langs->load("errors");
	print '<p><span class="dashicons dashicons-no"></span>'.$dyap_langs->trans("ErrorPHPDoesNotSupportCurl")."</p>\n";
	// $checksok = 0;		// If ko, just warning. So check must still be 1 (otherwise no way to install)
    } else {
	print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPSupport", "Curl")."</p>\n";
    }

    // Check if PHP calendar extension is available
    if (!function_exists("easter_date")) {
	print '<p><span class="dashicons dashicons-no"></span>'.$dyap_langs->trans("ErrorPHPDoesNotSupportCalendar")."</p>\n";
    } else {
	print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPSupport", "Calendar")."</p>\n";
    }


    // Check if UTF8 is supported
    if (!function_exists("utf8_encode")) {
	$dyap_langs->load("errors");
	print '<p><span class="dashicons dashicons-no"></span>'.$dyap_langs->trans("ErrorPHPDoesNotSupportUTF8")."</p>\n";
	// $checksok = 0; // If ko, just warning. So check must still be 1 (otherwise no way to install)
    } else {
	print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPSupport", "UTF8")."</p>\n";
    }

    // Check for mbstring extension
    if (!extension_loaded("mbstring")) {
	$dyap_langs->load("errors");
	print '<p><span class="dashicons dashicons-no"></span>'.$dyap_langs->trans("ErrorPHPDoesNotSupportMbstring")."</p>\n";
	// $checksok = 0; // If ko, just warning. So check must still be 1 (otherwise no way to install)
    } else {
	print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPSupport", "mbstring")."</p>\n";
    }

    // Check if intl methods are supported
    if (empty($_SERVER["SERVER_ADMIN"]) || $_SERVER["SERVER_ADMIN"] != 'doliwamp@localhost') {
	if (!function_exists("locale_get_primary_language") || !function_exists("locale_get_region")) {
	    $dyap_langs->load("errors");
	    print '<p><span class="dashicons dashicons-no"></span>'.$dyap_langs->trans("ErrorPHPDoesNotSupportIntl")."</p>\n";
	    // $checksok = 0;		// If ko, just warning. So check must still be 1 (otherwise no way to install)
	} else {
	    print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPSupport", "Intl")."</p>\n";
	}
    }

    if (!class_exists('ZipArchive')) {
	$dyap_langs->load("errors");
	print '<p><span class="dashicons dashicons-no"></span>'.$dyap_langs->trans("ErrorPHPDoesNotSupport", "ZIP")."</p>\n";
	// $checksok = 0;		// If ko, just warning. So check must still be 1 (otherwise no way to install)
    } else {
	print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPSupport", "ZIP")."</p>\n";
    }

    // Check memory
    $memrequiredorig = '64M';
    $memrequired = 64 * 1024 * 1024;
    $memmaxorig = @ini_get("memory_limit");
    $memmax = @ini_get("memory_limit");
    if ($memmaxorig != '') {
	preg_match('/([0-9]+)([a-zA-Z]*)/i', $memmax, $reg);
	if ($reg[2]) {
	    if (strtoupper($reg[2]) == 'G') {
		$memmax = $reg[1] * 1024 * 1024 * 1024;
	    }
	    if (strtoupper($reg[2]) == 'M') {
		$memmax = $reg[1] * 1024 * 1024;
	    }
	    if (strtoupper($reg[2]) == 'K') {
		$memmax = $reg[1] * 1024;
	    }
	}
	if ($memmax >= $memrequired || $memmax == -1) {
	    print '<p><span class="dashicons dashicons-yes"></span>'.$dyap_langs->trans("PHPMemoryOK", $memmaxorig, $memrequiredorig)."</p>\n";
	} else {
	    print '<p><span class="dashicons dashicons-no"></span>'.$dyap_langs->trans("PHPMemoryTooLow", $memmaxorig, $memrequiredorig)."</p>\n";
	}
    }

    print "</div>\n";

    return $checksok;
}

function dyapress_install() {
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    $dyap_langs->setDefaultLang(get_locale());
    $dyap_langs->load("admin");
    $dyap_langs->load("install");
    $dyap_langs->load("errors");

    if (dyapress_install_step1() &&
	dyapress_install_step2() &&
	dyapress_install_step5()) {

	print '<div class="update-message inline notice notice-alt updated-message notice-success"><p>'.$dyap_langs->trans('SystemIsInstalled').'</p></div>';
    }
}

function dyapress_install_step1() {
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    dolibarr_install_syslog("--- step1: begin");
    $main_dir = DOL_DOCUMENT_ROOT;
    $main_data_dir = DOL_DATA_ROOT;

    // Create directory for documents
    if (!is_dir($main_data_dir)) {
	dol_mkdir($main_data_dir);
    }

    if (!is_dir($main_data_dir)) {
	print '<div class="update-message inline notice notice-alt notice-error"><p>'.$dyap_langs->trans("ErrorDirDoesNotExists", $main_data_dir)." ".$dyap_langs->trans("YouMustCreateItAndAllowServerToWrite").'</p></div>';
	$error++;
    } else {
	// Create .htaccess file in document directory
	$pathhtaccess = $main_data_dir.'/.htaccess';
	if (!file_exists($pathhtaccess)) {
	    dolibarr_install_syslog("step1: .htaccess file did not exist, we created it in '".$main_data_dir."'");
	    $handlehtaccess = @fopen($pathhtaccess, 'w');
	    if ($handlehtaccess) {
		fwrite($handlehtaccess, 'Order allow,deny'."\n");
		fwrite($handlehtaccess, 'Deny from all'."\n");

		fclose($handlehtaccess);
		dolibarr_install_syslog("step1: .htaccess file created");
	    }
	}

	// Documents are stored above the web pages root to prevent being downloaded without authentification
	$dir = array();
	$dir[] = $main_data_dir."/mycompany";
	$dir[] = $main_data_dir."/medias";
	$dir[] = $main_data_dir."/users";
	$dir[] = $main_data_dir."/facture";
	$dir[] = $main_data_dir."/propale";
	$dir[] = $main_data_dir."/ficheinter";
	$dir[] = $main_data_dir."/produit";
	$dir[] = $main_data_dir."/doctemplates";

	// Loop on each directory of dir [] to create them if they do not exist
	$num = count($dir);
	for ($i = 0; $i < $num; $i++) {
	    if (is_dir($dir[$i])) {
		dolibarr_install_syslog("step1: directory '".$dir[$i]."' exists");
	    } else {
		if (dol_mkdir($dir[$i]) < 0) {
		    print '<div class="update-message inline notice notice-alt notice-error"><p>Failed to create directory: '.$dir[$i].'</p></div>';
		    $error++;
		} else {
		    dolibarr_install_syslog("step1: directory '".$dir[$i]."' created");
		}
	    }
	}

	require_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';

	// Copy directory medias
	$srcroot = $main_dir.'/install/medias';
	$destroot = $main_data_dir.'/medias';
	dolCopyDir($srcroot, $destroot, 0, 0);

	if ($error) {
	    print '<div class="update-message inline notice notice-alt notice-error"><p>'.$dyap_langs->trans('ErrorDirDoesNotExists').' '.$dyap_langs->trans("YouMustCreateItAndAllowServerToWrite").'</p>';
	    print '<p>'.$dyap_langs->trans("CorrectProblemAndReloadPage", \dyap_self().'&testget=ok').'</p></div>';
	} else {
	    //ODT templates
	    $srcroot = $main_dir.'/install/doctemplates';
	    $destroot = $main_data_dir.'/doctemplates';
	    $docs = array(
		'contracts' => 'contract',
		'invoices' => 'invoice',
		'orders' => 'order',
		'products' => 'product',
		'projects' => 'project',
		'proposals' => 'proposal',
		'shipments' => 'shipment',
		'supplier_proposals' => 'supplier_proposal',
		'tasks' => 'task_summary',
		'thirdparties' => 'thirdparty',
		'usergroups' => 'usergroups',
		'users' => 'user',
	    );
	    foreach ($docs as $cursordir => $cursorfile) {
		$src = $srcroot.'/'.$cursordir.'/template_'.$cursorfile.'.odt';
		$dirodt = $destroot.'/'.$cursordir;
		$dest = $dirodt.'/template_'.$cursorfile.'.odt';

		dol_mkdir($dirodt);
		$result = dol_copy($src, $dest, 0, 0);
		if ($result < 0) {
		    print '<div class="update-message inline notice notice-alt notice-error"><p>'.$dyap_langs->trans('ErrorFailToCopyFile',$src,$dest).'</p></div>';
		}
	    }
	}
    }
    dolibarr_install_syslog("--- step1: end");

    return !$error;
}

function dyapress_install_step2() {
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    dolibarr_install_syslog("--- step2: begin");
    $choix=1;
    // choix: 1:mysqli 2:pgsql 3:mssql 4:sqlite 5:sqlite3

    // To say sql requests are escaped for mysql so we need to unescape them
    $dyap_db->unescapeslashquot=true;

    $ok = 1;

    /**************************************************************************************
     *
     * Chargement fichiers tables/*.sql (non *.key.sql)
     * A faire avant les fichiers *.key.sql
     *
     ***************************************************************************************/
    if ($ok) {
	// We always choose in mysql directory (Conversion is done by driver to translate SQL syntax)
	$dir = DOL_DOCUMENT_ROOT . "/install/mysql/tables/";

	$ok = 0;
	$handle=opendir($dir);
	dolibarr_install_syslog("step2: open tables directory " . $dir . " handle=" . $handle);
	$tablefound = 0;
	$tabledata=array();
	if (is_resource($handle)) {
	    while (($dyap_file = readdir($handle)) !== false) {
		if (preg_match('/\.sql$/i', $dyap_file) && preg_match('/^llx_/i', $dyap_file) && !preg_match('/\.key\.sql$/i', $dyap_file) && !preg_match('/\-/', $dyap_file)) {
		    $tablefound++;
		    $tabledata[] = $dyap_file;
		}
	    }
	    closedir($handle);
	}

	// Sort list of sql files on alphabetical order (load order is important)
	sort($tabledata);
	foreach ($tabledata as $dyap_file) {
	    $name = substr($dyap_file, 0, dol_strlen($dyap_file) - 4);
	    $buffer = '';
	    $fp = fopen($dir.$dyap_file, "r");
	    if ($fp) {
		while (!feof($fp)) {
		    $buf = fgets($fp, 4096);
		    if (substr($buf, 0, 2) <> '--') {
			$buf = preg_replace('/--(.+)*/', '', $buf);
			$buffer .= $buf;
		    }
		}
		fclose($fp);

		$buffer = trim($buffer);
		if ($dyap_conf->db->type == 'mysql' || $dyap_conf->db->type == 'mysqli') {	// For Mysql 5.5+, we must replace type=innodb with ENGINE=innodb
		    $buffer = preg_replace('/type=innodb/i', 'ENGINE=innodb', $buffer);
		} else {
		    // Keyword ENGINE is MySQL-specific, so scrub it for
		    // other database types (mssql, pgsql)
		    $buffer = preg_replace('/type=innodb/i', '', $buffer);
		    $buffer = preg_replace('/ENGINE=innodb/i', '', $buffer);
		}

		// Replace the prefix tables
		if (MAIN_DB_PREFIX != 'llx_') {
		    $buffer = preg_replace('/llx_/i', MAIN_DB_PREFIX, $buffer);
		}

		//print "<tr><td>Creation de la table $name/td>";
		$requestnb++;

		dolibarr_install_syslog("step2: request: ".$buffer);
		$resql = $dyap_db->query($buffer, 0, 'dml');
		if ($resql) {
		    // print "<td>OK requete ==== $buffer</td></tr>";
		    $dyap_db->free($resql);
		} else {
		    if ($dyap_db->errno() == 'DB_ERROR_TABLE_ALREADY_EXISTS' ||
			$dyap_db->errno() == 'DB_ERROR_TABLE_OR_KEY_ALREADY_EXISTS') {
			//print "<td>Deja existante</td></tr>";
		    } else {
			print '<div class="update-message inline notice notice-alt notice-error"><p>';
			print $dyap_langs->trans("CreateTableAndPrimaryKey",$name);
			print ' (<i>'.$dyap_langs->trans("Request").' '.$requestnb.' : '.$buffer.' <br>Executed query : '.$dyap_db->lastquery;
			print '; '.$dyap_langs->trans("ErrorSQL")." ".$dyap_db->errno()." ".$dyap_db->error().'</i>)';
			print '</p></div>';
			$error++;
		    }
		}
	    } else {
		print '<div class="update-message inline notice notice-alt notice-error"><p>';
		print $dyap_langs->trans("CreateTableAndPrimaryKey",$name);
		print ' (<i>'.$dyap_langs->trans("Error").' Failed to open file '.$dir.$file.'</i>)';
		print '</p></div>';
		$error++;
		dolibarr_install_syslog("step2: failed to open file ".$dir.$dyap_file, LOG_ERR);
	    }
	}

	if ($tablefound) {
	    if ($error == 0) {
		$ok = 1;
	    }
	} else {
	    dolibarr_install_syslog("step2: failed to find files to create database in directory " . $dir, LOG_ERR);
	}
    }


    /***************************************************************************************
     *
     * Chargement fichiers tables/*.key.sql
     * A faire apres les fichiers *.sql
     *
     ***************************************************************************************/
    if ($ok) {
	// We always choose in mysql directory (Conversion is done by driver to translate SQL syntax)
	$dir = DOL_DOCUMENT_ROOT . "/install/mysql/tables/";

	$okkeys = 0;
	$handle=opendir($dir);
	dolibarr_install_syslog("step2: open keys directory " . $dir . " handle=" . $handle);
	$tablefound = 0;
	$tabledata=array();
	if (is_resource($handle))
	{
	    while (($dyap_file = readdir($handle)) !== false) {
		if (preg_match('/\.sql$/i', $dyap_file) && preg_match('/^llx_/i', $dyap_file) && preg_match('/\.key\.sql$/i', $dyap_file) && !preg_match('/\-/', $dyap_file)) {
		    $tablefound++;
		    $tabledata[]=$dyap_file;
		}
	    }
	    closedir($handle);
	}

	// Sort list of sql files on alphabetical order (load order is important)
	sort($tabledata);
	foreach($tabledata as $file)
	{
	    $name = substr($file, 0, dol_strlen($file) - 4);
	    //print "<tr><td>Creation de la table $name</td>";
	    $buffer = '';
	    $fp = fopen($dir.$file,"r");
	    if ($fp)
	    {
		while (!feof($fp))
		{
		    $buf = fgets($fp, 4096);

		    // Cas special de lignes autorisees pour certaines versions uniquement
		    if ($choix == 1 && preg_match('/^--\sV([0-9\.]+)/i',$buf,$reg))
		    {
			$versioncommande=explode('.',$reg[1]);
			if (count($versioncommande) && count($versionarray)
			    && versioncompare($versioncommande,$versionarray) <= 0)
			{
			    // Version qualified, delete SQL comments
			    $buf=preg_replace('/^--\sV([0-9\.]+)/i','',$buf);
			}
		    }
		    if ($choix == 2 && preg_match('/^--\sPOSTGRESQL\sV([0-9\.]+)/i',$buf,$reg))
		    {
			$versioncommande=explode('.',$reg[1]);
			if (count($versioncommande) && count($versionarray)
			    && versioncompare($versioncommande,$versionarray) <= 0)
			{
			    // Version qualified, delete SQL comments
			    $buf=preg_replace('/^--\sPOSTGRESQL\sV([0-9\.]+)/i','',$buf);
			}
		    }

		    // Ajout ligne si non commentaire
		    if (! preg_match('/^--/i',$buf)) $buffer .= $buf;
		}
		fclose($fp);

		// Si plusieurs requetes, on boucle sur chaque
		$listesql=explode(';',$buffer);
		foreach ($listesql as $req)
		{
		    $buffer=trim($req);
		    if ($buffer)
		    {
			// Replace the prefix tables
			if (MAIN_DB_PREFIX != 'llx_') {
			    $buffer=preg_replace('/llx_/i',MAIN_DB_PREFIX,$buffer);
			}

			//print "<tr><td>Creation des cles et index de la table $name: '$buffer'</td>";
			$requestnb++;

			dolibarr_install_syslog("step2: request: " . $buffer);
			$resql=$dyap_db->query($buffer,0,'dml');
			if ($resql) {
			    //print "<td>OK requete ==== $buffer</td></tr>";
			    $dyap_db->free($resql);
			} else {
			    if ($dyap_db->errno() == 'DB_ERROR_KEY_NAME_ALREADY_EXISTS' ||
				$dyap_db->errno() == 'DB_ERROR_CANNOT_CREATE' ||
				$dyap_db->errno() == 'DB_ERROR_PRIMARY_KEY_ALREADY_EXISTS' ||
				$dyap_db->errno() == 'DB_ERROR_TABLE_OR_KEY_ALREADY_EXISTS' ||
				preg_match('/duplicate key name/i',$dyap_db->error()))
			    {
				//print "<td>Deja existante</td></tr>";
				$key_exists = 1;
			    } else {
				print '<div class="update-message inline notice notice-alt notice-error"><p>';
				print $dyap_langs->trans("CreateOtherKeysForTable",$name).' (<i>';
				print $dyap_langs->trans("Request").' '.$requestnb.' : '.$dyap_db->lastqueryerror();
				print '; '.$dyap_langs->trans("ErrorSQL")." ".$dyap_db->errno()." ".$dyap_db->error().'</i>)';
				print '</p></div>';
				$error++;
			    }
			}
		    }
		}
	    } else {
		print '<div class="update-message inline notice notice-alt notice-error"><p>';
		print $dyap_langs->trans("CreateOtherKeysForTable",$name);
		print ' (<i>'.$dyap_langs->trans("Error")." Failed to open file ".$dir.$file.'</i>)';
		print '</p></div>';
		$error++;
		dolibarr_install_syslog("step2: failed to open file " . $dir . $file, LOG_ERR);
	    }
	}

	if ($tablefound && $error == 0) {
	    $okkeys = 1;
	}
    }


    /***************************************************************************************
     *
     * Chargement fichier functions.sql
     *
     ***************************************************************************************/
    if ($ok) {
	$dir = DOL_DOCUMENT_ROOT . '/install/';
	// For this file, we use directory according to database type
	if ($choix == 1) {
	    $dir .= "mysql/functions/";
	} elseif ($choix == 2) {
	    $dir .= "pgsql/functions/";
	} elseif ($choix == 3) {
	    $dir .= "mssql/functions/";
	} elseif ($choix == 4) {
	    $dir .= "sqlite3/functions/";
	}

	// Creation donnees
	$dyap_file = "functions.sql";
	if (file_exists($dir.$dyap_file)) {
	    $fp = fopen($dir.$dyap_file, "r");
	    dolibarr_install_syslog("step2: open function file ".$dir.$dyap_file." handle=".$fp);
	    if ($fp) {
                $buffer = '';
                while (!feof($fp)) {
                    $buf = fgets($fp, 4096);
                    if (substr($buf, 0, 2) <> '--') {
                        $buffer .= $buf."§";
                    }
                }
                fclose($fp);
            }
            //$buffer=preg_replace('/;\';/',";'§",$buffer);

            // If several requests, we loop on each of them
            $listesql=explode('§',$buffer);
            foreach ($listesql as $buffer) {
		$buffer = trim($buffer);
                if ($buffer) {
                    dolibarr_install_syslog("step2: request: " . $buffer);
                    print "<!-- Insert line : ".$buffer."<br>-->\n";
                    $resql=$dyap_db->query($buffer,0,'dml');
                    if ($resql) {
                        $ok = 1;
                        $dyap_db->free($resql);
                    } else {
                        if ($dyap_db->errno() == 'DB_ERROR_RECORD_ALREADY_EXISTS'
                        || $dyap_db->errno() == 'DB_ERROR_KEY_NAME_ALREADY_EXISTS') {
                            //print "Insert line : ".$buffer."<br>\n";
                        } else {
                            $ok = 0;

			    print '<div class="update-message inline notice notice-alt notice-error"><p>';
                            print $dyap_langs->trans("FunctionsCreation");
                            print ' (<i>'.$dyap_langs->trans("Request").' '.$requestnb.' : '.$buffer;
                            print '; '.$dyap_langs->trans("ErrorSQL")." ".$dyap_db->errno()." ".$dyap_db->error().'</i>)';
			    print '</p></div>';
                            $error++;
                        }
                    }
                }
            }

	    $ok = 1 ;
        }
    }


    /***************************************************************************************
     *
     * Load files data/*.sql
     *
     ***************************************************************************************/
    if ($ok) {
        // We always choose in mysql directory (Conversion is done by driver to translate SQL syntax)
        $dir = DOL_DOCUMENT_ROOT . "/install/mysql/data/";

        // Insert data
        $handle=opendir($dir);
        dolibarr_install_syslog("step2: open directory data " . $dir . " handle=" . $handle);
        $tablefound = 0;
        $tabledata=array();
	if (is_resource($handle)) {
	    while (($dyap_file = readdir($handle)) !== false) {
		if (preg_match('/\.sql$/i', $dyap_file) && preg_match('/^llx_/i', $dyap_file) && !preg_match('/\-/', $dyap_file)) {
		    if (preg_match('/^llx_accounting_account_/', $dyap_file)) {
			continue; // We discard data file of chart of account. Will be loaded when a chart is selected.
		    }

		    //print 'x'.$dyap_file.'-'.$createdata.'<br>';
		    if (is_numeric($createdata) || preg_match('/'.preg_quote($createdata).'/i', $dyap_file)) {
			$tablefound++;
			$tabledata[] = $dyap_file;
		    }
                }
            }
            closedir($handle);
        }

        // Sort list of data files on alphabetical order (load order is important)
        sort($tabledata);
	foreach ($tabledata as $dyap_file) {
	    $name = substr($dyap_file, 0, dol_strlen($dyap_file) - 4);
	    $fp = fopen($dir.$dyap_file, "r");
	    dolibarr_install_syslog("step2: open data file ".$dir.$dyap_file." handle=".$fp);
	    if ($fp) {
                $arrayofrequests=array();
                $linefound=0;
                $linegroup=0;
                $sizeofgroup=1; // Grouping request to have 1 query for several requests does not works with mysql, so we use 1.

                // Load all requests
                while (!feof($fp)) {
                    $buffer = fgets($fp, 4096);
                    $buffer = trim($buffer);
                    if ($buffer) {
			if (substr($buffer, 0, 2) == '--') {
			    continue;
			}

			if ($linefound && ($linefound % $sizeofgroup) == 0) {
			    $linegroup++;
			}
			if (empty($arrayofrequests[$linegroup])) {
			    $arrayofrequests[$linegroup] = $buffer;
			} else {
			    $arrayofrequests[$linegroup] .= " ".$buffer;
			}

			$linefound++;
                    }
                }
                fclose($fp);

                dolibarr_install_syslog("step2: found " . $linefound . " records, defined " . count($arrayofrequests) . " group(s).");

                $okallfile=1;
                $dyap_db->begin();

                // We loop on each requests of file
                foreach($arrayofrequests as $buffer)
		{
		    // Replace the prefix tables
		    if (MAIN_DB_PREFIX != 'llx_')
		    {
			$buffer=preg_replace('/llx_/i',MAIN_DB_PREFIX,$buffer);
		    }

		    //dolibarr_install_syslog("step2: request: " . $buffer);
		    $resql=$dyap_db->query($buffer,1);
		    if ($resql)
		    {
			//$dyap_db->free($resql);     // Not required as request we launch here does not return memory needs.
		    }
		    else
		    {
			if ($dyap_db->lasterrno() == 'DB_ERROR_RECORD_ALREADY_EXISTS')
			{
			    //print "<tr><td>Insertion ligne : $buffer</td><td>";
			}
			else
			{
			    $ok = 0;
			    $okallfile = 0;
			    print '<div class="update-message inline notice notice-alt notice-error"><p>';
			    print $dyap_langs->trans("ErrorSQL")." : ".$dyap_db->lasterrno()." - ".$dyap_db->lastqueryerror()." - ".$dyap_db->lasterror();
			    print '</p></div>';
			}
		    }
		}

		if ($okallfile) $dyap_db->commit();
		else $dyap_db->rollback();
	    }
	}

	$ok = 1;    // Data loading are not blocking errors
    }

    dolibarr_install_syslog("--- step2: end");

    return $ok;
}

function dyapress_install_step5()
{
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    dolibarr_install_syslog("--- step5: begin");

    $targetversion = DOL_VERSION;
    $success = 0;

    // Active module user
    $modName='modUser';
    $file = $modName . ".class.php";
    dolibarr_install_syslog('step5: load module user ' . DOL_DOCUMENT_ROOT . "/core/modules/" . $file, LOG_INFO);
    include_once DOL_DOCUMENT_ROOT ."/core/modules/".$file;
    $dyap_classname_modName="\\DyaPress\\".$modName;$objMod = new $dyap_classname_modName($dyap_db);
    $result=$objMod->init();
    if (! $result) print 'ERROR in activating module file='.$file;

    if ($dyap_db->connected) {
	$dyap_conf->setValues($dyap_db);
	// Reset forced setup after the setValues
	if (defined('SYSLOG_FILE')) {
	    $dyap_conf->global->SYSLOG_FILE = constant('SYSLOG_FILE');
	}
	$dyap_conf->global->MAIN_ENABLE_LOG_TO_HTML = 1;

	// Create admin user
	include_once DOL_DOCUMENT_ROOT.'/user/class/user.class.php';

	// Set default encryption to yes, generate a salt and set default encryption algorythm (but only if there is no user yet into database)
	$sql = "SELECT u.rowid, u.pass, u.pass_crypted";
	$sql .= " FROM ".MAIN_DB_PREFIX."user as u";
	$resql = $dyap_db->query($sql);
	if ($resql) {
	    $numrows = $dyap_db->num_rows($resql);
	    if ($numrows == 0) {
		// Define default setup for password encryption
		dolibarr_set_const($dyap_db, "DATABASE_PWD_ENCRYPTED", "1", 'chaine', 0, '', $dyap_conf->entity);
		dolibarr_set_const($dyap_db, "MAIN_SECURITY_SALT", dol_print_date(dol_now(), 'dayhourlog'), 'chaine', 0, '', 0); // All entities
		if (function_exists('password_hash')) {
		    dolibarr_set_const($dyap_db, "MAIN_SECURITY_HASH_ALGO", 'password_hash', 'chaine', 0, '', 0); // All entities
		} else {
		    dolibarr_set_const($dyap_db, "MAIN_SECURITY_HASH_ALGO", 'sha1md5', 'chaine', 0, '', 0); // All entities
		}
	    }

	    dolibarr_install_syslog('step5: DATABASE_PWD_ENCRYPTED = '.$dyap_conf->global->DATABASE_PWD_ENCRYPTED.' MAIN_SECURITY_HASH_ALGO = '.$dyap_conf->global->MAIN_SECURITY_HASH_ALGO, LOG_INFO);
	}

	// Create user used to create the admin user

	$createuser=new User($dyap_db);
	$createuser->id=0;
	$createuser->admin=1;

	// Set admin user
	$current_user = wp_get_current_user();

	$newuser = new User($dyap_db);
	$newuser->lastname=$current_user->user_lastname;
	$newuser->firstname=$current_user->user_firstname;
	$newuser->login = $current_user->user_login;
	$newuser->email = $current_user->user_email;
	$newuser->admin=1;
	$newuser->entity=0;

	$result=$newuser->create($createuser,1);
	if ($result > 0) {
	    $success = 1;
	} else {
	    if ($newuser->error == 'ErrorLoginAlreadyExists') {
		dolibarr_install_syslog('step5: AdminLoginAlreadyExists', LOG_WARNING);
		print '<div class="update-message inline notice notice-alt notice-error"><p>';
		print $dyap_langs->trans("AdminLoginAlreadyExists", $current_user->user_login) . "</p></div>";
		$success = 1;
	    } else {
		dolibarr_install_syslog('step5: FailedToCreateAdminLogin ' . $newuser->error, LOG_ERR);
		print '<div class="update-message inline notice notice-alt notice-error"><p>';
		print $dyap_langs->trans("FailedToCreateAdminLogin").' '.$newuser->error.'</p></div>';
	    }
	}

	if ($success) {
	    // Insert MAIN_VERSION_FIRST_INSTALL in a dedicated transaction. So if it fails (when first install was already done), we can do other following requests.
	    $dyap_db->begin();
	    dolibarr_install_syslog('step5: set MAIN_VERSION_FIRST_INSTALL const to '.$targetversion, LOG_DEBUG);
	    $resql = $dyap_db->query("INSERT INTO ".MAIN_DB_PREFIX."const(name, value, type, visible, note, entity) values(".$dyap_db->encrypt('MAIN_VERSION_FIRST_INSTALL').", ".$dyap_db->encrypt($targetversion).", 'chaine', 0, 'Dolibarr version when first install', 0)");
	    if ($resql) {
		$dyap_conf->global->MAIN_VERSION_FIRST_INSTALL = $targetversion;
		$dyap_db->commit();
	    } else {
		//if (! $resql) dol_print_error($dyap_db,'Error in setup program');      // We ignore errors. Key may already exists
		$dyap_db->commit();
	    }

	    $dyap_db->begin();

	    dolibarr_install_syslog('step5: set MAIN_VERSION_LAST_INSTALL const to '.$targetversion, LOG_DEBUG);
	    $resql = $dyap_db->query("DELETE FROM ".MAIN_DB_PREFIX."const WHERE ".$dyap_db->decrypt('name')." = 'MAIN_VERSION_LAST_INSTALL'");
	    if (!$resql) {
		dol_print_error($dyap_db, 'Error in setup program');
	    }
	    $resql = $dyap_db->query("INSERT INTO ".MAIN_DB_PREFIX."const(name,value,type,visible,note,entity) values(".$dyap_db->encrypt('MAIN_VERSION_LAST_INSTALL').", ".$dyap_db->encrypt($targetversion).", 'chaine', 0, 'Dolibarr version when last install', 0)");
	    if (!$resql) {
		dol_print_error($dyap_db, 'Error in setup program');
	    }
	    $dyap_conf->global->MAIN_VERSION_LAST_INSTALL = $targetversion;

	    if ($useforcedwizard) {
		dolibarr_install_syslog('step5: set MAIN_REMOVE_INSTALL_WARNING const to 1', LOG_DEBUG);
		$resql = $dyap_db->query("DELETE FROM ".MAIN_DB_PREFIX."const WHERE ".$dyap_db->decrypt('name')." = 'MAIN_REMOVE_INSTALL_WARNING'");
		if (!$resql) {
		    dol_print_error($dyap_db, 'Error in setup program');
		}
		$resql = $dyap_db->query("INSERT INTO ".MAIN_DB_PREFIX."const(name,value,type,visible,note,entity) values(".$dyap_db->encrypt('MAIN_REMOVE_INSTALL_WARNING').", ".$dyap_db->encrypt(1).", 'chaine', 1, 'Disable install warnings', 0)");
		if (!$resql) {
		    dol_print_error($dyap_db, 'Error in setup program');
		}
		$dyap_conf->global->MAIN_REMOVE_INSTALL_WARNING = 1;
	    }

	    // If we ask to force some modules to be enabled
	    if (!empty($force_install_module)) {
		if (!defined('DOL_DOCUMENT_ROOT') && !empty($dolibarr_main_document_root)) {
		    define('DOL_DOCUMENT_ROOT', $dolibarr_main_document_root);
		}

		$tmparray = explode(',', $force_install_module);
		foreach ($tmparray as $modtoactivate) {
		    $modtoactivatenew = preg_replace('/\.class\.php$/i', '', $modtoactivate);
		    print $dyap_langs->trans("ActivateModule", $modtoactivatenew).'<br>';

		    $dyap_file = $modtoactivatenew.'.class.php';
		    dolibarr_install_syslog('step5: activate module file='.$dyap_file);
		    $res = dol_include_once("/core/modules/".$dyap_file);

		    $res = activateModule($modtoactivatenew, 1);
		    if (!empty($res['errors'])) {
			print 'ERROR: failed to activateModule() file='.$dyap_file;
		    }
		}
	    }

	    dolibarr_install_syslog('step5: remove MAIN_NOT_INSTALLED const');
	    $resql = $dyap_db->query("DELETE FROM ".MAIN_DB_PREFIX."const WHERE ".$dyap_db->decrypt('name')."='MAIN_NOT_INSTALLED'");
	    if (!$resql) {
		dol_print_error($dyap_db, 'Error in setup program');
	    }

	    $dyap_db->commit();
	}
	else
	{
	    print $dyap_langs->trans("ErrorFailedToConnect")."<br>";
	}
    }

    // May fail if parameter already defined
    $resql = $dyap_db->query("INSERT INTO ".MAIN_DB_PREFIX."const(name,value,type,visible,note,entity) VALUES (".$dyap_db->encrypt('MAIN_LANG_DEFAULT').", ".$dyap_db->encrypt("auto").", 'chaine', 0, 'Default language', 1)");
    //if (! $resql) dol_print_error($dyap_db,'Error in setup program');

    $dyap_db->close();

    dolibarr_install_syslog("--- step5: end");

    return $success;
}

function dyapress_upgrade() {
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    // Cette page peut etre longue. On augmente le delai autorise.
    // Ne fonctionne que si on est pas en safe_mode.
    $err = error_reporting();
    error_reporting(0);
    if (!empty($dyap_conf->global->MAIN_OVERRIDE_TIME_LIMIT)) {
	@set_time_limit((int) $dyap_conf->global->MAIN_OVERRIDE_TIME_LIMIT);
    } else {
	@set_time_limit(600);
    }

    // la langue doit deja etre choisie
    $dyap_langs->load("admin");
    $dyap_langs->load("install");
    $dyap_langs->load("other");
    $dyap_langs->load("errors");

    if ( !dyapress_check_fix_sql() )
	return false;

    // Add specific CSS
    print '<style type="text/css">
	.dyapress-environment .dashicons-yes:before { color: #00a32a !important }
	.dyapress-environment .dashicons-no:before { color: #d63638 !important }
    </style><div class="dyapress-environment">';

    print '<div class="inline"><p><strong>'.$dyap_langs->trans('DatabaseMigration').'</strong></p></div>'."\n";

    // On cherche les version de mise a jour a appliquer
    $migrationscript=array(
	array('from'=>'3.0.0', 'to'=>'3.1.0'),
	array('from'=>'3.1.0', 'to'=>'3.2.0'),
	array('from'=>'3.2.0', 'to'=>'3.3.0'),
	array('from'=>'3.3.0', 'to'=>'3.4.0'),
	array('from'=>'3.4.0', 'to'=>'3.5.0'),
	array('from'=>'3.5.0', 'to'=>'3.6.0'),
	array('from'=>'3.6.0', 'to'=>'3.7.0'),
	array('from'=>'3.7.0', 'to'=>'3.8.0'),
	array('from'=>'3.8.0', 'to'=>'3.9.0'),
	array('from'=>'3.9.0', 'to'=>'4.0.0'),
	array('from'=>'4.0.0', 'to'=>'5.0.0'),
	array('from'=>'5.0.0', 'to'=>'6.0.0'),
	array('from'=>'6.0.0', 'to'=>'7.0.0'),
	array('from'=>'7.0.0', 'to'=>'8.0.0'),
	array('from'=>'8.0.0', 'to'=>'9.0.0'),
	array('from'=>'9.0.0', 'to'=>'10.0.0'),
	array('from'=>'10.0.0', 'to'=>'11.0.0'),
	array('from'=>'11.0.0', 'to'=>'12.0.0'),
	array('from'=>'12.0.0', 'to'=>'13.0.0'),
	array('from'=>'13.0.0', 'to'=>'14.0.0'),
	array('from'=>'14.0.0', 'to'=>'15.0.0'),
	array('from'=>'15.0.0', 'to'=>'16.0.0'),
    );

    if ((empty($dyap_conf->global->MAIN_VERSION_LAST_UPGRADE) && empty($dyap_conf->global->MAIN_VERSION_LAST_INSTALL)) && isset($dyap_conf->global->MAIN_MONNAIE))
	$dyap_conf->global->MAIN_VERSION_LAST_INSTALL = '8.0.2'; // FIXME: only 8.0.2 has been released with both MAIN_VERSION_LAST_UPGRADE and MAIN_VERSION_LAST_INSTALL empty

    $count=0;
    foreach ($migrationscript as $migarray) {
	$dolibarrlastupgradeversionarray=preg_split('/[\.-]/',isset($dyap_conf->global->MAIN_VERSION_LAST_UPGRADE) ? $dyap_conf->global->MAIN_VERSION_LAST_UPGRADE : (isset($dyap_conf->global->MAIN_VERSION_LAST_INSTALL)?$dyap_conf->global->MAIN_VERSION_LAST_INSTALL:''));

	$count++;
	$version=DOL_VERSION;
	$versionfrom=$migarray['from'];
	$versionto=$migarray['to'];
	$versionarray=preg_split('/[\.-]/',$version);
	$dolibarrversionfromarray=preg_split('/[\.-]/',$versionfrom);
	$dolibarrversiontoarray=preg_split('/[\.-]/',$versionto);
	// Define string newversionxxx that are used for text to show
	$newversionfrom=preg_replace('/(\.[0-9]+)$/i','.*',$versionfrom);
	$newversionto=preg_replace('/(\.[0-9]+)$/i','.*',$versionto);
	$newversionfrombis='';

	if (count($dolibarrlastupgradeversionarray) >= 2) { // If a database access is available and last upgrade version is known
	    // Now we check if this is the first qualified choice
	    if (empty($foundrecommandedchoice) && (versioncompare($dolibarrversiontoarray,$dolibarrlastupgradeversionarray) > 0 || versioncompare($dolibarrversiontoarray,$versionarray) < -2)) {
		dyapress_upgrade_step1($versionfrom, $versionto);
	    }
	}
    }

    dyapress_upgrade_step2($versionfrom, $versionto);

    dyapress_upgrade_finish();
}

function dyapress_upgrade_finish() {
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    $targetversion = DOL_VERSION;

    $dyap_conf->setValues($dyap_db);
    // Reset forced setup after the setValues
    if (defined('SYSLOG_FILE')) {
	$dyap_conf->global->SYSLOG_FILE = constant('SYSLOG_FILE');
    }
    $dyap_conf->global->MAIN_ENABLE_LOG_TO_HTML = 1;

    // Define if we need to update the MAIN_VERSION_LAST_UPGRADE value in database
    $tagdatabase = false;
    if (empty($dyap_conf->global->MAIN_VERSION_LAST_UPGRADE)) {
	$tagdatabase = true; // We don't know what it was before, so now we consider we are version choosed.
    } else {
	$mainversionlastupgradearray = preg_split('/[.-]/', $dyap_conf->global->MAIN_VERSION_LAST_UPGRADE);
	$targetversionarray = preg_split('/[.-]/', $targetversion);
	if (versioncompare($targetversionarray, $mainversionlastupgradearray) > 0) {
	    $tagdatabase = true;
	}
    }

    if ($tagdatabase) {
	dolibarr_install_syslog('step5: set MAIN_VERSION_LAST_UPGRADE const to value '.$targetversion);
	$resql = $dyap_db->query("DELETE FROM ".MAIN_DB_PREFIX."const WHERE ".$dyap_db->decrypt('name')." = 'MAIN_VERSION_LAST_UPGRADE'");
	if (!$resql) {
	    dol_print_error($dyap_db, 'Error in setup program');
	}
	$resql = $dyap_db->query("INSERT INTO ".MAIN_DB_PREFIX."const(name, value, type, visible, note, entity) VALUES (".$dyap_db->encrypt('MAIN_VERSION_LAST_UPGRADE').", ".$dyap_db->encrypt($targetversion).", 'chaine', 0, 'Dolibarr version for last upgrade', 0)");
	if (!$resql) {
	    dol_print_error($dyap_db, 'Error in setup program');
	}
	$dyap_conf->global->MAIN_VERSION_LAST_UPGRADE = $targetversion;
    } else {
	dolibarr_install_syslog('step5: we run an upgrade to version '.$targetversion.' but database was already upgraded to '.$dyap_conf->global->MAIN_VERSION_LAST_UPGRADE.'. We keep MAIN_VERSION_LAST_UPGRADE as it is.');
    }
}

function dyapress_check_fix_sql() {
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    // Test database version is not forbidden for migration
    if (empty($ignoredbversion))
    {
	$dbversion_disallowed=array(
	    array('type'=>'mysql','version'=>array(5,5,40)),
	    array('type'=>'mysqli','version'=>array(5,5,40)) //,
	    //array('type'=>'mysql','version'=>array(5,5,41)),
	    //array('type'=>'mysqli','version'=>array(5,5,41))
	);
	$listofforbiddenversion='';
	foreach ($dbversion_disallowed as $dbversion_totest)
	{
	    if ($dbversion_totest['type'] == $dyap_db->type) $listofforbiddenversion.=($listofforbiddenversion?', ':'').join('.',$dbversion_totest['version']);
	}
	foreach ($dbversion_disallowed as $dbversion_totest)
	{
	    //print $dyap_db->type.' - '.join('.',$versionarray).' - '.versioncompare($dbversion_totest['version'],$versionarray)."<br>\n";
	    if ($dbversion_totest['type'] == $dyap_db->type
		&& (versioncompare($dbversion_totest['version'],$versionarray) == 0 || versioncompare($dbversion_totest['version'],$versionarray)<=-4 || versioncompare($dbversion_totest['version'],$versionarray)>=4)
	    )
	    {
		// Warning: database version too low.
		print '<tr><td><div class="warning">'.$dyap_langs->trans("ErrorDatabaseVersionForbiddenForMigration",join('.',$versionarray),$listofforbiddenversion)."</div></td><td align=\"right\">".$dyap_langs->trans("Error")."</td></tr>\n";
		dolibarr_install_syslog("upgrade: " . $dyap_langs->transnoentities("ErrorDatabaseVersionForbiddenForMigration", join('.', $versionarray), $listofforbiddenversion));
		return false;
	    }
	}
    }
    /*
     * Delete duplicates in table categorie_association
     */
    $result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."categorie_association");
    if ($result)	// result defined for version 3.2 or -
    {
	$obj = $dyap_db->fetch_object($result);
	if ($obj)	// It table categorie_association exists
	{
	    $couples=array();
	    $filles=array();
	    $sql = "SELECT fk_categorie_mere, fk_categorie_fille";
	    $sql.= " FROM ".MAIN_DB_PREFIX."categorie_association";
	    dolibarr_install_syslog("upgrade: search duplicate");
	    $resql = $dyap_db->query($sql);
	    if ($resql)
	    {
		$num=$dyap_db->num_rows($resql);
		while ($obj=$dyap_db->fetch_object($resql))
		{
		    if (! isset($filles[$obj->fk_categorie_fille]))	// Only one record as child (a child has only on parent).
		    {
			if ($obj->fk_categorie_mere != $obj->fk_categorie_fille)
			{
			    $filles[$obj->fk_categorie_fille]=1;	// Set record for this child
			    $couples[$obj->fk_categorie_mere.'_'.$obj->fk_categorie_fille]=array('mere'=>$obj->fk_categorie_mere, 'fille'=>$obj->fk_categorie_fille);
			}
		    }
		}

		dolibarr_install_syslog("upgrade: result is num=" . $num . " count(couples)=" . count($couples));

		// If there is duplicates couples or child with two parents
		if (count($couples) > 0 && $num > count($couples))
		{
		    $error=0;

		    $dyap_db->begin();

		    // We delete all
		    $sql="DELETE FROM ".MAIN_DB_PREFIX."categorie_association";
		    dolibarr_install_syslog("upgrade: delete association");
		    $resqld=$dyap_db->query($sql);
		    if ($resqld)
		    {
			// And we insert only each record once
			foreach($couples as $key => $val)
			{
			    $sql ="INSERT INTO ".MAIN_DB_PREFIX."categorie_association(fk_categorie_mere,fk_categorie_fille)";
			    $sql.=" VALUES(".$val['mere'].", ".$val['fille'].")";
			    dolibarr_install_syslog("upgrade: insert association");
			    $resqli=$dyap_db->query($sql);
			    if (! $resqli) $error++;
			}
		    }

		    if (! $error)
		    {
			print '<tr><td>'.$dyap_langs->trans("RemoveDuplicates").'</td>';
			print '<td align="right">'.$dyap_langs->trans("Success").' ('.$num.'=>'.count($couples).')</td></tr>';
			$dyap_db->commit();
		    }
		    else
		    {
			print '<tr><td>'.$dyap_langs->trans("RemoveDuplicates").'</td>';
			print '<td align="right">'.$dyap_langs->trans("Failed").'</td></tr>';
			$dyap_db->rollback();
		    }
		}
	    }
	    else
	    {
		print '<div class="error">'.$dyap_langs->trans("Error").' '.$dyap_db->lasterror().'</div>';
	    }
	}
    }

    /*
     * Remove deprecated indexes and constraints for Mysql
     */
    if (preg_match('/mysql/',$dyap_db->type))
    {
	$versioncommande=array(4,0,0);
	if (count($versioncommande) && count($versionarray)
	    && versioncompare($versioncommande,$versionarray) <= 0)	// Si mysql >= 4.0
	{
	    // Suppression vieilles contraintes sans noms et en doubles
	    // Les contraintes indesirables ont un nom qui commence par 0_ ou se termine par ibfk_999
	    $listtables=array(
		MAIN_DB_PREFIX.'adherent_options',
		MAIN_DB_PREFIX.'bank_class',
		MAIN_DB_PREFIX.'c_ecotaxe',
		MAIN_DB_PREFIX.'c_methode_commande_fournisseur',   // table renamed
		MAIN_DB_PREFIX.'c_input_method'
	    );

	    $listtables = $dyap_db->DDLListTables($dyap_conf->db->name,'');
	    foreach ($listtables as $val)
	    {
		// Database prefix filter
		if (preg_match('/^'.MAIN_DB_PREFIX.'/', $val))
		{
		    //print "x".$val."<br>";
		    $sql = "SHOW CREATE TABLE ".$val;
		    $resql = $dyap_db->query($sql);
		    if ($resql)
		    {
			$values=$dyap_db->fetch_array($resql);
			$i=0;
			$createsql=$values[1];
			while (preg_match('/CONSTRAINT `(0_[0-9a-zA-Z]+|[_0-9a-zA-Z]+_ibfk_[0-9]+)`/i',$createsql,$reg) && $i < 100)
			{
			    $sqldrop="ALTER TABLE ".$val." DROP FOREIGN KEY ".$reg[1];
			    $resqldrop = $dyap_db->query($sqldrop);
			    if ($resqldrop)
			    {
				print '<tr><td colspan="2">'.$sqldrop.";</td></tr>\n";
			    }
			    $createsql=preg_replace('/CONSTRAINT `'.$reg[1].'`/i','XXX',$createsql);
			    $i++;
			}
			$dyap_db->free($resql);
		    }
		    else
		    {
			if ($dyap_db->lasterrno() != 'DB_ERROR_NOSUCHTABLE')
			{
			    print '<tr><td colspan="2"><font  class="error">'.$sql.' : '.$dyap_db->lasterror()."</font></td></tr>\n";
			}
		    }
		}
	    }
	}
    }

    return true;
}

/*
 * Load sql files
 */
function dyapress_upgrade_step1($versionfrom, $versionto) {
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    $dir = DOL_DOCUMENT_ROOT . "/install/mysql/migration/";	// We use mysql migration scripts whatever is database driver
    if (!empty($dirmodule)) {
	$dir = dol_buildpath('/'.$dirmodule.'/sql/', 0);
    }
    dolibarr_install_syslog("Scan sql files for migration files in ".$dir);

    // Clean last part to exclude minor version x.y.z -> x.y
    $newversionfrom = preg_replace('/(\.[0-9]+)$/i', '.0', $versionfrom);
    $snewversionto = preg_replace('/(\.[0-9]+)$/i', '.0', $versionto);

    $filelist = array();
    $i = 0;
    $ok = 0;
    $from = '^'.preg_quote($newversionfrom, '/');
    $to = preg_quote($newversionto.'.sql', '/').'$';

    // Get files list
    $filesindir = array();
    $handle = opendir($dir);
    if (is_resource($handle)) {
	while (($dyap_file = readdir($handle)) !== false) {
	    if (preg_match('/\.sql$/i', $dyap_file)) {
		$filesindir[] = $dyap_file;
	    }
	}
	sort($filesindir);
    } else {
	print '<div class="update-message inline notice notice-alt notice-error"><p>'.$dyap_langs->trans("ErrorCanNotReadDir")." : $dir".'</p></div>';
    }

    // Define which file to run
    foreach ($filesindir as $dyap_file) {
	if (preg_match('/'.$from.'\-/i', $dyap_file)) {
	    $filelist[] = $dyap_file;
	} elseif (preg_match('/\-'.$to.'/i', $dyap_file)) {	// First test may be false if we migrate from x.y.* to x.y.*
	    $filelist[] = $dyap_file;
	}
    }

    if (count($filelist) == 0) {
	print '<div class="update-message inline notice notice-alt notice-error"><p>'.$dyap_langs->trans("ErrorNoMigrationFilesFoundForParameters").'</p></div>';
    } else {
	$listoffileprocessed = array(); // Protection to avoid to process twice the same file

	// Loop on each migrate files
	foreach ($filelist as $dyap_file) {
	    if (in_array($dir.$dyap_file, $listoffileprocessed)) {
		continue;
	    }

	    // Run sql script
	    $ok = run_sql($dir.$dyap_file, 1, '', 1, '', 'default', 32768, 0, 0, 2);
	    $listoffileprocessed[$dir.$dyap_file] = $dir.$dyap_file;

	    print '<p><span class="dashicons dashicons-'.($ok ? 'yes': 'no').'"></span><b>'.$dyap_langs->trans("ChoosedMigrateScript")."</b>: ".$dyap_file."</p>\n";

	    // Scan if there is migration scripts that depends of Dolibarr version
	    // for modules htdocs/module/sql or htdocs/custom/module/sql (files called "dolibarr_x.y.z-a.b.c.sql")
	    $modulesfile = array();
	    foreach ($dyap_conf->file->dol_document_root as $type => $dirroot) {
		$handlemodule = @opendir($dirroot); // $dirroot may be '..'
		if (is_resource($handlemodule)) {
		    while (($filemodule = readdir($handlemodule)) !== false) {
			if (!preg_match('/\./', $filemodule) && is_dir($dirroot.'/'.$filemodule.'/sql')) {	// We exclude filemodule that contains . (are not directories) and are not directories.
			    //print "Scan for ".$dirroot . '/' . $filemodule . '/sql/'.$dyap_file;
			    if (is_file($dirroot.'/'.$filemodule.'/sql/dolibarr_'.$dyap_file)) {
				$modulesfile[$dirroot.'/'.$filemodule.'/sql/dolibarr_'.$dyap_file] = '/'.$filemodule.'/sql/dolibarr_'.$dyap_file;
			    }
			}
		    }
		    closedir($handlemodule);
		}
	    }

	    if (count($modulesfile)) {
		print '<tr><td colspan="2"><hr style="border-color: #ccc; border-top-style: none;"></td></tr>';

		foreach ($modulesfile as $modulefilelong => $modulefileshort) {
		    if (in_array($modulefilelong, $listoffileprocessed)) {
			continue;
		    }

		    // Run sql script
		    $okmodule = run_sql($modulefilelong, 1, '', 1); // Note: Result of migration of external module should not decide if we continue migration of Dolibarr or not.

		    print '<div class="update-message inline notice notice-alt '.($okmodule ? 'updated-message notice-success' : 'notice-error').'"><p>'.$dyap_langs->trans("ChoosedMigrateScript").' : '.$modulefileshort.' : '.$modulefilelong.'</p></div>'."\n";

		    $listoffileprocessed[$modulefilelong] = $modulefilelong;
		}
	    }
	}
    }
}

function dyapress_upgrade_step2($versionfrom, $versionto) {
    global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user, $dyap_mysoc, $dyap_hookmanager;

    print '<div class="inline"><p><strong>'.$dyap_langs->trans('DataMigration').'</strong></p></div>'."\n";

    // Create the global $dyap_hookmanager object
    include_once DOL_DOCUMENT_ROOT.'/core/class/hookmanager.class.php';
    $dyap_hookmanager = new HookManager($dyap_db);
    $dyap_hookmanager->initHooks(array('upgrade'));

    /***************************************************************************************
     *
     * Migration of data
     *
     ***************************************************************************************/
    $dyap_db->begin();

    // Current version is $dyap_conf->global->MAIN_VERSION_LAST_UPGRADE
    // Version to install is DOL_VERSION
    $dolibarrlastupgradeversionarray = preg_split('/[\.-]/', isset($dyap_conf->global->MAIN_VERSION_LAST_UPGRADE) ? $dyap_conf->global->MAIN_VERSION_LAST_UPGRADE : (isset($dyap_conf->global->MAIN_VERSION_LAST_INSTALL) ? $dyap_conf->global->MAIN_VERSION_LAST_INSTALL : ''));

    // Chaque action de migration doit renvoyer une ligne sur 4 colonnes avec
    // dans la 1ere colonne, la description de l'action a faire
    // dans la 4eme colonne, le texte 'OK' si fait ou 'AlreadyDone' si rien n'est fait ou 'Error'

    $versiontoarray = explode('.', $versionto);
    $versionranarray = explode('.', DOL_VERSION);


    // Force to execute this at begin to avoid the new core code into Dolibarr to be broken.
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'user ADD COLUMN birth date';
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'user ADD COLUMN dateemployment date';
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'user ADD COLUMN dateemploymentend date';
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'user ADD COLUMN default_range integer';
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'user ADD COLUMN default_c_exp_tax_cat integer';
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'extrafields ADD COLUMN langs varchar(24)';
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'extrafields ADD COLUMN fieldcomputed text';
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'extrafields ADD COLUMN fielddefault varchar(255)';
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX."extrafields ADD COLUMN enabled varchar(255) DEFAULT '1'";
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'extrafields ADD COLUMN help text';
    $dyap_db->query($sql, 1);
    $sql = 'ALTER TABLE '.MAIN_DB_PREFIX.'user_rights ADD COLUMN entity integer DEFAULT 1 NOT NULL';
    $dyap_db->query($sql, 1);


    $afterversionarray=explode('.','2.0.0');
    $beforeversionarray=explode('.','2.7.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	// Script pour V2 -> V2.1
	migrate_paiements($dyap_db,$dyap_langs,$dyap_conf);

	migrate_contracts_det($dyap_db,$dyap_langs,$dyap_conf);

	migrate_contracts_date1($dyap_db,$dyap_langs,$dyap_conf);

	migrate_contracts_date2($dyap_db,$dyap_langs,$dyap_conf);

	migrate_contracts_date3($dyap_db,$dyap_langs,$dyap_conf);

	migrate_contracts_open($dyap_db,$dyap_langs,$dyap_conf);

	migrate_modeles($dyap_db,$dyap_langs,$dyap_conf);

	migrate_price_propal($dyap_db,$dyap_langs,$dyap_conf);

	migrate_price_commande($dyap_db,$dyap_langs,$dyap_conf);

	migrate_price_commande_fournisseur($dyap_db,$dyap_langs,$dyap_conf);

	migrate_price_contrat($dyap_db,$dyap_langs,$dyap_conf);

	migrate_paiementfourn_facturefourn($dyap_db,$dyap_langs,$dyap_conf);


	// Script pour V2.1 -> V2.2
	migrate_paiements_orphelins_1($dyap_db,$dyap_langs,$dyap_conf);

	migrate_paiements_orphelins_2($dyap_db,$dyap_langs,$dyap_conf);

	migrate_links_transfert($dyap_db,$dyap_langs,$dyap_conf);


	// Script pour V2.2 -> V2.4
	migrate_commande_expedition($dyap_db,$dyap_langs,$dyap_conf);

	migrate_commande_livraison($dyap_db,$dyap_langs,$dyap_conf);

	migrate_detail_livraison($dyap_db,$dyap_langs,$dyap_conf);


	// Script pour V2.5 -> V2.6
	migrate_stocks($dyap_db,$dyap_langs,$dyap_conf);


	// Script pour V2.6 -> V2.7
	migrate_menus($dyap_db,$dyap_langs,$dyap_conf);

	migrate_commande_deliveryaddress($dyap_db,$dyap_langs,$dyap_conf);

	migrate_restore_missing_links($dyap_db,$dyap_langs,$dyap_conf);

	migrate_rename_directories($dyap_db,$dyap_langs,$dyap_conf,'/compta','/banque');

	migrate_rename_directories($dyap_db,$dyap_langs,$dyap_conf,'/societe','/mycompany');
    }

    // Script for 2.8
    $afterversionarray=explode('.','2.7.9');
    $beforeversionarray=explode('.','2.8.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	migrate_price_facture($dyap_db, $dyap_langs, $dyap_conf); // Code of this function works for 2.8+ because need field tva_tx

	migrate_relationship_tables($dyap_db,$dyap_langs,$dyap_conf,'co_exp','fk_commande','commande','fk_expedition','shipping');

	migrate_relationship_tables($dyap_db,$dyap_langs,$dyap_conf,'pr_exp','fk_propal','propal','fk_expedition','shipping');

	migrate_relationship_tables($dyap_db,$dyap_langs,$dyap_conf,'pr_liv','fk_propal','propal','fk_livraison','delivery');

	migrate_relationship_tables($dyap_db,$dyap_langs,$dyap_conf,'co_liv','fk_commande','commande','fk_livraison','delivery');

	migrate_relationship_tables($dyap_db,$dyap_langs,$dyap_conf,'co_pr','fk_propale','propal','fk_commande','commande');

	migrate_relationship_tables($dyap_db,$dyap_langs,$dyap_conf,'fa_pr','fk_propal','propal','fk_facture','facture');

	migrate_relationship_tables($dyap_db,$dyap_langs,$dyap_conf,'co_fa','fk_commande','commande','fk_facture','facture');

	migrate_project_user_resp($dyap_db,$dyap_langs,$dyap_conf);

	migrate_project_task_actors($dyap_db,$dyap_langs,$dyap_conf);
    }

    // Script for 2.9
    $afterversionarray=explode('.','2.8.9');
    $beforeversionarray=explode('.','2.9.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	migrate_project_task_time($dyap_db,$dyap_langs,$dyap_conf);

	migrate_customerorder_shipping($dyap_db,$dyap_langs,$dyap_conf);

	migrate_shipping_delivery($dyap_db,$dyap_langs,$dyap_conf);

	migrate_shipping_delivery2($dyap_db,$dyap_langs,$dyap_conf);
    }

    // Script for 3.0
    $afterversionarray=explode('.','2.9.9');
    $beforeversionarray=explode('.','3.0.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	// No particular code
    }

    // Script for 3.1
    $afterversionarray=explode('.','3.0.9');
    $beforeversionarray=explode('.','3.1.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	migrate_rename_directories($dyap_db,$dyap_langs,$dyap_conf,'/rss','/externalrss');

	migrate_actioncomm_element($dyap_db,$dyap_langs,$dyap_conf);
    }

    // Script for 3.2
    $afterversionarray=explode('.','3.1.9');
    $beforeversionarray=explode('.','3.2.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	migrate_price_contrat($dyap_db,$dyap_langs,$dyap_conf);

	migrate_mode_reglement($dyap_db,$dyap_langs,$dyap_conf);

	migrate_clean_association($dyap_db,$dyap_langs,$dyap_conf);
    }

    // Script for 3.3
    $afterversionarray=explode('.','3.2.9');
    $beforeversionarray=explode('.','3.3.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	migrate_categorie_association($dyap_db,$dyap_langs,$dyap_conf);
    }

    // Script for 3.4
    // No specific scripts

    // Tasks to do always and only into last targeted version
    $afterversionarray=explode('.','3.6.9');	// target is after this
    $beforeversionarray=explode('.','3.7.9');	// target is before this
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	migrate_event_assignement($dyap_db,$dyap_langs,$dyap_conf);
    }

    // Scripts for 3.9
    $afterversionarray=explode('.','3.7.9');
    $beforeversionarray=explode('.','3.8.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	// No particular code
    }

    // Scripts for 4.0
    $afterversionarray=explode('.','3.9.9');
    $beforeversionarray=explode('.','4.0.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	migrate_rename_directories($dyap_db,$dyap_langs,$dyap_conf,'/fckeditor','/medias');
    }

    // Scripts for 5.0
    $afterversionarray=explode('.','4.0.9');
    $beforeversionarray=explode('.','5.0.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	// Migrate to add entity value into llx_societe_remise
	migrate_remise_entity($dyap_db,$dyap_langs,$dyap_conf);

	// Migrate to add entity value into llx_societe_remise_except
	migrate_remise_except_entity($dyap_db,$dyap_langs,$dyap_conf);
    }

    // Scripts for 6.0
    $afterversionarray=explode('.','5.0.9');
    $beforeversionarray=explode('.','6.0.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	if (! empty($dyap_conf->multicompany->enabled)) {
	    // Migrate to add entity value into llx_user_rights
	    migrate_user_rights_entity($dyap_db, $dyap_langs, $dyap_conf);

	    // Migrate to add entity value into llx_usergroup_rights
	    migrate_usergroup_rights_entity($dyap_db, $dyap_langs, $dyap_conf);
	}
    }

    // Scripts for 7.0
    $afterversionarray=explode('.','6.0.9');
    $beforeversionarray=explode('.','7.0.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	// Migrate contact association
	migrate_event_assignement_contact($dyap_db,$dyap_langs,$dyap_conf);

	migrate_reset_blocked_log($dyap_db,$dyap_langs,$dyap_conf);
    }

    // Scripts for 8.0
    $afterversionarray=explode('.','7.0.9');
    $beforeversionarray=explode('.','8.0.9');
    if (versioncompare($versiontoarray,$afterversionarray) >= 0 && versioncompare($versiontoarray,$beforeversionarray) <= 0) {
	print '<p><span class="dashicons dashicons-yes"></span><b>before 8.0</b>'."</p>\n";
	migrate_rename_directories($dyap_db,$dyap_langs,$dyap_conf,'/contracts','/contract');
	print '<p><span class="dashicons dashicons-yes"></span><b>after 8.0</b>'."</p>\n";
    }

    // Scripts for 9.0
    $afterversionarray = explode('.', '8.0.9');
    $beforeversionarray = explode('.', '9.0.9');
    if (versioncompare($versiontoarray, $afterversionarray) >= 0 && versioncompare($versiontoarray, $beforeversionarray) <= 0) {
	migrate_user_photospath();
    }

    // Scripts for 11.0
    $afterversionarray = explode('.', '10.0.9');
    $beforeversionarray = explode('.', '11.0.9');
    if (versioncompare($versiontoarray, $afterversionarray) >= 0 && versioncompare($versiontoarray, $beforeversionarray) <= 0) {
	migrate_users_socialnetworks();
	migrate_members_socialnetworks();
	migrate_contacts_socialnetworks();
	migrate_thirdparties_socialnetworks();
    }

    // Scripts for 14.0
    $afterversionarray = explode('.', '13.0.9');
    $beforeversionarray = explode('.', '14.0.9');
    if (versioncompare($versiontoarray, $afterversionarray) >= 0 && versioncompare($versiontoarray, $beforeversionarray) <= 0) {
	migrate_export_import_profiles('export');
	migrate_export_import_profiles('import');
    }
 
    // Code executed only if migration is LAST ONE. Must always be done.
    if (versioncompare($versiontoarray, $versionranarray) >= 0 || versioncompare($versiontoarray, $versionranarray) <= -3) {
	// Reload modules (this must be always done and only into last targeted version, because code to reload module may need table structure of last version)
	$listofmodule = array(
	    'MAIN_MODULE_ACCOUNTING'=>'newboxdefonly',
	    'MAIN_MODULE_AGENDA'=>'newboxdefonly',
	    'MAIN_MODULE_BOM'=>'menuonly',
	    'MAIN_MODULE_BANQUE'=>'menuonly',
	    'MAIN_MODULE_BARCODE'=>'newboxdefonly',
	    'MAIN_MODULE_CRON'=>'newboxdefonly',
	    'MAIN_MODULE_COMMANDE'=>'newboxdefonly',
	    'MAIN_MODULE_BLOCKEDLOG'=>'noboxes',
	    'MAIN_MODULE_DEPLACEMENT'=>'newboxdefonly',
	    'MAIN_MODULE_DON'=>'newboxdefonly',
	    'MAIN_MODULE_ECM'=>'newboxdefonly',
	    'MAIN_MODULE_EXTERNALSITE'=>'newboxdefonly',
	    'MAIN_MODULE_EXPENSEREPORT'=>'newboxdefonly',
	    'MAIN_MODULE_FACTURE'=>'newboxdefonly',
	    'MAIN_MODULE_FOURNISSEUR'=>'newboxdefonly',
	    'MAIN_MODULE_HOLIDAY'=>'newboxdefonly',
	    'MAIN_MODULE_MARGIN'=>'menuonly',
	    'MAIN_MODULE_MRP'=>'menuonly',
	    'MAIN_MODULE_OPENSURVEY'=>'newboxdefonly',
	    'MAIN_MODULE_PAYBOX'=>'newboxdefonly',
	    'MAIN_MODULE_PRINTING'=>'newboxdefonly',
	    'MAIN_MODULE_PRODUIT'=>'newboxdefonly',
	    'MAIN_MODULE_RECRUITMENT'=>'menuonly',
	    'MAIN_MODULE_RESOURCE'=>'noboxes',
	    'MAIN_MODULE_SALARIES'=>'newboxdefonly',
	    'MAIN_MODULE_SERVICE'=>'newboxdefonly',
	    'MAIN_MODULE_SYSLOG'=>'newboxdefonly',
	    'MAIN_MODULE_SOCIETE'=>'newboxdefonly',
	    'MAIN_MODULE_STRIPE'=>'menuonly',
	    'MAIN_MODULE_TICKET'=>'newboxdefonly',
	    'MAIN_MODULE_TAKEPOS'=>'newboxdefonly',
	    'MAIN_MODULE_USER'=>'newboxdefonly', //This one must be always done and only into last targeted version)
	    'MAIN_MODULE_VARIANTS'=>'newboxdefonly',
	    'MAIN_MODULE_WEBSITE'=>'newboxdefonly',
	);

	print '<p><span class="dashicons dashicons-yes"></span><b>reload_modules</b>'."</p>\n";
	$result = migrate_reload_modules($dyap_db, $dyap_langs, $dyap_conf, $listofmodule);
	if ($result < 0) {
	    $error++;
	}
	print '<p><span class="dashicons dashicons-yes"></span><b>reload_menu</b>'."</p>\n";
	// Reload menus (this must be always and only into last targeted version)
	$result = migrate_reload_menu($dyap_db, $dyap_langs, $dyap_conf);
	if ($result < 0) {
	    $error++;
	}
    }

    print '<p><span class="dashicons dashicons-yes"></span><b>after hookmanager init hooks</b>'."</p>\n";
    // Can force activation of some module during migration with parameter 'enablemodules=MAIN_MODULE_XXX,MAIN_MODULE_YYY,...'
    // In most cases (online install or upgrade) $enablemodules is empty. Can be forced when ran from command line.
    if (!$error && $enablemodules) {
	// Reload modules (this must be always done and only into last targeted version)
	$listofmodules = array();
	$enablemodules = preg_replace('/enablemodules=/', '', $enablemodules);
	$tmplistofmodules = explode(',', $enablemodules);
	foreach ($tmplistofmodules as $value) {
	    $listofmodules[$value] = 'forceactivate';
	}

	$resultreloadmodules = migrate_reload_modules($dyap_db, $dyap_langs, $dyap_conf, $listofmodules, 1);
	if ($resultreloadmodules < 0) {
	    $error++;
	}
    }


    // Can call a dedicated external upgrade process
    if (!$error) {
	$parameters = array('versionfrom' => $versionfrom, 'versionto' => $versionto);
	$dyap_object = new \stdClass();
	$dyap_action = "upgrade";
	$reshook = $dyap_hookmanager->executeHooks('doUpgrade2', $parameters, $dyap_object, $dyap_action); // Note that $dyap_action and $dyap_object may have been modified by some hooks
	if ($dyap_hookmanager->resNbOfHooks > 0) {
	    if ($reshook < 0) {
		print '<p><span class="dashicons dashicons-no error"></span><b>'.$dyap_langs->trans("UpgradeExternalModule")."</b>: ".$dyap_hookmanager->error."<!-- (".$reshook.") --></p>\n";
	    } else {
		print '<p><span class="dashicons dashicons-yes"></span><b>'.$dyap_langs->trans("UpgradeExternalModule")."</b>: OK <!-- (".$reshook.") --></p>\n";
	    }
	} else {
	    //if (! empty($dyap_conf->modules))
	    if (!empty($dyap_conf->modules_parts['hooks'])) {     // If there is at least one module with one hook, we show message to say nothing was done
		print '<p><span class="dashicons dashicons-yes error"></span><b>'.$dyap_langs->trans("UpgradeExternalModule")."</b>: ".$dyap_langs->trans("None")."</p>\n";
	    }
	}
    }

    if (!$error) {
	// Set constant to ask to remake a new ping to inform about upgrade (if first ping was done and OK)
	$sql = 'UPDATE '.MAIN_DB_PREFIX."const SET VALUE = 'torefresh' WHERE name = 'MAIN_FIRST_PING_OK_ID'";
	$dyap_db->query($sql, 1);
    }

    // We always commit.
    // Process is designed so we can run it several times whatever is situation.
    $dyap_db->commit();
    // $dyap_db->close();

    // Copy directory medias $srcroot = DOL_DOCUMENT_ROOT.'/install/medias';
    $destroot = DOL_DATA_ROOT.'/medias';
    dolCopyDir($srcroot, $destroot, 0, 0);


    // Actions for all versions (no database change but delete some files and directories)
    migrate_delete_old_files($dyap_db, $dyap_langs, $dyap_conf);
    migrate_delete_old_dir($dyap_db, $dyap_langs, $dyap_conf);
    // Actions for all versions (no database change but create some directories)
    dol_mkdir(DOL_DATA_ROOT.'/bank');
    // Actions for all versions (no database change but rename some directories)
    migrate_rename_directories($dyap_db, $dyap_langs, $dyap_conf, '/banque/bordereau', '/bank/checkdeposits');

    /*
    print '<div class="update-message inline notice notice-alt '.($ok ? 'updated-message notice-success' : 'notice-error').'"><p>'.$dyap_langs->trans('MigrationFinished').'</p>';
    if (!$ok)  print '<p><b>'.$dyap_langs->trans('UpgradeExternalModule').'</b>:</p>'.$dyap_hookmanager->error;
    print '</div>'."\n";
     */
}

/**
 * Reporte liens vers une facture de paiements sur table de jointure (lien n-n paiements factures)
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_paiements($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationPaymentsUpdate')."</b><br>\n";

	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."paiement", "fk_facture");
	$dyap_obj = $dyap_db->fetch_object($result);
	if ($dyap_obj) {
		$sql = "SELECT p.rowid, p.fk_facture, p.amount";
		$sql .= " FROM ".MAIN_DB_PREFIX."paiement as p";
		$sql .= " WHERE p.fk_facture > 0";

		$resql = $dyap_db->query($sql);

		dolibarr_install_syslog("upgrade2::migrate_paiements");
		if ($resql) {
			$i = 0;
			$row = array();
			$num = $dyap_db->num_rows($resql);

			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);
				$row[$i][0] = $dyap_obj->rowid;
				$row[$i][1] = $dyap_obj->fk_facture;
				$row[$i][2] = $dyap_obj->amount;
				$i++;
			}
		} else {
			dol_print_error($dyap_db);
		}

		if ($num) {
			print $dyap_langs->trans('MigrationPaymentsNumberToUpdate', $num)."<br>\n";
			if ($dyap_db->begin()) {
				$res = 0;
				$num = count($row);
				for ($i = 0; $i < $num; $i++) {
					$sql = "INSERT INTO ".MAIN_DB_PREFIX."paiement_facture (fk_facture, fk_paiement, amount)";
					$sql .= " VALUES (".((int) $row[$i][1]).",".((int) $row[$i][0]).",".((float) $row[$i][2]).")";

					$res += $dyap_db->query($sql);

					$sql = "UPDATE ".MAIN_DB_PREFIX."paiement SET fk_facture = 0 WHERE rowid = ".((int) $row[$i][0]);

					$res += $dyap_db->query($sql);

					print $dyap_langs->trans('MigrationProcessPaymentUpdate', $row[$i][0])."<br>\n";
				}
			}

			if ($res == (2 * count($row))) {
				$dyap_db->commit();
				print $dyap_langs->trans('MigrationSuccessfullUpdate')."<br>";
			} else {
				$dyap_db->rollback();
				print $dyap_langs->trans('MigrationUpdateFailed').'<br>';
			}
		} else {
			print $dyap_langs->trans('MigrationPaymentsNothingToUpdate')."<br>\n";
		}
	} else {
		print $dyap_langs->trans('MigrationPaymentsNothingToUpdate')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Corrige paiement orphelins (liens paumes suite a bugs)
 * Pour verifier s'il reste des orphelins:
 * select * from llx_paiement as p left join llx_paiement_facture as pf on pf.fk_paiement=p.rowid WHERE pf.rowid IS NULL AND (p.fk_facture = 0 OR p.fk_facture IS NULL)
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_paiements_orphelins_1($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationPaymentsUpdate')."</b><br>\n";

	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."paiement", "fk_facture");
	$dyap_obj = $dyap_db->fetch_object($result);
	if ($dyap_obj) {
		// Tous les enregistrements qui sortent de cette requete devrait avoir un pere dans llx_paiement_facture
		$sql = "SELECT distinct p.rowid, p.datec, p.amount as pamount, bu.fk_bank, b.amount as bamount,";
		$sql .= " bu2.url_id as socid";
		$sql .= " FROM (".MAIN_DB_PREFIX."paiement as p, ".MAIN_DB_PREFIX."bank_url as bu, ".MAIN_DB_PREFIX."bank as b)";
		$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."paiement_facture as pf ON pf.fk_paiement = p.rowid";
		$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."bank_url as bu2 ON (bu.fk_bank=bu2.fk_bank AND bu2.type = 'company')";
		$sql .= " WHERE pf.rowid IS NULL AND (p.rowid=bu.url_id AND bu.type='payment') AND bu.fk_bank = b.rowid";
		$sql .= " AND b.rappro = 1";
		$sql .= " AND (p.fk_facture = 0 OR p.fk_facture IS NULL)";

		$resql = $dyap_db->query($sql);

		dolibarr_install_syslog("upgrade2::migrate_paiements_orphelins_1");
		$row = array();
		if ($resql) {
			$i = $j = 0;
			$num = $dyap_db->num_rows($resql);

			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);
				if ($dyap_obj->pamount == $dyap_obj->bamount && $dyap_obj->socid) {	// Pour etre sur d'avoir bon cas
					$row[$j]['paymentid'] = $dyap_obj->rowid; // paymentid
					$row[$j]['pamount'] = $dyap_obj->pamount;
					$row[$j]['fk_bank'] = $dyap_obj->fk_bank;
					$row[$j]['bamount'] = $dyap_obj->bamount;
					$row[$j]['socid'] = $dyap_obj->socid;
					$row[$j]['datec'] = $dyap_obj->datec;
					$j++;
				}
				$i++;
			}
		} else {
			dol_print_error($dyap_db);
		}

		if (count($row)) {
			print $dyap_langs->trans('OrphelinsPaymentsDetectedByMethod', 1).': '.count($row)."<br>\n";
			$dyap_db->begin();

			$res = 0;
			$num = count($row);
			for ($i = 0; $i < $num; $i++) {
				if ($dyap_conf->global->MAIN_FEATURES_LEVEL == 2) {
					print '* '.$row[$i]['datec'].' paymentid='.$row[$i]['paymentid'].' pamount='.$row[$i]['pamount'].' fk_bank='.$row[$i]['fk_bank'].' bamount='.$row[$i]['bamount'].' socid='.$row[$i]['socid'].'<br>';
				}

				// On cherche facture sans lien paiement et du meme montant et pour meme societe.
				$sql = " SELECT distinct f.rowid from ".MAIN_DB_PREFIX."facture as f";
				$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."paiement_facture as pf ON f.rowid = pf.fk_facture";
				$sql .= " WHERE f.fk_statut in (2,3) AND fk_soc = ".((int) $row[$i]['socid'])." AND total_ttc = ".((float) $row[$i]['pamount']);
				$sql .= " AND pf.fk_facture IS NULL";
				$sql .= " ORDER BY f.fk_statut";
				//print $sql.'<br>';
				$resql = $dyap_db->query($sql);
				if ($resql) {
					$num = $dyap_db->num_rows($resql);
					//print 'Nb of invoice found for this amount and company :'.$num.'<br>';
					if ($num >= 1) {
						$dyap_obj = $dyap_db->fetch_object($resql);
						$facid = $dyap_obj->rowid;

						$sql = "INSERT INTO ".MAIN_DB_PREFIX."paiement_facture (fk_facture, fk_paiement, amount)";
						$sql .= " VALUES (".((int) $facid).",".((int) $row[$i]['paymentid']).", ".((float) $row[$i]['pamount']).")";

						$res += $dyap_db->query($sql);

						print $dyap_langs->trans('MigrationProcessPaymentUpdate', 'facid='.$facid.'-paymentid='.$row[$i]['paymentid'].'-amount='.$row[$i]['pamount'])."<br>\n";
					}
				} else {
					print 'ERROR';
				}
			}

			if ($res > 0) {
				print $dyap_langs->trans('MigrationSuccessfullUpdate')."<br>";
			} else {
				print $dyap_langs->trans('MigrationPaymentsNothingUpdatable')."<br>\n";
			}

			$dyap_db->commit();
		} else {
			print $dyap_langs->trans('MigrationPaymentsNothingUpdatable')."<br>\n";
		}
	} else {
		print $dyap_langs->trans('MigrationPaymentsNothingUpdatable')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Corrige paiement orphelins (liens paumes suite a bugs)
 * Pour verifier s'il reste des orphelins:
 * select * from llx_paiement as p left join llx_paiement_facture as pf on pf.fk_paiement=p.rowid WHERE pf.rowid IS NULL AND (p.fk_facture = 0 OR p.fk_facture IS NULL)
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_paiements_orphelins_2($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationPaymentsUpdate')."</b><br>\n";

	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."paiement", "fk_facture");
	$dyap_obj = $dyap_db->fetch_object($result);
	if ($dyap_obj) {
		// Tous les enregistrements qui sortent de cette requete devrait avoir un pere dans llx_paiement_facture
		$sql = "SELECT distinct p.rowid, p.datec, p.amount as pamount, bu.fk_bank, b.amount as bamount,";
		$sql .= " bu2.url_id as socid";
		$sql .= " FROM (".MAIN_DB_PREFIX."paiement as p, ".MAIN_DB_PREFIX."bank_url as bu, ".MAIN_DB_PREFIX."bank as b)";
		$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."paiement_facture as pf ON pf.fk_paiement = p.rowid";
		$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."bank_url as bu2 ON (bu.fk_bank = bu2.fk_bank AND bu2.type = 'company')";
		$sql .= " WHERE pf.rowid IS NULL AND (p.fk_bank = bu.fk_bank AND bu.type = 'payment') AND bu.fk_bank = b.rowid";
		$sql .= " AND (p.fk_facture = 0 OR p.fk_facture IS NULL)";

		$resql = $dyap_db->query($sql);

		dolibarr_install_syslog("upgrade2::migrate_paiements_orphelins_2");
		$row = array();
		if ($resql) {
			$i = $j = 0;
			$num = $dyap_db->num_rows($resql);

			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);
				if ($dyap_obj->pamount == $dyap_obj->bamount && $dyap_obj->socid) {	// Pour etre sur d'avoir bon cas
					$row[$j]['paymentid'] = $dyap_obj->rowid; // paymentid
					$row[$j]['pamount'] = $dyap_obj->pamount;
					$row[$j]['fk_bank'] = $dyap_obj->fk_bank;
					$row[$j]['bamount'] = $dyap_obj->bamount;
					$row[$j]['socid'] = $dyap_obj->socid;
					$row[$j]['datec'] = $dyap_obj->datec;
					$j++;
				}
				$i++;
			}
		} else {
			dol_print_error($dyap_db);
		}

		$nberr = 0;

		$num = count($row);
		if ($num) {
			print $dyap_langs->trans('OrphelinsPaymentsDetectedByMethod', 2).': '.count($row)."<br>\n";
			$dyap_db->begin();

			$res = 0;
			for ($i = 0; $i < $num; $i++) {
				if ($dyap_conf->global->MAIN_FEATURES_LEVEL == 2) {
					print '* '.$row[$i]['datec'].' paymentid='.$row[$i]['paymentid'].' pamount='.$row[$i]['pamount'].' fk_bank='.$row[$i]['fk_bank'].' '.$row[$i]['bamount'].' socid='.$row[$i]['socid'].'<br>';
				}

				// On cherche facture sans lien paiement et du meme montant et pour meme societe.
				$sql = " SELECT distinct f.rowid from ".MAIN_DB_PREFIX."facture as f";
				$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."paiement_facture as pf ON f.rowid = pf.fk_facture";
				$sql .= " WHERE f.fk_statut in (2,3) AND fk_soc = ".((int) $row[$i]['socid'])." AND total_ttc = ".((float) $row[$i]['pamount']);
				$sql .= " AND pf.fk_facture IS NULL";
				$sql .= " ORDER BY f.fk_statut";
				//print $sql.'<br>';
				$resql = $dyap_db->query($sql);
				if ($resql) {
					$num = $dyap_db->num_rows($resql);
					//print 'Nb of invoice found for this amount and company :'.$num.'<br>';
					if ($num >= 1) {
						$dyap_obj = $dyap_db->fetch_object($resql);
						$facid = $dyap_obj->rowid;

						$sql = "INSERT INTO ".MAIN_DB_PREFIX."paiement_facture (fk_facture, fk_paiement, amount)";
						$sql .= " VALUES (".((int) $facid).",".((int) $row[$i]['paymentid']).", ".((float) $row[$i]['pamount']).")";

						$res += $dyap_db->query($sql);

						print $dyap_langs->trans('MigrationProcessPaymentUpdate', 'facid='.$facid.'-paymentid='.$row[$i]['paymentid'].'-amount='.$row[$i]['pamount'])."<br>\n";
					}
				} else {
					print 'ERROR';
					$nberr++;
				}
			}

			if ($res > 0) {
				print $dyap_langs->trans('MigrationSuccessfullUpdate')."<br>";
			} else {
				print $dyap_langs->trans('MigrationPaymentsNothingUpdatable')."<br>\n";
			}

			$dyap_db->commit();
		} else {
			print $dyap_langs->trans('MigrationPaymentsNothingUpdatable')."<br>\n";
		}

		// Delete obsolete fields fk_facture
		$dyap_db->begin();

		$sql = "ALTER TABLE ".MAIN_DB_PREFIX."paiement DROP COLUMN fk_facture";
		$dyap_db->query($sql);

		if (!$nberr) {
			$dyap_db->commit();
		} else {
			print 'ERROR';
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('MigrationPaymentsNothingUpdatable')."<br>\n";
	}

	print '</td></tr>';
}


/**
 * Mise a jour des contrats (gestion du contrat + detail de contrat)
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_contracts_det($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	$nberr = 0;

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationContractsUpdate')."</b><br>\n";

	$sql = "SELECT c.rowid as cref, c.date_contrat, c.statut, c.fk_product, c.fk_facture, c.fk_user_author,";
	$sql .= " p.ref, p.label, p.description, p.price, p.tva_tx, p.duration, cd.rowid";
	$sql .= " FROM ".MAIN_DB_PREFIX."contrat as c";
	$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."product as p";
	$sql .= " ON c.fk_product = p.rowid";
	$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."contratdet as cd";
	$sql .= " ON c.rowid=cd.fk_contrat";
	$sql .= " WHERE cd.rowid IS NULL AND p.rowid IS NOT NULL";
	$resql = $dyap_db->query($sql);

	dolibarr_install_syslog("upgrade2::migrate_contracts_det");
	if ($resql) {
		$i = 0;
		$row = array();
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			print $dyap_langs->trans('MigrationContractsNumberToUpdate', $num)."<br>\n";
			$dyap_db->begin();

			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$sql = "INSERT INTO ".MAIN_DB_PREFIX."contratdet (";
				$sql .= "fk_contrat, fk_product, statut, label, description,";
				$sql .= "date_ouverture_prevue, date_ouverture, date_fin_validite, tva_tx, qty,";
				$sql .= "subprice, price_ht, fk_user_author, fk_user_ouverture)";
				$sql .= " VALUES (";
				$sql .= ((int) $dyap_obj->cref).", ".($dyap_obj->fk_product ? ((int) $dyap_obj->fk_product) : 0).", ";
				$sql .= "0, ";
				$sql .= "'".$dyap_db->escape($dyap_obj->label)."', null, ";
				$sql .= ($dyap_obj->date_contrat ? "'".$dyap_db->idate($dyap_db->jdate($dyap_obj->date_contrat))."'" : "null").", ";
				$sql .= "null, ";
				$sql .= "null, ";
				$sql .= ((float) $dyap_obj->tva_tx).", 1, ";
				$sql .= ((float) $dyap_obj->price).", ".((float) $dyap_obj->price).", ".((int) $dyap_obj->fk_user_author).",";
				$sql .= "null";
				$sql .= ")";

				if ($dyap_db->query($sql)) {
					print $dyap_langs->trans('MigrationContractsLineCreation', $dyap_obj->cref)."<br>\n";
				} else {
					dol_print_error($dyap_db);
					$nberr++;
				}

				$i++;
			}

			if (!$nberr) {
				//      $dyap_db->rollback();
				$dyap_db->commit();
				print $dyap_langs->trans('MigrationSuccessfullUpdate')."<br>";
			} else {
				$dyap_db->rollback();
				print $dyap_langs->trans('MigrationUpdateFailed').'<br>';
			}
		} else {
			print $dyap_langs->trans('MigrationContractsNothingToUpdate')."<br>\n";
		}
	} else {
		print $dyap_langs->trans('MigrationContractsFieldDontExist')."<br>\n";
		//    dol_print_error($dyap_db);
	}

	print '</td></tr>';
}

/**
 * Function to migrate links into llx_bank_url
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_links_transfert($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	$nberr = 0;

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationBankTransfertsUpdate')."</b><br>\n";

	$sql = "SELECT ba.rowid as barowid, bb.rowid as bbrowid";
	$sql .= " FROM ".MAIN_DB_PREFIX."bank as bb, ".MAIN_DB_PREFIX."bank as ba";
	$sql .= " LEFT JOIN ".MAIN_DB_PREFIX."bank_url as bu ON bu.fk_bank = ba.rowid";
	$sql .= " WHERE ba.amount = -bb.amount AND ba.fk_account <> bb.fk_account";
	$sql .= " AND ba.datev = bb.datev AND ba.datec = bb.datec";
	$sql .= " AND bu.fk_bank IS NULL";
	$resql = $dyap_db->query($sql);

	dolibarr_install_syslog("upgrade2::migrate_links_transfert");
	if ($resql) {
		$i = 0;
		$row = array();
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			print $dyap_langs->trans('MigrationBankTransfertsToUpdate', $num)."<br>\n";
			$dyap_db->begin();

			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$sql = "INSERT INTO ".MAIN_DB_PREFIX."bank_url (";
				$sql .= "fk_bank, url_id, url, label, type";
				$sql .= ")";
				$sql .= " VALUES (";
				$sql .= $dyap_obj->barowid.",".$dyap_obj->bbrowid.", '/compta/bank/line.php?rowid=', '(banktransfert)', 'banktransfert'";
				$sql .= ")";

				//print $sql.'<br>';
				dolibarr_install_syslog("migrate_links_transfert");

				if (!$dyap_db->query($sql)) {
					dol_print_error($dyap_db);
					$nberr++;
				}

				$i++;
			}

			if (!$nberr) {
				//      $dyap_db->rollback();
				$dyap_db->commit();
				print $dyap_langs->trans('MigrationSuccessfullUpdate')."<br>";
			} else {
				$dyap_db->rollback();
				print $dyap_langs->trans('MigrationUpdateFailed').'<br>';
			}
		} else {
			print $dyap_langs->trans('MigrationBankTransfertsNothingToUpdate')."<br>\n";
		}
	} else {
		dol_print_error($dyap_db);
	}

	print '</td></tr>';
}

/**
 * Mise a jour des date de contrats non renseignees
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_contracts_date1($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationContractsEmptyDatesUpdate')."</b><br>\n";

	$sql = "update ".MAIN_DB_PREFIX."contrat set date_contrat=tms where date_contrat is null";
	dolibarr_install_syslog("upgrade2::migrate_contracts_date1");
	$resql = $dyap_db->query($sql);
	if (!$resql) {
		dol_print_error($dyap_db);
	}
	if ($dyap_db->affected_rows($resql) > 0) {
		print $dyap_langs->trans('MigrationContractsEmptyDatesUpdateSuccess')."<br>\n";
	} else {
		print $dyap_langs->trans('MigrationContractsEmptyDatesNothingToUpdate')."<br>\n";
	}

	$sql = "update ".MAIN_DB_PREFIX."contrat set datec=tms where datec is null";
	dolibarr_install_syslog("upgrade2::migrate_contracts_date1");
	$resql = $dyap_db->query($sql);
	if (!$resql) {
		dol_print_error($dyap_db);
	}
	if ($dyap_db->affected_rows($resql) > 0) {
		print $dyap_langs->trans('MigrationContractsEmptyCreationDatesUpdateSuccess')."<br>\n";
	} else {
		print $dyap_langs->trans('MigrationContractsEmptyCreationDatesNothingToUpdate')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Update contracts with date min real if service date is lower
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Language
 * @param	Conf		$dyap_conf	Conf
 * @return	void
 */
function migrate_contracts_date2($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	$nberr = 0;

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationContractsInvalidDatesUpdate')."</b><br>\n";

	$sql = "SELECT c.rowid as cref, c.datec, c.date_contrat, MIN(cd.date_ouverture) as datemin";
	$sql .= " FROM ".MAIN_DB_PREFIX."contrat as c,";
	$sql .= " ".MAIN_DB_PREFIX."contratdet as cd";
	$sql .= " WHERE c.rowid=cd.fk_contrat AND cd.date_ouverture IS NOT NULL";
	$sql .= " GROUP BY c.rowid, c.date_contrat";
	$resql = $dyap_db->query($sql);

	dolibarr_install_syslog("upgrade2::migrate_contracts_date2");
	if ($resql) {
		$i = 0;
		$row = array();
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			$nbcontratsmodifie = 0;
			$dyap_db->begin();

			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);
				if ($dyap_obj->date_contrat > $dyap_obj->datemin) {
					$datemin = $dyap_db->jdate($dyap_obj->datemin);

					print $dyap_langs->trans('MigrationContractsInvalidDateFix', $dyap_obj->cref, $dyap_obj->date_contrat, $dyap_obj->datemin)."<br>\n";
					$sql = "UPDATE ".MAIN_DB_PREFIX."contrat";
					$sql .= " SET date_contrat='".$dyap_db->idate($datemin)."'";
					$sql .= " WHERE rowid = ".((int) $dyap_obj->cref);
					$resql2 = $dyap_db->query($sql);
					if (!$resql2) {
						dol_print_error($dyap_db);
					}

					$nbcontratsmodifie++;
				}
				$i++;
			}

			$dyap_db->commit();

			if ($nbcontratsmodifie) {
				print $dyap_langs->trans('MigrationContractsInvalidDatesNumber', $nbcontratsmodifie)."<br>\n";
			} else {
				print  $dyap_langs->trans('MigrationContractsInvalidDatesNothingToUpdate')."<br>\n";
			}
		}
	} else {
		dol_print_error($dyap_db);
	}

	print '</td></tr>';
}

/**
 * Mise a jour des dates de creation de contrat
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_contracts_date3($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationContractsIncoherentCreationDateUpdate')."</b><br>\n";

	$sql = "update ".MAIN_DB_PREFIX."contrat set datec=date_contrat where datec is null or datec > date_contrat";
	dolibarr_install_syslog("upgrade2::migrate_contracts_date3");
	$resql = $dyap_db->query($sql);
	if (!$resql) {
		dol_print_error($dyap_db);
	}
	if ($dyap_db->affected_rows($resql) > 0) {
		print $dyap_langs->trans('MigrationContractsIncoherentCreationDateUpdateSuccess')."<br>\n";
	} else {
		print $dyap_langs->trans('MigrationContractsIncoherentCreationDateNothingToUpdate')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Reouverture des contrats qui ont au moins une ligne non fermee
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_contracts_open($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationReopeningContracts')."</b><br>\n";

	$sql = "SELECT c.rowid as cref FROM ".MAIN_DB_PREFIX."contrat as c, ".MAIN_DB_PREFIX."contratdet as cd";
	$sql .= " WHERE cd.statut = 4 AND c.statut=2 AND c.rowid=cd.fk_contrat";
	dolibarr_install_syslog("upgrade2::migrate_contracts_open");
	$resql = $dyap_db->query($sql);
	if (!$resql) {
		dol_print_error($dyap_db);
	}
	if ($dyap_db->affected_rows($resql) > 0) {
		$i = 0;
		$row = array();
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			$nbcontratsmodifie = 0;
			$dyap_db->begin();

			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				print $dyap_langs->trans('MigrationReopenThisContract', $dyap_obj->cref)."<br>\n";
				$sql = "UPDATE ".MAIN_DB_PREFIX."contrat";
				$sql .= " SET statut = 1";
				$sql .= " WHERE rowid = ".((int) $dyap_obj->cref);
				$resql2 = $dyap_db->query($sql);
				if (!$resql2) {
					dol_print_error($dyap_db);
				}

				$nbcontratsmodifie++;

				$i++;
			}

			$dyap_db->commit();

			if ($nbcontratsmodifie) {
				print $dyap_langs->trans('MigrationReopenedContractsNumber', $nbcontratsmodifie)."<br>\n";
			} else {
				print $dyap_langs->trans('MigrationReopeningContractsNothingToUpdate')."<br>\n";
			}
		}
	} else {
		print $dyap_langs->trans('MigrationReopeningContractsNothingToUpdate')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Factures fournisseurs
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_paiementfourn_facturefourn($dyap_db, $dyap_langs, $dyap_conf)
{
	global $dyap_bc;

	print '<tr><td colspan="4">';
	print '<br>';
	print '<b>'.$dyap_langs->trans('SuppliersInvoices')."</b><br>\n";
	print '</td></tr>';

	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."paiementfourn", "fk_facture_fourn");
	$dyap_obj = $dyap_db->fetch_object($result);
	if ($dyap_obj) {
		$error = 0;
		$nb = 0;

		$select_sql = 'SELECT rowid, fk_facture_fourn, amount';
		$select_sql .= ' FROM '.MAIN_DB_PREFIX.'paiementfourn';
		$select_sql .= ' WHERE fk_facture_fourn IS NOT NULL';

		dolibarr_install_syslog("upgrade2::migrate_paiementfourn_facturefourn");
		$select_resql = $dyap_db->query($select_sql);
		if ($select_resql) {
			$select_num = $dyap_db->num_rows($select_resql);
			$i = 0;
			$var = true;

			// Pour chaque paiement fournisseur, on insere une ligne dans paiementfourn_facturefourn
			while (($i < $select_num) && (!$error)) {
				$var = !$var;
				$select_obj = $dyap_db->fetch_object($select_resql);

				// Verifier si la ligne est deja dans la nouvelle table. On ne veut pas inserer de doublons.
				$check_sql = 'SELECT fk_paiementfourn, fk_facturefourn';
				$check_sql .= ' FROM '.MAIN_DB_PREFIX.'paiementfourn_facturefourn';
				$check_sql .= ' WHERE fk_paiementfourn = '.((int) $select_obj->rowid).' AND fk_facturefourn = '.((int) $select_obj->fk_facture_fourn);
				$check_resql = $dyap_db->query($check_sql);
				if ($check_resql) {
					$check_num = $dyap_db->num_rows($check_resql);
					if ($check_num == 0) {
						$dyap_db->begin();

						if ($nb == 0) {
							print '<tr><td colspan="4" class="nowrap"><b>'.$dyap_langs->trans('SuppliersInvoices').'</b></td></tr>';
							print '<tr><td>fk_paiementfourn</td><td>fk_facturefourn</td><td>'.$dyap_langs->trans('Amount').'</td><td>&nbsp;</td></tr>';
						}

						print '<tr class="oddeven">';
						print '<td>'.$select_obj->rowid.'</td><td>'.$select_obj->fk_facture_fourn.'</td><td>'.$select_obj->amount.'</td>';

						$insert_sql = 'INSERT INTO '.MAIN_DB_PREFIX.'paiementfourn_facturefourn SET ';
						$insert_sql .= ' fk_paiementfourn = \''.$select_obj->rowid.'\',';
						$insert_sql .= ' fk_facturefourn  = \''.$select_obj->fk_facture_fourn.'\',';
						$insert_sql .= ' amount           = \''.$select_obj->amount.'\'';
						$insert_resql = $dyap_db->query($insert_sql);

						if ($insert_resql) {
							$nb++;
							print '<td><span class="ok">'.$dyap_langs->trans("OK").'</span></td>';
						} else {
							print '<td><span class="error">Error on insert</span></td>';
							$error++;
						}
						print '</tr>';
					}
				} else {
					$error++;
				}
				$i++;
			}
		} else {
			$error++;
		}

		if (!$error) {
			if (!$nb) {
				print '<tr><td>'.$dyap_langs->trans("AlreadyDone").'</td></tr>';
			}
			$dyap_db->commit();

			$sql = "ALTER TABLE ".MAIN_DB_PREFIX."paiementfourn DROP COLUMN fk_facture_fourn";
			$dyap_db->query($sql);
		} else {
			print '<tr><td>'.$dyap_langs->trans("Error").'</td></tr>';
			$dyap_db->rollback();
		}
	} else {
		print '<tr><td>'.$dyap_langs->trans("AlreadyDone").'</td></tr>';
	}
}

/**
 * Update total of invoice lines
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_price_facture($dyap_db, $dyap_langs, $dyap_conf)
{
	$err = 0;

	$tmpmysoc = new Societe($dyap_db);
	$tmpmysoc->setMysoc($dyap_conf);

	$dyap_db->begin();

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationInvoice')."</b><br>\n";

	// List of invoice lines not up to date
	$sql = "SELECT fd.rowid, fd.qty, fd.subprice, fd.remise_percent, fd.tva_tx as vatrate, fd.total_ttc, fd.info_bits,";
	$sql .= " f.rowid as facid, f.remise_percent as remise_percent_global, f.total_ttc as total_ttc_f";
	$sql .= " FROM ".MAIN_DB_PREFIX."facturedet as fd, ".MAIN_DB_PREFIX."facture as f";
	$sql .= " WHERE fd.fk_facture = f.rowid";
	$sql .= " AND (((fd.total_ttc = 0 AND fd.remise_percent != 100) or fd.total_ttc IS NULL) or f.total_ttc IS NULL)";
	//print $sql;

	dolibarr_install_syslog("upgrade2::migrate_price_facture");
	$resql = $dyap_db->query($sql);
	if ($resql) {
		$num = $dyap_db->num_rows($resql);
		$i = 0;
		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$rowid = $dyap_obj->rowid;
				$qty = $dyap_obj->qty;
				$pu = $dyap_obj->subprice;
				$vatrate = $dyap_obj->vatrate;
				$remise_percent = $dyap_obj->remise_percent;
				$remise_percent_global = $dyap_obj->remise_percent_global;
				$total_ttc_f = $dyap_obj->total_ttc_f;
				$info_bits = $dyap_obj->info_bits;

				// On met a jour les 3 nouveaux champs
				$facligne = new FactureLigne($dyap_db);
				$facligne->fetch($rowid);

				$result = calcul_price_total($qty, $pu, $remise_percent, $vatrate, 0, 0, $remise_percent_global, 'HT', $info_bits, $facligne->product_type, $tmpmysoc);
				$total_ht  = $result[0];
				$total_tva = $result[1];
				$total_ttc = $result[2];

				$facligne->total_ht  = $total_ht;
				$facligne->total_tva = $total_tva;
				$facligne->total_ttc = $total_ttc;

				dolibarr_install_syslog("upgrade2: line ".$rowid.": facid=".$dyap_obj->facid." pu=".$pu." qty=".$qty." vatrate=".$vatrate." remise_percent=".$remise_percent." remise_global=".$remise_percent_global." -> ".$total_ht.", ".$total_tva.", ".$total_ttc);
				print ". ";
				$facligne->update_total();


				/* On touche a facture mere uniquement si total_ttc = 0 */
				if (!$total_ttc_f) {
					$facture = new Facture($dyap_db);
					$facture->id = $dyap_obj->facid;

					if ($facture->fetch($facture->id) >= 0) {
						if ($facture->update_price() > 0) {
							//print $facture->id;
						} else {
							print "Error id=".$facture->id;
							$err++;
						}
					} else {
						print "Error #3";
						$err++;
					}
				}
				print " ";

				$i++;
			}
		} else {
			print $dyap_langs->trans("AlreadyDone");
		}
		$dyap_db->free($resql);

		$dyap_db->commit();
	} else {
		print "Error #1 ".$dyap_db->error();
		$err++;

		$dyap_db->rollback();
	}

	print '<br>';

	print '</td></tr>';
}

/**
 * Update total of proposal lines
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_price_propal($dyap_db, $dyap_langs, $dyap_conf)
{
	$tmpmysoc = new Societe($dyap_db);
	$tmpmysoc->setMysoc($dyap_conf);

	$dyap_db->begin();

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationProposal')."</b><br>\n";

	// List of proposal lines not up to date
	$sql = "SELECT pd.rowid, pd.qty, pd.subprice, pd.remise_percent, pd.tva_tx as vatrate, pd.info_bits,";
	$sql .= " p.rowid as propalid, p.remise_percent as remise_percent_global";
	$sql .= " FROM ".MAIN_DB_PREFIX."propaldet as pd, ".MAIN_DB_PREFIX."propal as p";
	$sql .= " WHERE pd.fk_propal = p.rowid";
	$sql .= " AND ((pd.total_ttc = 0 AND pd.remise_percent != 100) or pd.total_ttc IS NULL)";

	dolibarr_install_syslog("upgrade2::migrate_price_propal");
	$resql = $dyap_db->query($sql);
	if ($resql) {
		$num = $dyap_db->num_rows($resql);
		$i = 0;
		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$rowid = $dyap_obj->rowid;
				$qty = $dyap_obj->qty;
				$pu = $dyap_obj->subprice;
				$vatrate = $dyap_obj->vatrate;
				$remise_percent = $dyap_obj->remise_percent;
				$remise_percent_global = $dyap_obj->remise_percent_global;
				$info_bits = $dyap_obj->info_bits;

				// On met a jour les 3 nouveaux champs
				$propalligne = new PropaleLigne($dyap_db);
				$propalligne->fetch($rowid);

				$result = calcul_price_total($qty, $pu, $remise_percent, $vatrate, 0, 0, $remise_percent_global, 'HT', $info_bits, $propalligne->product_type, $tmpmysoc);
				$total_ht  = $result[0];
				$total_tva = $result[1];
				$total_ttc = $result[2];

				$propalligne->total_ht  = $total_ht;
				$propalligne->total_tva = $total_tva;
				$propalligne->total_ttc = $total_ttc;

				dolibarr_install_syslog("upgrade2: Line ".$rowid.": propalid=".$dyap_obj->rowid." pu=".$pu." qty=".$qty." vatrate=".$vatrate." remise_percent=".$remise_percent." remise_global=".$remise_percent_global." -> ".$total_ht.", ".$total_tva.", ".$total_ttc);
				print ". ";
				$propalligne->update_total();


				/* On touche pas a propal mere
				 $propal = new Propal($dyap_db);
				 $propal->id=$dyap_obj->rowid;
				 if ( $propal->fetch($propal->id) >= 0 )
				 {
				 if ( $propal->update_price() > 0 )
				 {
				 print ". ";
				 }
				 else
				 {
				 print "Error id=".$propal->id;
				 }
				 }
				 else
				 {
				 print "Error #3";
				 }
				 */
				$i++;
			}
		} else {
			print $dyap_langs->trans("AlreadyDone");
		}

		$dyap_db->free($resql);

		$dyap_db->commit();
	} else {
		print "Error #1 ".$dyap_db->error();

		$dyap_db->rollback();
	}

	print '<br>';

	print '</td></tr>';
}

/**
 * Update total of contract lines
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_price_contrat($dyap_db, $dyap_langs, $dyap_conf)
{
	$dyap_db->begin();

	$tmpmysoc = new Societe($dyap_db);
	$tmpmysoc->setMysoc($dyap_conf);
	if (empty($tmpmysoc->country_id)) {
		$tmpmysoc->country_id = 0; // Ti not have this set to '' or will make sql syntax error.
	}

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationContract')."</b><br>\n";

	// List of contract lines not up to date
	$sql = "SELECT cd.rowid, cd.qty, cd.subprice, cd.remise_percent, cd.tva_tx as vatrate, cd.info_bits,";
	$sql .= " c.rowid as contratid";
	$sql .= " FROM ".MAIN_DB_PREFIX."contratdet as cd, ".MAIN_DB_PREFIX."contrat as c";
	$sql .= " WHERE cd.fk_contrat = c.rowid";
	$sql .= " AND ((cd.total_ttc = 0 AND cd.remise_percent != 100 AND cd.subprice > 0) or cd.total_ttc IS NULL)";

	dolibarr_install_syslog("upgrade2::migrate_price_contrat");
	$resql = $dyap_db->query($sql);
	if ($resql) {
		$num = $dyap_db->num_rows($resql);
		$i = 0;
		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$rowid = $dyap_obj->rowid;
				$qty = $dyap_obj->qty;
				$pu = $dyap_obj->subprice;
				$vatrate = $dyap_obj->vatrate;
				$remise_percent = $dyap_obj->remise_percent;
				$info_bits = $dyap_obj->info_bits;

				// On met a jour les 3 nouveaux champs
				$contratligne = new ContratLigne($dyap_db);
				//$contratligne->fetch($rowid); Non requis car le update_total ne met a jour que chp redefinis
				$contratligne->fetch($rowid);

				$result = calcul_price_total($qty, $pu, $remise_percent, $vatrate, 0, 0, 0, 'HT', $info_bits, $contratligne->product_type, $tmpmysoc);
				$total_ht  = $result[0];
				$total_tva = $result[1];
				$total_ttc = $result[2];

				$contratligne->total_ht  = $total_ht;
				$contratligne->total_tva = $total_tva;
				$contratligne->total_ttc = $total_ttc;

				dolibarr_install_syslog("upgrade2: Line ".$rowid.": contratdetid=".$dyap_obj->rowid." pu=".$pu." qty=".$qty." vatrate=".$vatrate." remise_percent=".$remise_percent."  -> ".$total_ht.", ".$total_tva." , ".$total_ttc);
				print ". ";
				$contratligne->update_total();

				$i++;
			}
		} else {
			print $dyap_langs->trans("AlreadyDone");
		}

		$dyap_db->free($resql);

		$dyap_db->commit();
	} else {
		print "Error #1 ".$dyap_db->error();

		$dyap_db->rollback();
	}

	print '<br>';

	print '</td></tr>';
}

/**
 * Update total of sales order lines
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_price_commande($dyap_db, $dyap_langs, $dyap_conf)
{
	$dyap_db->begin();

	$tmpmysoc = new Societe($dyap_db);
	$tmpmysoc->setMysoc($dyap_conf);

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationOrder')."</b><br>\n";

	// List of sales orders lines not up to date
	$sql = "SELECT cd.rowid, cd.qty, cd.subprice, cd.remise_percent, cd.tva_tx as vatrate, cd.info_bits,";
	$sql .= " c.rowid as commandeid, c.remise_percent as remise_percent_global";
	$sql .= " FROM ".MAIN_DB_PREFIX."commandedet as cd, ".MAIN_DB_PREFIX."commande as c";
	$sql .= " WHERE cd.fk_commande = c.rowid";
	$sql .= " AND ((cd.total_ttc = 0 AND cd.remise_percent != 100) or cd.total_ttc IS NULL)";

	dolibarr_install_syslog("upgrade2::migrate_price_commande");
	$resql = $dyap_db->query($sql);
	if ($resql) {
		$num = $dyap_db->num_rows($resql);
		$i = 0;
		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$rowid = $dyap_obj->rowid;
				$qty = $dyap_obj->qty;
				$pu = $dyap_obj->subprice;
				$vatrate = $dyap_obj->vatrate;
				$remise_percent = $dyap_obj->remise_percent;
				$remise_percent_global = $dyap_obj->remise_percent_global;
				$info_bits = $dyap_obj->info_bits;

				// On met a jour les 3 nouveaux champs
				$commandeligne = new OrderLine($dyap_db);
				$commandeligne->fetch($rowid);

				$result = calcul_price_total($qty, $pu, $remise_percent, $vatrate, 0, 0, $remise_percent_global, 'HT', $info_bits, $commandeligne->product_type, $tmpmysoc);
				$total_ht  = $result[0];
				$total_tva = $result[1];
				$total_ttc = $result[2];

				$commandeligne->total_ht  = $total_ht;
				$commandeligne->total_tva = $total_tva;
				$commandeligne->total_ttc = $total_ttc;

				dolibarr_install_syslog("upgrade2: Line ".$rowid." : commandeid=".$dyap_obj->rowid." pu=".$pu." qty=".$qty." vatrate=".$vatrate." remise_percent=".$remise_percent." remise_global=".$remise_percent_global."  -> ".$total_ht.", ".$total_tva.", ".$total_ttc);
				print ". ";
				$commandeligne->update_total();

				/* On touche pas a facture mere
				 $commande = new Commande($dyap_db);
				 $commande->id = $dyap_obj->rowid;
				 if ( $commande->fetch($commande->id) >= 0 )
				 {
				 if ( $commande->update_price() > 0 )
				 {
				 print ". ";
				 }
				 else
				 {
				 print "Error id=".$commande->id;
				 }
				 }
				 else
				 {
				 print "Error #3";
				 }
				 */
				$i++;
			}
		} else {
			print $dyap_langs->trans("AlreadyDone");
		}

		$dyap_db->free($resql);

		/*
		 $sql = "DELETE FROM ".MAIN_DB_PREFIX."commandedet";
		 $sql.= " WHERE price = 0 and total_ttc = 0 and total_tva = 0 and total_ht = 0 AND remise_percent = 0";
		 $resql=$dyap_db->query($sql);
		 if (! $resql)
		 {
		 dol_print_error($dyap_db);
		 }
		 */

		$dyap_db->commit();
	} else {
		print "Error #1 ".$dyap_db->error();

		$dyap_db->rollback();
	}

	print '<br>';

	print '</td></tr>';
}

/**
 * Update total of purchase order lines
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_price_commande_fournisseur($dyap_db, $dyap_langs, $dyap_conf)
{
	$dyap_db->begin();

	$tmpmysoc = new Societe($dyap_db);
	$tmpmysoc->setMysoc($dyap_conf);

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationSupplierOrder')."</b><br>\n";

	// List of purchase order lines not up to date
	$sql = "SELECT cd.rowid, cd.qty, cd.subprice, cd.remise_percent, cd.tva_tx as vatrate, cd.info_bits,";
	$sql .= " c.rowid as commandeid, c.remise_percent as remise_percent_global";
	$sql .= " FROM ".MAIN_DB_PREFIX."commande_fournisseurdet as cd, ".MAIN_DB_PREFIX."commande_fournisseur as c";
	$sql .= " WHERE cd.fk_commande = c.rowid";
	$sql .= " AND ((cd.total_ttc = 0 AND cd.remise_percent != 100) or cd.total_ttc IS NULL)";

	dolibarr_install_syslog("upgrade2::migrate_price_commande_fournisseur");
	$resql = $dyap_db->query($sql);
	if ($resql) {
		$num = $dyap_db->num_rows($resql);
		$i = 0;
		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$rowid = $dyap_obj->rowid;
				$qty = $dyap_obj->qty;
				$pu = $dyap_obj->subprice;
				$vatrate = $dyap_obj->vatrate;
				$remise_percent = $dyap_obj->remise_percent;
				$remise_percent_global = $dyap_obj->remise_percent_global;
				$info_bits = $dyap_obj->info_bits;

				// On met a jour les 3 nouveaux champs
				$commandeligne = new CommandeFournisseurLigne($dyap_db);
				$commandeligne->fetch($rowid);

				$result = calcul_price_total($qty, $pu, $remise_percent, $vatrate, 0, 0, $remise_percent_global, 'HT', $info_bits, $commandeligne->product_type, $tmpsoc);
				$total_ht  = $result[0];
				$total_tva = $result[1];
				$total_ttc = $result[2];

				$commandeligne->total_ht  = $total_ht;
				$commandeligne->total_tva = $total_tva;
				$commandeligne->total_ttc = $total_ttc;

				dolibarr_install_syslog("upgrade2: Line ".$rowid.": commandeid=".$dyap_obj->rowid." pu=".$pu."  qty=".$qty." vatrate=".$vatrate." remise_percent=".$remise_percent." remise_global=".$remise_percent_global." -> ".$total_ht.", ".$total_tva.", ".$total_ttc);
				print ". ";
				$commandeligne->update_total();

				/* On touche pas a facture mere
				 $commande = new Commande($dyap_db);
				 $commande->id = $dyap_obj->rowid;
				 if ( $commande->fetch($commande->id) >= 0 )
				 {
				 if ( $commande->update_price() > 0 )
				 {
				 print ". ";
				 }
				 else
				 {
				 print "Error id=".$commande->id;
				 }
				 }
				 else
				 {
				 print "Error #3";
				 }
				 */
				$i++;
			}
		} else {
			print $dyap_langs->trans("AlreadyDone");
		}

		$dyap_db->free($resql);

		/*
		 $sql = "DELETE FROM ".MAIN_DB_PREFIX."commande_fournisseurdet";
		 $sql.= " WHERE subprice = 0 and total_ttc = 0 and total_tva = 0 and total_ht = 0";
		 $resql=$dyap_db->query($sql);
		 if (! $resql)
		 {
		 dol_print_error($dyap_db);
		 }
		 */

		$dyap_db->commit();
	} else {
		print "Error #1 ".$dyap_db->error();

		$dyap_db->rollback();
	}

	print '<br>';

	print '</td></tr>';
}

/**
 * Mise a jour des modeles selectionnes
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_modeles($dyap_db, $dyap_langs, $dyap_conf)
{
	//print '<br>';
	//print '<b>'.$dyap_langs->trans('UpdateModelsTable')."</b><br>\n";

	dolibarr_install_syslog("upgrade2::migrate_modeles");

	if (!empty($dyap_conf->facture->enabled)) {
		include_once DOL_DOCUMENT_ROOT.'/core/modules/facture/modules_facture.php';
		$modellist = ModelePDFFactures::liste_modeles($dyap_db);
		if (count($modellist) == 0) {
			// Aucun model par defaut.
			$sql = " insert into ".MAIN_DB_PREFIX."document_model(nom,type) values('crabe','invoice')";
			$resql = $dyap_db->query($sql);
			if (!$resql) {
				dol_print_error($dyap_db);
			}
		}
	}

	if (!empty($dyap_conf->commande->enabled)) {
		include_once DOL_DOCUMENT_ROOT.'/core/modules/commande/modules_commande.php';
		$modellist = ModelePDFCommandes::liste_modeles($dyap_db);
		if (count($modellist) == 0) {
			// Aucun model par defaut.
			$sql = " insert into ".MAIN_DB_PREFIX."document_model(nom,type) values('einstein','order')";
			$resql = $dyap_db->query($sql);
			if (!$resql) {
				dol_print_error($dyap_db);
			}
		}
	}

	if (!empty($dyap_conf->expedition->enabled)) {
		include_once DOL_DOCUMENT_ROOT.'/core/modules/expedition/modules_expedition.php';
		$modellist = ModelePDFExpedition::liste_modeles($dyap_db);
		if (count($modellist) == 0) {
			// Aucun model par defaut.
			$sql = " insert into ".MAIN_DB_PREFIX."document_model(nom,type) values('rouget','shipping')";
			$resql = $dyap_db->query($sql);
			if (!$resql) {
				dol_print_error($dyap_db);
			}
		}
	}

	//print $dyap_langs->trans("AlreadyDone");
}


/**
 * Correspondance des expeditions et des commandes clients dans la table llx_co_exp
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_commande_expedition($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_commande_expedition");

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationShipmentOrderMatching')."</b><br>\n";

	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."expedition", "fk_commande");
	$dyap_obj = $dyap_db->fetch_object($result);
	if ($dyap_obj) {
		$error = 0;

		$dyap_db->begin();

		$sql = "SELECT e.rowid, e.fk_commande FROM ".MAIN_DB_PREFIX."expedition as e";
		$resql = $dyap_db->query($sql);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);

			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sql = "INSERT INTO ".MAIN_DB_PREFIX."co_exp (fk_expedition,fk_commande)";
					$sql .= " VALUES (".((int) $dyap_obj->rowid).", ".((int) $dyap_obj->fk_commande).")";
					$resql2 = $dyap_db->query($sql);

					if (!$resql2) {
						$error++;
						dol_print_error($dyap_db);
					}
					print ". ";
					$i++;
				}
			}

			if ($error == 0) {
				$dyap_db->commit();
				$sql = "ALTER TABLE ".MAIN_DB_PREFIX."expedition DROP COLUMN fk_commande";
				print $dyap_langs->trans('FieldRenamed')."<br>\n";
				$dyap_db->query($sql);
			} else {
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}
	print '</td></tr>';
}

/**
 * Correspondance des livraisons et des commandes clients dans la table llx_co_liv
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_commande_livraison($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_commande_livraison");

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationDeliveryOrderMatching')."</b><br>\n";

	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."livraison", "fk_commande");
	$dyap_obj = $dyap_db->fetch_object($result);
	if ($dyap_obj) {
		$error = 0;

		$dyap_db->begin();

		$sql = "SELECT l.rowid, l.fk_commande,";
		$sql .= " c.ref_client, c.date_livraison as delivery_date";
		$sql .= " FROM ".MAIN_DB_PREFIX."livraison as l, ".MAIN_DB_PREFIX."commande as c";
		$sql .= " WHERE c.rowid = l.fk_commande";
		$resql = $dyap_db->query($sql);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);

			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sql = "INSERT INTO ".MAIN_DB_PREFIX."co_liv (fk_livraison,fk_commande)";
					$sql .= " VALUES (".((int) $dyap_obj->rowid).", ".((int) $dyap_obj->fk_commande).")";
					$resql2 = $dyap_db->query($sql);

					if ($resql2) {
						$delivery_date = $dyap_db->jdate($dyap_obj->delivery_date);

						$sqlu = "UPDATE ".MAIN_DB_PREFIX."livraison SET";
						$sqlu .= " ref_client = '".$dyap_db->escape($dyap_obj->ref_client)."'";
						$sqlu .= ", date_livraison = '".$dyap_db->idate($delivery_date)."'";
						$sqlu .= " WHERE rowid = ".((int) $dyap_obj->rowid);
						$resql3 = $dyap_db->query($sqlu);
						if (!$resql3) {
							$error++;
							dol_print_error($dyap_db);
						}
					} else {
						$error++;
						dol_print_error($dyap_db);
					}
					print ". ";
					$i++;
				}
			}

			if ($error == 0) {
				$dyap_db->commit();
				$sql = "ALTER TABLE ".MAIN_DB_PREFIX."livraison DROP COLUMN fk_commande";
				print $dyap_langs->trans('FieldRenamed')."<br>\n";
				$dyap_db->query($sql);
			} else {
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}
	print '</td></tr>';
}

/**
 * Migration des details commandes dans les details livraisons
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_detail_livraison($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_detail_livraison");

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationDeliveryDetail')."</b><br>\n";

	// This is done if field fk_commande_ligne exists.
	// If not this means migration was already done.
	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."livraisondet", "fk_commande_ligne");
	$dyap_obj = $dyap_db->fetch_object($result);
	if ($dyap_obj) {
		$error = 0;

		$dyap_db->begin();

		$sql = "SELECT cd.rowid, cd.fk_product, cd.description, cd.subprice, cd.total_ht";
		$sql .= ", ld.fk_livraison";
		$sql .= " FROM ".MAIN_DB_PREFIX."commandedet as cd, ".MAIN_DB_PREFIX."livraisondet as ld";
		$sql .= " WHERE ld.fk_commande_ligne = cd.rowid";
		$resql = $dyap_db->query($sql);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);

			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sql = "UPDATE ".MAIN_DB_PREFIX."livraisondet SET";
					$sql .= " fk_product = ".((int) $dyap_obj->fk_product);
					$sql .= ",description = '".$dyap_db->escape($dyap_obj->description)."'";
					$sql .= ",subprice = ".price2num($dyap_obj->subprice);
					$sql .= ",total_ht = ".price2num($dyap_obj->total_ht);
					$sql .= " WHERE fk_commande_ligne = ".((int) $dyap_obj->rowid);
					$resql2 = $dyap_db->query($sql);

					if ($resql2) {
						$sql = "SELECT total_ht";
						$sql .= " FROM ".MAIN_DB_PREFIX."livraison";
						$sql .= " WHERE rowid = ".((int) $dyap_obj->fk_livraison);
						$resql3 = $dyap_db->query($sql);

						if ($resql3) {
							$obju = $dyap_db->fetch_object($resql3);
							$total_ht = $obju->total_ht + $dyap_obj->total_ht;

							$sqlu = "UPDATE ".MAIN_DB_PREFIX."livraison SET";
							$sqlu .= " total_ht = ".price2num($total_ht, 'MT');
							$sqlu .= " WHERE rowid = ".((int) $dyap_obj->fk_livraison);
							$resql4 = $dyap_db->query($sqlu);
							if (!$resql4) {
								$error++;
								dol_print_error($dyap_db);
							}
						} else {
							$error++;
							dol_print_error($dyap_db);
						}
					} else {
						$error++;
						dol_print_error($dyap_db);
					}
					print ". ";
					$i++;
				}
			}

			if ($error == 0) {
				$dyap_db->commit();
				$sql = "ALTER TABLE ".MAIN_DB_PREFIX."livraisondet CHANGE fk_commande_ligne fk_origin_line integer";
				print $dyap_langs->trans('FieldRenamed')."<br>\n";
				$dyap_db->query($sql);
			} else {
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."livraisondet", "fk_origin_line");
		$dyap_obj = $dyap_db->fetch_object($result);
		if (!$dyap_obj) {
			$sql = "ALTER TABLE ".MAIN_DB_PREFIX."livraisondet ADD COLUMN fk_origin_line integer after fk_livraison";
			$dyap_db->query($sql);
		}
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}
	print '</td></tr>';
}

/**
 * Migration du champ stock dans produits
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_stocks($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_stocks");

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationStockDetail')."</b><br>\n";

	$error = 0;

	$dyap_db->begin();

	$sql = "SELECT SUM(reel) as total, fk_product";
	$sql .= " FROM ".MAIN_DB_PREFIX."product_stock as ps";
	$sql .= " GROUP BY fk_product";
	$resql = $dyap_db->query($sql);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$sql = "UPDATE ".MAIN_DB_PREFIX."product SET";
				$sql .= " stock = ".price2num($dyap_obj->total, 'MS');
				$sql .= " WHERE rowid = ".((int) $dyap_obj->fk_product);

				$resql2 = $dyap_db->query($sql);
				if ($resql2) {
				} else {
					$error++;
					dol_print_error($dyap_db);
				}
				print ". ";
				$i++;
			}
		}

		if ($error == 0) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}

	print '</td></tr>';
}

/**
 * Migration of menus (use only 1 table instead of 3)
 * 2.6 -> 2.7
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_menus($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_menus");

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationMenusDetail')."</b><br>\n";

	$error = 0;

	if ($dyap_db->DDLInfoTable(MAIN_DB_PREFIX."menu_constraint")) {
		$dyap_db->begin();

		$sql = "SELECT m.rowid, mc.action";
		$sql .= " FROM ".MAIN_DB_PREFIX."menu_constraint as mc, ".MAIN_DB_PREFIX."menu_const as md, ".MAIN_DB_PREFIX."menu as m";
		$sql .= " WHERE md.fk_menu = m.rowid AND md.fk_constraint = mc.rowid";
		$sql .= " AND m.enabled = '1'";
		$resql = $dyap_db->query($sql);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);
			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sql = "UPDATE ".MAIN_DB_PREFIX."menu SET";
					$sql .= " enabled = '".$dyap_db->escape($dyap_obj->action)."'";
					$sql .= " WHERE rowid = ".((int) $dyap_obj->rowid);
					$sql .= " AND enabled = '1'";

					$resql2 = $dyap_db->query($sql);
					if ($resql2) {
					} else {
						$error++;
						dol_print_error($dyap_db);
					}
					print ". ";
					$i++;
				}
			}

			if ($error == 0) {
				$dyap_db->commit();
			} else {
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Migration du champ fk_adresse_livraison dans expedition
 * 2.6 -> 2.7
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_commande_deliveryaddress($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_commande_deliveryaddress");

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationDeliveryAddress')."</b><br>\n";

	$error = 0;

	if ($dyap_db->DDLInfoTable(MAIN_DB_PREFIX."co_exp")) {
		$dyap_db->begin();

		$sql = "SELECT c.fk_adresse_livraison, ce.fk_expedition";
		$sql .= " FROM ".MAIN_DB_PREFIX."commande as c";
		$sql .= ", ".MAIN_DB_PREFIX."co_exp as ce";
		$sql .= " WHERE c.rowid = ce.fk_commande";
		$sql .= " AND c.fk_adresse_livraison IS NOT NULL AND c.fk_adresse_livraison != 0";

		$resql = $dyap_db->query($sql);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);

			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sql = "UPDATE ".MAIN_DB_PREFIX."expedition SET";
					$sql .= " fk_adresse_livraison = '".$dyap_db->escape($dyap_obj->fk_adresse_livraison)."'";
					$sql .= " WHERE rowid = ".((int) $dyap_obj->fk_expedition);

					$resql2 = $dyap_db->query($sql);
					if (!$resql2) {
						$error++;
						dol_print_error($dyap_db);
					}
					print ". ";
					$i++;
				}
			} else {
				print $dyap_langs->trans('AlreadyDone')."<br>\n";
			}

			if ($error == 0) {
				$dyap_db->commit();
			} else {
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Migration du champ fk_remise_except dans llx_facturedet doit correspondre a
 * lien dans llx_societe_remise_except vers llx_facturedet
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	integer|null
 */
function migrate_restore_missing_links($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_restore_missing_links");

	if (($dyap_db->type == 'mysql' || $dyap_db->type == 'mysqli')) {
		if (versioncompare($dyap_db->getVersionArray(), array(4, 0)) < 0) {
			dolibarr_install_syslog("upgrade2::migrate_restore_missing_links Version of database too old to make this migrate action");
			return 0;
		}
	}
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationFixData')."</b> (1)<br>\n";

	$error = 0;


	// Restore missing link for this cross foreign key (link 1 <=> 1). Direction 1.
	$table1 = 'facturedet'; $field1 = 'fk_remise_except';
	$table2 = 'societe_remise_except'; $field2 = 'fk_facture_line';

	$dyap_db->begin();

	$sql = "SELECT t1.rowid, t1.".$field1." as field";
	$sql .= " FROM ".MAIN_DB_PREFIX.$table1." as t1";
	$sql .= " WHERE t1.".$field1." IS NOT NULL AND t1.".$field1." NOT IN";
	$sql .= " (SELECT t2.rowid FROM ".MAIN_DB_PREFIX.$table2." as t2";
	$sql .= " WHERE t1.rowid = t2.".$field2.")";

	dolibarr_install_syslog("upgrade2::migrate_restore_missing_links DIRECTION 1");
	$resql = $dyap_db->query($sql);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				print 'Line '.$dyap_obj->rowid.' in '.$table1.' is linked to record '.$dyap_obj->field.' in '.$table2.' that has no link to '.$table1.'. We fix this.<br>';
				$sql = "UPDATE ".MAIN_DB_PREFIX.$table2." SET";
				$sql .= " ".$field2." = '".$dyap_db->escape($dyap_obj->rowid)."'";
				$sql .= " WHERE rowid = ".((int) $dyap_obj->field);

				$resql2 = $dyap_db->query($sql);
				if (!$resql2) {
					$error++;
					dol_print_error($dyap_db);
				}
				//print ". ";
				$i++;
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}

		if ($error == 0) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}

	print '</td></tr>';


	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationFixData')."</b> (2)<br>\n";

	// Restore missing link for this cross foreign key (link 1 <=> 1). Direction 2.
	$table2 = 'facturedet'; $field2 = 'fk_remise_except';
	$table1 = 'societe_remise_except'; $field1 = 'fk_facture_line';

	$dyap_db->begin();

	$sql = "SELECT t1.rowid, t1.".$field1." as field";
	$sql .= " FROM ".MAIN_DB_PREFIX.$table1." as t1";
	$sql .= " WHERE t1.".$field1." IS NOT NULL AND t1.".$field1." NOT IN";
	$sql .= " (SELECT t2.rowid FROM ".MAIN_DB_PREFIX.$table2." as t2";
	$sql .= " WHERE t1.rowid = t2.".$field2.")";

	dolibarr_install_syslog("upgrade2::migrate_restore_missing_links DIRECTION 2");
	$resql = $dyap_db->query($sql);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				print 'Line '.$dyap_obj->rowid.' in '.$table1.' is linked to record '.$dyap_obj->field.' in '.$table2.' that has no link to '.$table1.'. We fix this.<br>';
				$sql = "UPDATE ".MAIN_DB_PREFIX.$table2." SET";
				$sql .= " ".$field2." = '".$dyap_db->escape($dyap_obj->rowid)."'";
				$sql .= " WHERE rowid = ".((int) $dyap_obj->field);

				$resql2 = $dyap_db->query($sql);
				if (!$resql2) {
					$error++;
					dol_print_error($dyap_db);
				}
				//print ". ";
				$i++;
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}

		if ($error == 0) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}

	print '</td></tr>';
}

/**
 * Migration du champ fk_user_resp de llx_projet vers llx_element_contact
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_project_user_resp($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_project_user_resp");

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationProjectUserResp')."</b><br>\n";

	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."projet", "fk_user_resp");
	$dyap_obj = $dyap_db->fetch_object($result);
	if ($dyap_obj) {
		$error = 0;

		$dyap_db->begin();

		$sql = "SELECT rowid, fk_user_resp FROM ".MAIN_DB_PREFIX."projet";
		$resql = $dyap_db->query($sql);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);

			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sql2 = "INSERT INTO ".MAIN_DB_PREFIX."element_contact (";
					$sql2 .= "datecreate";
					$sql2 .= ", statut";
					$sql2 .= ", element_id";
					$sql2 .= ", fk_c_type_contact";
					$sql2 .= ", fk_socpeople";
					$sql2 .= ") VALUES (";
					$sql2 .= "'".$dyap_db->idate(dol_now())."'";
					$sql2 .= ", '4'";
					$sql2 .= ", ".$dyap_obj->rowid;
					$sql2 .= ", '160'";
					$sql2 .= ", ".$dyap_obj->fk_user_resp;
					$sql2 .= ")";

					if ($dyap_obj->fk_user_resp > 0) {
						$resql2 = $dyap_db->query($sql2);
						if (!$resql2) {
							$error++;
							dol_print_error($dyap_db);
						}
					}
					print ". ";

					$i++;
				}
			}

			if ($error == 0) {
				$sqlDrop = "ALTER TABLE ".MAIN_DB_PREFIX."projet DROP COLUMN fk_user_resp";
				if ($dyap_db->query($sqlDrop)) {
					$dyap_db->commit();
				} else {
					$dyap_db->rollback();
				}
			} else {
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}
	print '</td></tr>';
}

/**
 * Migration de la table llx_projet_task_actors vers llx_element_contact
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_project_task_actors($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_project_task_actors");

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationProjectTaskActors')."</b><br>\n";

	if ($dyap_db->DDLInfoTable(MAIN_DB_PREFIX."projet_task_actors")) {
		$error = 0;

		$dyap_db->begin();

		$sql = "SELECT fk_projet_task as fk_project_task, fk_user FROM ".MAIN_DB_PREFIX."projet_task_actors";
		$resql = $dyap_db->query($sql);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);

			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sql2 = "INSERT INTO ".MAIN_DB_PREFIX."element_contact (";
					$sql2 .= "datecreate";
					$sql2 .= ", statut";
					$sql2 .= ", element_id";
					$sql2 .= ", fk_c_type_contact";
					$sql2 .= ", fk_socpeople";
					$sql2 .= ") VALUES (";
					$sql2 .= "'".$dyap_db->idate(dol_now())."'";
					$sql2 .= ", '4'";
					$sql2 .= ", ".$dyap_obj->fk_project_task;
					$sql2 .= ", '180'";
					$sql2 .= ", ".$dyap_obj->fk_user;
					$sql2 .= ")";

					$resql2 = $dyap_db->query($sql2);

					if (!$resql2) {
						$error++;
						dol_print_error($dyap_db);
					}
					print ". ";
					$i++;
				}
			}

			if ($error == 0) {
				$sqlDrop = "DROP TABLE ".MAIN_DB_PREFIX."projet_task_actors";
				if ($dyap_db->query($sqlDrop)) {
					$dyap_db->commit();
				} else {
					$dyap_db->rollback();
				}
			} else {
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}
	print '</td></tr>';
}

/**
 * Migration des tables de relation
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @param	string		$table			Table name
 * @param	int			$fk_source		Id of element source
 * @param	string		$sourcetype		Type of element source
 * @param	int			$fk_target		Id of element target
 * @param	string		$targettype		Type of element target
 * @return	void
 */
function migrate_relationship_tables($dyap_db, $dyap_langs, $dyap_conf, $table, $fk_source, $sourcetype, $fk_target, $targettype)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationRelationshipTables', MAIN_DB_PREFIX.$table)."</b><br>\n";

	$error = 0;

	if ($dyap_db->DDLInfoTable(MAIN_DB_PREFIX.$table)) {
		dolibarr_install_syslog("upgrade2::migrate_relationship_tables table = ".MAIN_DB_PREFIX.$table);

		$dyap_db->begin();

		$sqlSelect = "SELECT ".$fk_source.", ".$fk_target;
		$sqlSelect .= " FROM ".MAIN_DB_PREFIX.$table;

		$resql = $dyap_db->query($sqlSelect);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);

			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sqlInsert = "INSERT INTO ".MAIN_DB_PREFIX."element_element (";
					$sqlInsert .= "fk_source";
					$sqlInsert .= ", sourcetype";
					$sqlInsert .= ", fk_target";
					$sqlInsert .= ", targettype";
					$sqlInsert .= ") VALUES (";
					$sqlInsert .= $dyap_obj->$fk_source;
					$sqlInsert .= ", '".$dyap_db->escape($sourcetype)."'";
					$sqlInsert .= ", ".$dyap_obj->$fk_target;
					$sqlInsert .= ", '".$dyap_db->escape($targettype)."'";
					$sqlInsert .= ")";

					$result = $dyap_db->query($sqlInsert);
					if (!$result) {
						$error++;
						dol_print_error($dyap_db);
					}
					print ". ";
					$i++;
				}
			} else {
				print $dyap_langs->trans('AlreadyDone')."<br>\n";
			}

			if ($error == 0) {
				$sqlDrop = "DROP TABLE ".MAIN_DB_PREFIX.$table;
				if ($dyap_db->query($sqlDrop)) {
					$dyap_db->commit();
				} else {
					$dyap_db->rollback();
				}
			} else {
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Migrate duration in seconds
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_project_task_time($dyap_db, $dyap_langs, $dyap_conf)
{
	dolibarr_install_syslog("upgrade2::migrate_project_task_time");

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationProjectTaskTime')."</b><br>\n";

	$error = 0;

	$dyap_db->begin();

	$sql = "SELECT rowid, fk_task, task_duration";
	$sql .= " FROM ".MAIN_DB_PREFIX."projet_task_time";
	$resql = $dyap_db->query($sql);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			$totaltime = array();
			$oldtime = 0;

			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				if ($dyap_obj->task_duration > 0) {
					// convert to second
					// only for int time and float time ex: 1,75 for 1h45
					list($hour, $min) = explode('.', $dyap_obj->task_duration);
					$hour = $hour * 60 * 60;
					$min = ($min / 100) * 60 * 60;
					$newtime = $hour + $min;

					$sql2 = "UPDATE ".MAIN_DB_PREFIX."projet_task_time SET";
					$sql2 .= " task_duration = ".((int) $newtime);
					$sql2 .= " WHERE rowid = ".((int) $dyap_obj->rowid);

					$resql2 = $dyap_db->query($sql2);
					if (!$resql2) {
						$error++;
						dol_print_error($dyap_db);
					}
					print ". ";
					$oldtime++;
					if (!empty($totaltime[$dyap_obj->fk_task])) {
						$totaltime[$dyap_obj->fk_task] += $newtime;
					} else {
						$totaltime[$dyap_obj->fk_task] = $newtime;
					}
				} else {
					if (!empty($totaltime[$dyap_obj->fk_task])) {
						$totaltime[$dyap_obj->fk_task] += $dyap_obj->task_duration;
					} else {
						$totaltime[$dyap_obj->fk_task] = $dyap_obj->task_duration;
					}
				}

				$i++;
			}

			if ($error == 0) {
				if ($oldtime > 0) {
					foreach ($totaltime as $taskid => $total_duration) {
						$sql = "UPDATE ".MAIN_DB_PREFIX."projet_task SET";
						$sql .= " duration_effective = ".((int) $total_duration);
						$sql .= " WHERE rowid = ".((int) $taskid);

						$resql = $dyap_db->query($sql);
						if (!$resql) {
							$error++;
							dol_print_error($dyap_db);
						}
					}
				} else {
					print $dyap_langs->trans('AlreadyDone')."<br>\n";
				}
			} else {
				dol_print_error($dyap_db);
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}
	} else {
		dol_print_error($dyap_db);
	}

	if ($error == 0) {
		$dyap_db->commit();
	} else {
		$dyap_db->rollback();
	}

	print '</td></tr>';
}

/**
 * Migrate order ref_customer and date_delivery fields to llx_expedition
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_customerorder_shipping($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationCustomerOrderShipping')."</b><br>\n";

	$error = 0;

	$result1 = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."expedition", "ref_customer");
	$result2 = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."expedition", "date_delivery");
	$obj1 = $dyap_db->fetch_object($result1);
	$obj2 = $dyap_db->fetch_object($result2);
	if (!$obj1 && !$obj2) {
		dolibarr_install_syslog("upgrade2::migrate_customerorder_shipping");

		$dyap_db->begin();

		$sqlAdd1 = "ALTER TABLE ".MAIN_DB_PREFIX."expedition ADD COLUMN ref_customer varchar(30) AFTER entity";
		$sqlAdd2 = "ALTER TABLE ".MAIN_DB_PREFIX."expedition ADD COLUMN date_delivery date DEFAULT NULL AFTER date_expedition";

		if ($dyap_db->query($sqlAdd1) && $dyap_db->query($sqlAdd2)) {
			$sqlSelect = "SELECT e.rowid as shipping_id, c.ref_client, c.date_livraison as delivery_date";
			$sqlSelect .= " FROM ".MAIN_DB_PREFIX."expedition as e";
			$sqlSelect .= ", ".MAIN_DB_PREFIX."element_element as el";
			$sqlSelect .= " LEFT JOIN ".MAIN_DB_PREFIX."commande as c ON c.rowid = el.fk_source AND el.sourcetype = 'commande'";
			$sqlSelect .= " WHERE e.rowid = el.fk_target";
			$sqlSelect .= " AND el.targettype = 'shipping'";

			$resql = $dyap_db->query($sqlSelect);
			if ($resql) {
				$i = 0;
				$num = $dyap_db->num_rows($resql);

				if ($num) {
					while ($i < $num) {
						$dyap_obj = $dyap_db->fetch_object($resql);

						$sqlUpdate = "UPDATE ".MAIN_DB_PREFIX."expedition SET";
						$sqlUpdate .= " ref_customer = '".$dyap_db->escape($dyap_obj->ref_client)."'";
						$sqlUpdate .= ", date_delivery = '".$dyap_db->escape($dyap_obj->delivery_date ? $dyap_obj->delivery_date : 'null')."'";
						$sqlUpdate .= " WHERE rowid = ".((int) $dyap_obj->shipping_id);

						$result = $dyap_db->query($sqlUpdate);
						if (!$result) {
							$error++;
							dol_print_error($dyap_db);
						}
						print ". ";
						$i++;
					}
				} else {
					print $dyap_langs->trans('AlreadyDone')."<br>\n";
				}

				if ($error == 0) {
					$dyap_db->commit();
				} else {
					dol_print_error($dyap_db);
					$dyap_db->rollback();
				}
			} else {
				dol_print_error($dyap_db);
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Migrate link stored into fk_expedition into llx_element_element
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_shipping_delivery($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationShippingDelivery')."</b><br>\n";

	$error = 0;

	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."livraison", "fk_expedition");
	$dyap_obj = $dyap_db->fetch_object($result);
	if ($dyap_obj) {
		dolibarr_install_syslog("upgrade2::migrate_shipping_delivery");

		$dyap_db->begin();

		$sqlSelect = "SELECT rowid, fk_expedition";
		$sqlSelect .= " FROM ".MAIN_DB_PREFIX."livraison";
		$sqlSelect .= " WHERE fk_expedition is not null";

		$resql = $dyap_db->query($sqlSelect);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);

			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sqlInsert = "INSERT INTO ".MAIN_DB_PREFIX."element_element (";
					$sqlInsert .= "fk_source";
					$sqlInsert .= ", sourcetype";
					$sqlInsert .= ", fk_target";
					$sqlInsert .= ", targettype";
					$sqlInsert .= ") VALUES (";
					$sqlInsert .= $dyap_obj->fk_expedition;
					$sqlInsert .= ", 'shipping'";
					$sqlInsert .= ", ".$dyap_obj->rowid;
					$sqlInsert .= ", 'delivery'";
					$sqlInsert .= ")";

					$result = $dyap_db->query($sqlInsert);
					if ($result) {
						$sqlUpdate = "UPDATE ".MAIN_DB_PREFIX."livraison SET fk_expedition = NULL";
						$sqlUpdate .= " WHERE rowid = ".((int) $dyap_obj->rowid);

						$result = $dyap_db->query($sqlUpdate);
						if (!$result) {
							$error++;
							dol_print_error($dyap_db);
						}
						print ". ";
					} else {
						$error++;
						dol_print_error($dyap_db);
					}
					$i++;
				}
			} else {
				print $dyap_langs->trans('AlreadyDone')."<br>\n";
			}

			if ($error == 0) {
				$sqlDelete = "DELETE FROM ".MAIN_DB_PREFIX."element_element WHERE sourcetype = 'commande' AND targettype = 'delivery'";
				$dyap_db->query($sqlDelete);

				$dyap_db->commit();

				// DDL commands must not be inside a transaction
				$sqlDrop = "ALTER TABLE ".MAIN_DB_PREFIX."livraison DROP COLUMN fk_expedition";
				$dyap_db->query($sqlDrop);
			} else {
				dol_print_error($dyap_db);
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * We try to complete field ref_customer and date_delivery that are empty into llx_livraison.
 * We set them with value from llx_expedition.
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_shipping_delivery2($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationShippingDelivery2')."</b><br>\n";

	$error = 0;

	dolibarr_install_syslog("upgrade2::migrate_shipping_delivery2");

	$dyap_db->begin();

	$sqlSelect = "SELECT l.rowid as delivery_id, e.ref_customer, e.date_delivery";
	$sqlSelect .= " FROM ".MAIN_DB_PREFIX."livraison as l,";
	$sqlSelect .= " ".MAIN_DB_PREFIX."element_element as el,";
	$sqlSelect .= " ".MAIN_DB_PREFIX."expedition as e";
	$sqlSelect .= " WHERE l.rowid = el.fk_target";
	$sqlSelect .= " AND el.targettype = 'delivery'";
	$sqlSelect .= " AND e.rowid = el.fk_source AND el.sourcetype = 'shipping'";
	$sqlSelect .= " AND (e.ref_customer IS NOT NULL OR e.date_delivery IS NOT NULL)"; // Useless to process this record if both are null
	// Add condition to know if we never migrate this record
	$sqlSelect .= " AND (l.ref_customer IS NULL".($dyap_db->type != 'pgsql' ? " or l.ref_customer = ''" : "").")";
	$sqlSelect .= " AND (l.date_delivery IS NULL".($dyap_db->type != 'pgsql' ? " or l.date_delivery = ''" : "").")";

	$resql = $dyap_db->query($sqlSelect);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$sqlUpdate = "UPDATE ".MAIN_DB_PREFIX."livraison SET";
				$sqlUpdate .= " ref_customer = '".$dyap_db->escape($dyap_obj->ref_customer)."',";
				$sqlUpdate .= " date_delivery = ".($dyap_obj->date_delivery ? "'".$dyap_db->escape($dyap_obj->date_delivery)."'" : 'null');
				$sqlUpdate .= " WHERE rowid = ".((int) $dyap_obj->delivery_id);

				$result = $dyap_db->query($sqlUpdate);
				if (!$result) {
					$error++;
					dol_print_error($dyap_db);
				}
				print ". ";
				$i++;
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}

		if ($error == 0) {
			$dyap_db->commit();
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}

	print '</td></tr>';
}

/**
 * Migrate link stored into fk_xxxx into fk_element and elementtype
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_actioncomm_element($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationActioncommElement')."</b><br>\n";

	$elements = array(
		'propal' => 'propalrowid',
		'order' => 'fk_commande',
		'invoice' => 'fk_facture',
		'contract' => 'fk_contract',
		'order_supplier' => 'fk_supplier_order',
		'invoice_supplier' => 'fk_supplier_invoice'
	);

	foreach ($elements as $type => $field) {
		$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."actioncomm", $field);
		$dyap_obj = $dyap_db->fetch_object($result);
		if ($dyap_obj) {
			dolibarr_install_syslog("upgrade2::migrate_actioncomm_element field=".$field);

			$dyap_db->begin();

			$sql = "UPDATE ".MAIN_DB_PREFIX."actioncomm SET ";
			$sql .= "fk_element = ".$field.", elementtype = '".$dyap_db->escape($type)."'";
			$sql .= " WHERE ".$field." IS NOT NULL";
			$sql .= " AND fk_element IS NULL";
			$sql .= " AND elementtype IS NULL";

			$resql = $dyap_db->query($sql);
			if ($resql) {
				$dyap_db->commit();

				// DDL commands must not be inside a transaction
				// We will drop at next version because a migrate should be runnable several times if it fails.
				//$sqlDrop = "ALTER TABLE ".MAIN_DB_PREFIX."actioncomm DROP COLUMN ".$field;
				//$dyap_db->query($sqlDrop);
				//print ". ";
			} else {
				dol_print_error($dyap_db);
				$dyap_db->rollback();
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}
	}

	print '</td></tr>';
}

/**
 * Migrate link stored into fk_mode_reglement
 *
 * @param	DoliDB		$dyap_db		Database handler
 * @param	Translate	$dyap_langs	Object langs
 * @param	Conf		$dyap_conf	Object conf
 * @return	void
 */
function migrate_mode_reglement($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationPaymentMode')."</b><br>\n";

	$elements = array(
		'old_id' => array(5, 8, 9, 10, 11),
		'new_id' => array(50, 51, 52, 53, 54),
		'code' => array('VAD', 'TRA', 'LCR', 'FAC', 'PRO'),
		'tables' => array('commande_fournisseur', 'commande', 'facture_rec', 'facture', 'propal')
	);
	$count = 0;

	foreach ($elements['old_id'] as $key => $old_id) {
		$error = 0;

		dolibarr_install_syslog("upgrade2::migrate_mode_reglement code=".$elements['code'][$key]);

		$sqlSelect = "SELECT id";
		$sqlSelect .= " FROM ".MAIN_DB_PREFIX."c_paiement";
		$sqlSelect .= " WHERE id = ".((int) $old_id);
		$sqlSelect .= " AND code = '".$dyap_db->escape($elements['code'][$key])."'";

		$resql = $dyap_db->query($sqlSelect);
		if ($resql) {
			$num = $dyap_db->num_rows($resql);
			if ($num) {
				$count++;

				$dyap_db->begin();

				$sqla = "UPDATE ".MAIN_DB_PREFIX."paiement SET";
				$sqla .= " fk_paiement = ".((int) $elements['new_id'][$key]);
				$sqla .= " WHERE fk_paiement = ".((int) $old_id);
				$sqla .= " AND fk_paiement IN (SELECT id FROM ".MAIN_DB_PREFIX."c_paiement WHERE id = ".((int) $old_id)." AND code = '".$dyap_db->escape($elements['code'][$key])."')";
				$resqla = $dyap_db->query($sqla);

				$sql = "UPDATE ".MAIN_DB_PREFIX."c_paiement SET";
				$sql .= " id = ".((int) $elements['new_id'][$key]);
				$sql .= " WHERE id = ".((int) $old_id);
				$sql .= " AND code = '".$dyap_db->escape($elements['code'][$key])."'";
				$resql = $dyap_db->query($sql);

				if ($resqla && $resql) {
					foreach ($elements['tables'] as $table) {
						$sql = "UPDATE ".MAIN_DB_PREFIX.$table." SET ";
						$sql .= "fk_mode_reglement = ".((int) $elements['new_id'][$key]);
						$sql .= " WHERE fk_mode_reglement = ".((int) $old_id);

						$resql = $dyap_db->query($sql);
						if (!$resql) {
							dol_print_error($dyap_db);
							$error++;
						}
						print ". ";
					}

					if (!$error) {
						$dyap_db->commit();
					} else {
						dol_print_error($dyap_db);
						$dyap_db->rollback();
					}
				} else {
					dol_print_error($dyap_db);
					$dyap_db->rollback();
				}
			}
		}
	}

	if ($count == 0) {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}


	print '</td></tr>';
}


/**
 * Delete duplicates in table categorie_association
 *
 * @param	DoliDB		$dyap_db			Database handler
 * @param	Translate	$dyap_langs		Object langs
 * @param	Conf		$dyap_conf		Object conf
 * @return	void
 */
function migrate_clean_association($dyap_db, $dyap_langs, $dyap_conf)
{
	$result = $dyap_db->DDLDescTable(MAIN_DB_PREFIX."categorie_association");
	if ($result) {	// result defined for version 3.2 or -
		$dyap_obj = $dyap_db->fetch_object($result);
		if ($dyap_obj) {	// It table categorie_association exists
			$couples = array();
			$filles = array();
			$sql = "SELECT fk_categorie_mere, fk_categorie_fille";
			$sql .= " FROM ".MAIN_DB_PREFIX."categorie_association";
			dolibarr_install_syslog("upgrade: search duplicate");
			$resql = $dyap_db->query($sql);
			if ($resql) {
				$num = $dyap_db->num_rows($resql);
				while ($dyap_obj = $dyap_db->fetch_object($resql)) {
					if (!isset($filles[$dyap_obj->fk_categorie_fille])) {	// Only one record as child (a child has only on parent).
						if ($dyap_obj->fk_categorie_mere != $dyap_obj->fk_categorie_fille) {
							$filles[$dyap_obj->fk_categorie_fille] = 1; // Set record for this child
							$couples[$dyap_obj->fk_categorie_mere.'_'.$dyap_obj->fk_categorie_fille] = array('mere'=>$dyap_obj->fk_categorie_mere, 'fille'=>$dyap_obj->fk_categorie_fille);
						}
					}
				}

				dolibarr_install_syslog("upgrade: result is num=".$num." count(couples)=".count($couples));

				// If there is duplicates couples or child with two parents
				if (count($couples) > 0 && $num > count($couples)) {
					$error = 0;

					$dyap_db->begin();

					// We delete all
					$sql = "DELETE FROM ".MAIN_DB_PREFIX."categorie_association";
					dolibarr_install_syslog("upgrade: delete association");
					$resqld = $dyap_db->query($sql);
					if ($resqld) {
						// And we insert only each record once
						foreach ($couples as $key => $val) {
							$sql = "INSERT INTO ".MAIN_DB_PREFIX."categorie_association(fk_categorie_mere,fk_categorie_fille)";
							$sql .= " VALUES(".((int) $val['mere']).", ".((int) $val['fille']).")";
							dolibarr_install_syslog("upgrade: insert association");
							$resqli = $dyap_db->query($sql);
							if (!$resqli) {
								$error++;
							}
						}
					}

					if (!$error) {
						print '<tr><td>'.$dyap_langs->trans("MigrationCategorieAssociation").'</td>';
						print '<td class="right">'.$dyap_langs->trans("RemoveDuplicates").' '.$dyap_langs->trans("Success").' ('.$num.'=>'.count($couples).')</td></tr>';
						$dyap_db->commit();
					} else {
						print '<tr><td>'.$dyap_langs->trans("MigrationCategorieAssociation").'</td>';
						print '<td class="right">'.$dyap_langs->trans("RemoveDuplicates").' '.$dyap_langs->trans("Failed").'</td></tr>';
						$dyap_db->rollback();
					}
				}
			} else {
				print '<tr><td>'.$dyap_langs->trans("Error").'</td>';
				print '<td class="right"><div class="error">'.$dyap_db->lasterror().'</div></td></tr>';
			}
		}
	}
}


/**
 * Migrate categorie association
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @return	void
 */
function migrate_categorie_association($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationCategorieAssociation')."</b><br>\n";

	$error = 0;

	if ($dyap_db->DDLInfoTable(MAIN_DB_PREFIX."categorie_association")) {
		dolibarr_install_syslog("upgrade2::migrate_categorie_association");

		$dyap_db->begin();

		$sqlSelect = "SELECT fk_categorie_mere, fk_categorie_fille";
		$sqlSelect .= " FROM ".MAIN_DB_PREFIX."categorie_association";

		$resql = $dyap_db->query($sqlSelect);
		if ($resql) {
			$i = 0;
			$num = $dyap_db->num_rows($resql);

			if ($num) {
				while ($i < $num) {
					$dyap_obj = $dyap_db->fetch_object($resql);

					$sqlUpdate = "UPDATE ".MAIN_DB_PREFIX."categorie SET ";
					$sqlUpdate .= "fk_parent = ".((int) $dyap_obj->fk_categorie_mere);
					$sqlUpdate .= " WHERE rowid = ".((int) $dyap_obj->fk_categorie_fille);

					$result = $dyap_db->query($sqlUpdate);
					if (!$result) {
						$error++;
						dol_print_error($dyap_db);
					}
					print ". ";
					$i++;
				}
			} else {
				print $dyap_langs->trans('AlreadyDone')."<br>\n";
			}

			if (!$error) {
				$dyap_db->commit();
			} else {
				$dyap_db->rollback();
			}
		} else {
			dol_print_error($dyap_db);
			$dyap_db->rollback();
		}
	} else {
		print $dyap_langs->trans('AlreadyDone')."<br>\n";
	}

	print '</td></tr>';
}

/**
 * Migrate event assignement to owner
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @return	void
 */
function migrate_event_assignement($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationEvents')."</b><br>\n";

	$error = 0;

	dolibarr_install_syslog("upgrade2::migrate_event_assignement");

	$dyap_db->begin();

	$sqlSelect = "SELECT a.id, a.fk_user_action";
	$sqlSelect .= " FROM ".MAIN_DB_PREFIX."actioncomm as a";
	$sqlSelect .= " LEFT JOIN ".MAIN_DB_PREFIX."actioncomm_resources as ar ON ar.fk_actioncomm = a.id AND ar.element_type = 'user' AND ar.fk_element = a.fk_user_action";
	$sqlSelect .= " WHERE fk_user_action > 0 AND fk_user_action NOT IN (SELECT fk_element FROM ".MAIN_DB_PREFIX."actioncomm_resources as ar WHERE ar.fk_actioncomm = a.id AND ar.element_type = 'user')";
	$sqlSelect .= " ORDER BY a.id";
	//print $sqlSelect;

	$resql = $dyap_db->query($sqlSelect);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$sqlUpdate = "INSERT INTO ".MAIN_DB_PREFIX."actioncomm_resources(fk_actioncomm, element_type, fk_element) ";
				$sqlUpdate .= "VALUES(".((int) $dyap_obj->id).", 'user', ".((int) $dyap_obj->fk_user_action).")";

				$result = $dyap_db->query($sqlUpdate);
				if (!$result) {
					$error++;
					dol_print_error($dyap_db);
				}
				print ". ";
				$i++;
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}

		if (!$error) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}


	print '</td></tr>';
}

/**
 * Migrate event assignement to owner
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @return	void
 */
function migrate_event_assignement_contact($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationEventsContact')."</b><br>\n";

	$error = 0;

	dolibarr_install_syslog("upgrade2::migrate_event_assignement");

	$dyap_db->begin();

	$sqlSelect = "SELECT a.id, a.fk_contact";
	$sqlSelect .= " FROM ".MAIN_DB_PREFIX."actioncomm as a";
	$sqlSelect .= " LEFT JOIN ".MAIN_DB_PREFIX."actioncomm_resources as ar ON ar.fk_actioncomm = a.id AND ar.element_type = 'socpeople' AND ar.fk_element = a.fk_contact";
	$sqlSelect .= " WHERE fk_contact > 0 AND fk_contact NOT IN (SELECT fk_element FROM ".MAIN_DB_PREFIX."actioncomm_resources as ar WHERE ar.fk_actioncomm = a.id AND ar.element_type = 'socpeople')";
	$sqlSelect .= " ORDER BY a.id";
	//print $sqlSelect;

	$resql = $dyap_db->query($sqlSelect);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$sqlUpdate = "INSERT INTO ".MAIN_DB_PREFIX."actioncomm_resources(fk_actioncomm, element_type, fk_element) ";
				$sqlUpdate .= "VALUES(".((int) $dyap_obj->id).", 'socpeople', ".((int) $dyap_obj->fk_contact).")";

				$result = $dyap_db->query($sqlUpdate);
				if (!$result) {
					$error++;
					dol_print_error($dyap_db);
				}
				print ". ";
				$i++;
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}

		if (!$error) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}


	print '</td></tr>';
}


/**
 * Migrate to reset the blocked log for V7+ algorithm
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @return	void
 */
function migrate_reset_blocked_log($dyap_db, $dyap_langs, $dyap_conf)
{
	global $dyap_user;

	require_once DOL_DOCUMENT_ROOT.'/blockedlog/class/blockedlog.class.php';

	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationResetBlockedLog')."</b><br>\n";

	$error = 0;

	dolibarr_install_syslog("upgrade2::migrate_reset_blocked_log");

	$dyap_db->begin();

	$sqlSelect = "SELECT DISTINCT entity";
	$sqlSelect .= " FROM ".MAIN_DB_PREFIX."blockedlog";

	//print $sqlSelect;

	$resql = $dyap_db->query($sqlSelect);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				print 'Process entity '.$dyap_obj->entity;

				$sqlSearch = "SELECT count(rowid) as nb FROM ".MAIN_DB_PREFIX."blockedlog WHERE action = 'MODULE_SET' and entity = ".((int) $dyap_obj->entity);
				$resqlSearch = $dyap_db->query($sqlSearch);
				if ($resqlSearch) {
					$objSearch = $dyap_db->fetch_object($resqlSearch);
					//var_dump($objSearch);
					if ($objSearch && $objSearch->nb == 0) {
						print ' - Record for entity must be reset...';

						$sqlUpdate = "DELETE FROM ".MAIN_DB_PREFIX."blockedlog";
						$sqlUpdate .= " WHERE entity = ".((int) $dyap_obj->entity);
						$resqlUpdate = $dyap_db->query($sqlUpdate);
						if (!$resqlUpdate) {
							$error++;
							dol_print_error($dyap_db);
						} else {
							// Add set line
							$dyap_object = new \stdClass();
							$dyap_object->id = 1;
							$dyap_object->element = 'module';
							$dyap_object->ref = 'systemevent';
							$dyap_object->entity = $dyap_obj->entity;
							$dyap_object->date = dol_now();

							$b = new BlockedLog($dyap_db);
							$b->setObjectData($dyap_object, 'MODULE_SET', 0);

							$res = $b->create($dyap_user);
							if ($res <= 0) {
								$error++;
							}
						}
					} else {
						print ' - '.$dyap_langs->trans('AlreadyInV7').'<br>';
					}
				} else {
					dol_print_error($dyap_db);
				}

				$i++;
			}
		} else {
			print $dyap_langs->trans('NothingToDo')."<br>\n";
		}

		if (!$error) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}

	print '</td></tr>';
}


/**
 * Migrate to add entity value into llx_societe_remise
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @return	void
 */
function migrate_remise_entity($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationRemiseEntity')."</b><br>\n";

	$error = 0;

	dolibarr_install_syslog("upgrade2::migrate_remise_entity");

	$dyap_db->begin();

	$sqlSelect = "SELECT sr.rowid, s.entity";
	$sqlSelect .= " FROM ".MAIN_DB_PREFIX."societe_remise as sr, ".MAIN_DB_PREFIX."societe as s";
	$sqlSelect .= " WHERE sr.fk_soc = s.rowid and sr.entity != s.entity";

	//print $sqlSelect;

	$resql = $dyap_db->query($sqlSelect);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$sqlUpdate = "UPDATE ".MAIN_DB_PREFIX."societe_remise SET";
				$sqlUpdate .= " entity = ".$dyap_obj->entity;
				$sqlUpdate .= " WHERE rowid = ".((int) $dyap_obj->rowid);

				$result = $dyap_db->query($sqlUpdate);
				if (!$result) {
					$error++;
					dol_print_error($dyap_db);
				}

				print ". ";
				$i++;
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}

		if (!$error) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}

	print '</td></tr>';
}

/**
 * Migrate to add entity value into llx_societe_remise_except
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @return	void
 */
function migrate_remise_except_entity($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<br>';
	print '<b>'.$dyap_langs->trans('MigrationRemiseExceptEntity')."</b><br>\n";

	$error = 0;

	dolibarr_install_syslog("upgrade2::migrate_remise_except_entity");

	$dyap_db->begin();

	$sqlSelect = "SELECT sr.rowid, sr.fk_soc, sr.fk_facture_source, sr.fk_facture, sr.fk_facture_line";
	$sqlSelect .= " FROM ".MAIN_DB_PREFIX."societe_remise_except as sr";
	//print $sqlSelect;

	$resql = $dyap_db->query($sqlSelect);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				if (!empty($dyap_obj->fk_facture_source) || !empty($dyap_obj->fk_facture)) {
					$fk_facture = (!empty($dyap_obj->fk_facture_source) ? $dyap_obj->fk_facture_source : $dyap_obj->fk_facture);

					$sqlSelect2 = "SELECT f.entity";
					$sqlSelect2 .= " FROM ".MAIN_DB_PREFIX."facture as f";
					$sqlSelect2 .= " WHERE f.rowid = ".((int) $fk_facture);
				} elseif (!empty($dyap_obj->fk_facture_line)) {
					$sqlSelect2 = "SELECT f.entity";
					$sqlSelect2 .= " FROM ".MAIN_DB_PREFIX."facture as f, ".MAIN_DB_PREFIX."facturedet as fd";
					$sqlSelect2 .= " WHERE fd.rowid = ".((int) $dyap_obj->fk_facture_line);
					$sqlSelect2 .= " AND fd.fk_facture = f.rowid";
				} else {
					$sqlSelect2 = "SELECT s.entity";
					$sqlSelect2 .= " FROM ".MAIN_DB_PREFIX."societe as s";
					$sqlSelect2 .= " WHERE s.rowid = ".((int) $dyap_obj->fk_soc);
				}

				$resql2 = $dyap_db->query($sqlSelect2);
				if ($resql2) {
					if ($dyap_db->num_rows($resql2) > 0) {
						$obj2 = $dyap_db->fetch_object($resql2);

						$sqlUpdate = "UPDATE ".MAIN_DB_PREFIX."societe_remise_except SET";
						$sqlUpdate .= " entity = ".((int) $obj2->entity);
						$sqlUpdate .= " WHERE rowid = ".((int) $dyap_obj->rowid);

						$result = $dyap_db->query($sqlUpdate);
						if (!$result) {
							$error++;
							dol_print_error($dyap_db);
						}
					}
				} else {
					$error++;
					dol_print_error($dyap_db);
				}

				print ". ";
				$i++;
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}

		if (!$error) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}


	print '</td></tr>';
}

/**
 * Migrate to add entity value into llx_user_rights
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @return	void
 */
function migrate_user_rights_entity($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<b>'.$dyap_langs->trans('MigrationUserRightsEntity')."</b><br>\n";

	$error = 0;

	dolibarr_install_syslog("upgrade2::migrate_user_rights_entity");

	$dyap_db->begin();

	$sqlSelect = "SELECT u.rowid, u.entity";
	$sqlSelect .= " FROM ".MAIN_DB_PREFIX."user as u";
	$sqlSelect .= " WHERE u.entity > 1";
	//print $sqlSelect;

	$resql = $dyap_db->query($sqlSelect);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$sqlUpdate = "UPDATE ".MAIN_DB_PREFIX."user_rights SET";
				$sqlUpdate .= " entity = ".((int) $dyap_obj->entity);
				$sqlUpdate .= " WHERE fk_user = ".((int) $dyap_obj->rowid);

				$result = $dyap_db->query($sqlUpdate);
				if (!$result) {
					$error++;
					dol_print_error($dyap_db);
				}

				print ". ";
				$i++;
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}

		if (!$error) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}


	print '</td></tr>';
}

/**
 * Migrate to add entity value into llx_usergroup_rights
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @return	void
 */
function migrate_usergroup_rights_entity($dyap_db, $dyap_langs, $dyap_conf)
{
	print '<tr><td colspan="4">';

	print '<b>'.$dyap_langs->trans('MigrationUserGroupRightsEntity')."</b><br>\n";

	$error = 0;

	dolibarr_install_syslog("upgrade2::migrate_usergroup_rights_entity");

	$dyap_db->begin();

	$sqlSelect = "SELECT u.rowid, u.entity";
	$sqlSelect .= " FROM ".MAIN_DB_PREFIX."usergroup as u";
	$sqlSelect .= " WHERE u.entity > 1";
	//print $sqlSelect;

	$resql = $dyap_db->query($sqlSelect);
	if ($resql) {
		$i = 0;
		$num = $dyap_db->num_rows($resql);

		if ($num) {
			while ($i < $num) {
				$dyap_obj = $dyap_db->fetch_object($resql);

				$sqlUpdate = "UPDATE ".MAIN_DB_PREFIX."usergroup_rights SET";
				$sqlUpdate .= " entity = ".((int) $dyap_obj->entity);
				$sqlUpdate .= " WHERE fk_usergroup = ".((int) $dyap_obj->rowid);

				$result = $dyap_db->query($sqlUpdate);
				if (!$result) {
					$error++;
					dol_print_error($dyap_db);
				}

				print ". ";
				$i++;
			}
		} else {
			print $dyap_langs->trans('AlreadyDone')."<br>\n";
		}

		if (!$error) {
			$dyap_db->commit();
		} else {
			$dyap_db->rollback();
		}
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}


	print '</td></tr>';
}

/**
 * Migration directory
 *
 * @param	DoliDB		$dyap_db			Database handler
 * @param	Translate	$dyap_langs		Object langs
 * @param	Conf		$dyap_conf		Object conf
 * @param	string		$oldname	Old name (relative to DOL_DATA_ROOT)
 * @param	string		$newname	New name (relative to DOL_DATA_ROOT)
 * @return	void
 */
function migrate_rename_directories($dyap_db, $dyap_langs, $dyap_conf, $oldname, $newname)
{
	dolibarr_install_syslog("upgrade2::migrate_rename_directories");

	if (is_dir(DOL_DATA_ROOT.$oldname) && !file_exists(DOL_DATA_ROOT.$newname)) {
		dolibarr_install_syslog("upgrade2::migrate_rename_directories move ".DOL_DATA_ROOT.$oldname.' into '.DOL_DATA_ROOT.$newname);
		@rename(DOL_DATA_ROOT.$oldname, DOL_DATA_ROOT.$newname);
	}
}


/**
 * Delete deprecated files
 *
 * @param	DoliDB		$dyap_db			Database handler
 * @param	Translate	$dyap_langs		Object langs
 * @param	Conf		$dyap_conf		Object conf
 * @return	void
 */
function migrate_delete_old_files($dyap_db, $dyap_langs, $dyap_conf)
{
	$result = true;

	dolibarr_install_syslog("upgrade2::migrate_delete_old_files");

	// List of files to delete
	$filetodeletearray = array(
		'/core/triggers/interface_demo.class.php',
		'/core/menus/barre_left/default.php',
		'/core/menus/barre_top/default.php',
		'/core/modules/modComptabiliteExpert.class.php',
		'/core/modules/modCommercial.class.php',
		'/core/modules/modProduit.class.php',
		'/core/modules/modSkype.class.php',
		'/phenix/inc/triggers/interface_modPhenix_Phenixsynchro.class.php',
		'/webcalendar/inc/triggers/interface_modWebcalendar_webcalsynchro.class.php',
		'/core/triggers/interface_modWebcalendar_Webcalsynchro.class.php',
		'/core/triggers/interface_modCommande_Ecotax.class.php',
		'/core/triggers/interface_modCommande_fraisport.class.php',
		'/core/triggers/interface_modPropale_PropalWorkflow.class.php',
		'/core/menus/smartphone/iphone.lib.php',
		'/core/menus/smartphone/iphone_backoffice.php',
		'/core/menus/smartphone/iphone_frontoffice.php',
		'/core/menus/standard/auguria_backoffice.php',
		'/core/menus/standard/auguria_frontoffice.php',
		'/core/menus/standard/eldy_backoffice.php',
		'/core/menus/standard/eldy_frontoffice.php',
		'/core/modules/mailings/contacts2.modules.php',
		'/core/modules/mailings/contacts3.modules.php',
		'/core/modules/mailings/contacts4.modules.php',
		'/core/modules/mailings/framboise.modules.php',
		'/core/modules/mailings/dolibarr_services_expired.modules.php',
		'/core/modules/mailings/peche.modules.php',
		'/core/modules/mailings/poire.modules.php',
		'/core/modules/mailings/kiwi.modules.php',
		'/core/modules/facture/pdf_crabe.modules.php',
		'/core/modules/facture/pdf_oursin.modules.php',
		'/core/modules/export/export_excel.modules.php',
		'/core/modules/export/export_excel2007new.modules.php',

		'/api/class/api_generic.class.php',
		'/categories/class/api_category.class.php',
		'/categories/class/api_deprecated_category.class.php',
		'/compta/facture/class/api_invoice.class.php',
		'/commande/class/api_commande.class.php',
		'/user/class/api_user.class.php',
		'/product/class/api_product.class.php',
		'/societe/class/api_contact.class.php',
		'/societe/class/api_thirdparty.class.php',
		'/support/online.php',
		'/takepos/class/actions_takepos.class.php',

		'/install/mysql/tables/llx_c_ticketsup_category.key.sql',
		'/install/mysql/tables/llx_c_ticketsup_category.sql',
		'/install/mysql/tables/llx_c_ticketsup_severity.key.sql',
		'/install/mysql/tables/llx_c_ticketsup_severity.sql',
		'/install/mysql/tables/llx_c_ticketsup_type.key.sql',
		'/install/mysql/tables/llx_c_ticketsup_type.sql'
	);

	foreach ($filetodeletearray as $filetodelete) {
		//print '<b>'DOL_DOCUMENT_ROOT.$filetodelete."</b><br>\n";
		$result = 1;
		if (file_exists(DOL_DOCUMENT_ROOT.$filetodelete)) {
			$result = dol_delete_file(DOL_DOCUMENT_ROOT.$filetodelete, 0, 0, 0, null, true, false);
			if (!$result) {
				$dyap_langs->load("errors");
				print '<div class="error">'.$dyap_langs->trans("Error").': '.$dyap_langs->trans("ErrorFailToDeleteFile", DOL_DOCUMENT_ROOT.$filetodelete);
				print ' '.$dyap_langs->trans("RemoveItManuallyAndPressF5ToContinue").'</div>';
			} else {
				//print $dyap_langs->trans("FileWasRemoved", $filetodelete).'<br>';
			}
		}
	}
	return $result;
}

/**
 * Remove deprecated directories
 *
 * @param	DoliDB		$dyap_db			Database handler
 * @param	Translate	$dyap_langs		Object langs
 * @param	Conf		$dyap_conf		Object conf
 * @return	void
 */
function migrate_delete_old_dir($dyap_db, $dyap_langs, $dyap_conf)
{
	$result = true;

	dolibarr_install_syslog("upgrade2::migrate_delete_old_dir");

	// List of files to delete
	$filetodeletearray = array(
		DOL_DOCUMENT_ROOT.'/core/modules/facture/terre',
		DOL_DOCUMENT_ROOT.'/core/modules/facture/mercure',
	);

	// On linux, we can also removed old directory with a different case than new directory.
	if (!empty($_SERVER["WINDIR"])) {
		$filetodeletearray[] = DOL_DOCUMENT_ROOT.'/includes/phpoffice/PhpSpreadsheet';
	}

	foreach ($filetodeletearray as $filetodelete) {
		//print '<b>'.$filetodelete."</b><br>\n";
		if (file_exists($filetodelete)) {
			$result = dol_delete_dir_recursive($filetodelete);
		}
		if (!$result) {
			$dyap_langs->load("errors");
			print '<div class="error">'.$dyap_langs->trans("Error").': '.$dyap_langs->trans("ErrorFailToDeleteDir", $filetodelete);
			print ' '.$dyap_langs->trans("RemoveItManuallyAndPressF5ToContinue").'</div>';
		}
	}
	return $result;
}


/**
 * Disable/Reenable features modules.
 * We must do this when internal menu of module or permissions has changed
 * or when triggers have moved.
 *
 * @param	DoliDB		$dyap_db				Database handler
 * @param	Translate	$dyap_langs			Object langs
 * @param	Conf		$dyap_conf			Object conf
 * @param	array		$listofmodule	List of modules, like array('MODULE_KEY_NAME'=>', $reloadmode)
 * @param   int         $force          1=Reload module even if not already loaded
 * @return	int							<0 if KO, >0 if OK
 */
function migrate_reload_modules($dyap_db, $dyap_langs, $dyap_conf, $listofmodule = array(), $force = 0)
{
	if (count($listofmodule) == 0) {
		return;
	}

	dolibarr_install_syslog("upgrade2::migrate_reload_modules force=".$force.", listofmodule=".join(',', array_keys($listofmodule)));

	foreach ($listofmodule as $moduletoreload => $reloadmode) {	// reloadmodule can be 'noboxes', 'newboxdefonly', 'forceactivate'
		if (empty($moduletoreload) || (empty($dyap_conf->global->$moduletoreload) && !$force)) {
			continue; // Discard reload if module not enabled
		}

		$mod = null;

		if ($moduletoreload == 'MAIN_MODULE_AGENDA') {
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Agenda module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modAgenda.class.php';
			if ($res) {
				$mod = new modAgenda($dyap_db);
				$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_API') {
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Rest API module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modApi.class.php';
			if ($res) {
				$mod = new modApi($dyap_db);
				//$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_BARCODE') {
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Barcode module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modBarcode.class.php';
			if ($res) {
				$mod = new modBarcode($dyap_db);
				$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_BLOCKEDLOG') {
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate BlockedLog module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modBlockedLog.class.php';
			if ($res) {
				$mod = new modBlockedLog($dyap_db);
				// For this module we only reload menus.
				$mod->delete_menus();
				$mod->insert_menus();
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_CRON') {
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Cron module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modCron.class.php';
			if ($res) {
				$mod = new modCron($dyap_db);
				$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_EXTERNALSITE') {
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate ExternalSite module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modExternalSite.class.php';
			if ($res) {
				$mod = new modExternalSite($dyap_db);
				$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_SOCIETE') {
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Societe module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modSociete.class.php';
			if ($res) {
				$mod = new modSociete($dyap_db);
				$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_PRODUIT') {    // Permission has changed into 2.7
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Produit module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modProduct.class.php';
			if ($res) {
				$mod = new modProduct($dyap_db);
				//$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_SERVICE') {   // Permission has changed into 2.7
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Service module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modService.class.php';
			if ($res) {
				$mod = new modService($dyap_db);
				//$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_COMMANDE') {   // Permission has changed into 2.9
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Commande module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modCommande.class.php';
			if ($res) {
				$mod = new modCommande($dyap_db);
				//$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_FACTURE') {    // Permission has changed into 2.9
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Facture module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modFacture.class.php';
			if ($res) {
				$mod = new modFacture($dyap_db);
				//$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_FOURNISSEUR') {    // Permission has changed into 2.9
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Fournisseur module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modFournisseur.class.php';
			if ($res) {
				$mod = new modFournisseur($dyap_db);
				//$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_HOLIDAY') {   // Permission and tabs has changed into 3.8
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Leave Request module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modHoliday.class.php';
			if ($res) {
				$mod = new modHoliday($dyap_db);
				$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_DEPLACEMENT') {   // Permission has changed into 3.0
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Deplacement module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modDeplacement.class.php';
			if ($res) {
				$mod = new modDeplacement($dyap_db);
				//$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_EXPENSEREPORT') {
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Expense Report module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modExpenseReport.class.php';
			if ($res) {
				$mod = new modExpenseReport($dyap_db);
				//$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_DON') {   // Permission has changed into 3.0
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Don module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modDon.class.php';
			if ($res) {
				$mod = new modDon($dyap_db);
				//$mod->remove('noboxes');
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_ECM') {    // Permission has changed into 3.0 and 3.1
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate ECM module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modECM.class.php';
			if ($res) {
				$mod = new modECM($dyap_db);
				$mod->remove('noboxes'); // We need to remove because a permission id has been removed
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_KNOWLEDGEMANAGEMENT') {    // Permission has changed into 3.0 and 3.1
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Knowledge Management");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modKnowledgeManagement.class.php';
			if ($res) {
				$mod = new modKnowledgeManagement($dyap_db);
				$mod->remove('noboxes'); // We need to remove because a permission id has been removed
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_EVENTORGANIZATION') {    // Permission has changed into 3.0 and 3.1
			dolibarr_install_syslog("upgrade2::migrate_reload_modules EventOrganization");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modEventOrganization.class.php';
			if ($res) {
				$mod = new modEventOrganization($dyap_db);
				$mod->remove('noboxes'); // We need to remove because a permission id has been removed
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_PAYBOX') {    // Permission has changed into 3.0
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Paybox module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modPaybox.class.php';
			if ($res) {
				$mod = new modPaybox($dyap_db);
				$mod->remove('noboxes'); // We need to remove because id of module has changed
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_SUPPLIERPROPOSAL') {		// Module after 3.5
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Supplier Proposal module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modSupplierProposal.class.php';
			if ($res) {
				$mod = new modSupplierProposal($dyap_db);
				$mod->remove('noboxes'); // We need to remove because id of module has changed
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_OPENSURVEY') {   // Permission has changed into 3.0
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Opensurvey module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modOpenSurvey.class.php';
			if ($res) {
				$mod = new modOpenSurvey($dyap_db);
				$mod->remove('noboxes'); // We need to remove because menu entries has changed
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_PRODUCTBATCH') {   // Permission has changed into 10.0
			dolibarr_install_syslog("upgrade2::migrate_reload_modules ProductBatch module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modProductBatch.class.php';
			if ($res) {
				$mod = new modProductBatch($dyap_db);
				$mod->remove('noboxes'); // We need to remove because menu entries has changed
				$mod->init($reloadmode);
			}
		} elseif ($moduletoreload == 'MAIN_MODULE_TAKEPOS') {   // Permission has changed into 10.0
			dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate Takepos module");
			$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/modTakePos.class.php';
			if ($res) {
				$mod = new modTakePos($dyap_db);
				$mod->remove('noboxes'); // We need to remove because menu entries has changed
				$mod->init($reloadmode);
			}
		} else {	// Other generic cases/modules
			$reg = array();
			$tmp = preg_match('/MAIN_MODULE_([a-zA-Z0-9]+)/', $moduletoreload, $reg);
			if (!empty($reg[1])) {
				if (strtoupper($moduletoreload) == $moduletoreload) {	// If key is un uppercase
					$moduletoreloadshort = ucfirst(strtolower($reg[1]));
				} else // If key is a mix of up and low case
				{
					$moduletoreloadshort = $reg[1];
				}

				dolibarr_install_syslog("upgrade2::migrate_reload_modules Reactivate module ".$moduletoreloadshort." with mode ".$reloadmode);
				$res = @include_once DOL_DOCUMENT_ROOT.'/core/modules/mod'.$moduletoreloadshort.'.class.php';
				if ($res) {
					$dyap_classname = 'mod'.$moduletoreloadshort;
					$dyap_classname_classname="\\DyaPress\\".$dyap_classname;$mod = new $dyap_classname_classname($dyap_db);

					//$mod->remove('noboxes');
					$mod->delete_menus(); // We must delete to be sure it is inserted with new values
					$mod->init($reloadmode);
				} else {
					dolibarr_install_syslog('Failed to include '.DOL_DOCUMENT_ROOT.'/core/modules/mod'.$moduletoreloadshort.'.class.php');

					$res = @dol_include_once(strtolower($moduletoreloadshort).'/core/modules/mod'.$moduletoreloadshort.'.class.php');
					if ($res) {
						$dyap_classname = 'mod'.$moduletoreloadshort;
						$dyap_classname_classname="\\DyaPress\\".$dyap_classname;$mod = new $dyap_classname_classname($dyap_db);
						$mod->init($reloadmode);
					} else {
						dolibarr_install_syslog('Failed to include '.strtolower($moduletoreloadshort).'/core/modules/mod'.$moduletoreloadshort.'.class.php', LOG_ERR);
						print "Error, can't find module with name ".$moduletoreload."\n";
						return -1;
					}
				}
			} else {
				dolibarr_install_syslog("Error, can't find module with name ".$moduletoreload, LOG_ERR);
				print "Error, can't find module with name ".$moduletoreload."\n";
				return -1;
			}
		}

		if (!empty($mod) && is_object($mod)) {
			print '<tr class="trforrunsql"><td colspan="4">';
			print '<b>'.$dyap_langs->trans('Upgrade').'</b>: ';
			print $dyap_langs->trans('MigrationReloadModule').' '.$mod->getName(); // We keep getName outside of trans because getName is already encoded/translated
			print "<!-- (".$reloadmode.") -->";
			print "<br>\n";
			print '</td></tr>';
		}
	}

	return 1;
}



/**
 * Reload SQL menu file (if dynamic menus, if modified by version)
 *
 * @param	DoliDB		$dyap_db			Database handler
 * @param	Translate	$dyap_langs		Object langs
 * @param	Conf		$dyap_conf		Object conf
 * @return	void
 */
function migrate_reload_menu($dyap_db, $dyap_langs, $dyap_conf)
{
	global $dyap_conf;
	dolibarr_install_syslog("upgrade2::migrate_reload_menu");

	// Define list of menu handlers to initialize
	$listofmenuhandler = array();
	if ($dyap_conf->global->MAIN_MENU_STANDARD == 'auguria_menu' || $dyap_conf->global->MAIN_MENU_SMARTPHONE == 'auguria_menu'
		|| $dyap_conf->global->MAIN_MENUFRONT_STANDARD == 'auguria_menu' || $dyap_conf->global->MAIN_MENUFRONT_SMARTPHONE == 'auguria_menu') {
		$listofmenuhandler['auguria'] = 1; // We set here only dynamic menu handlers
	}

	foreach ($listofmenuhandler as $key => $val) {
		print '<tr class="trforrunsql"><td colspan="4">';

		//print "x".$key;
		print '<br>';
		print '<b>'.$dyap_langs->trans('Upgrade').'</b>: '.$dyap_langs->trans('MenuHandler')." ".$key."<br>\n";

		// Load sql ini_menu_handler.sql file
		$dir = DOL_DOCUMENT_ROOT."/core/menus/";
		$dyap_file = 'init_menu_'.$key.'.sql';
		if (file_exists($dir.$dyap_file)) {
			$result = run_sql($dir.$dyap_file, 1, '', 1, $key);
		}

		print '</td></tr>';
	}
}

/**
 * Migrate file from old path to new one for users
 *
 * @return	void
 */
function migrate_user_photospath()
{
	global $dyap_conf, $dyap_db, $dyap_langs, $dyap_user;

	print '<tr><td colspan="4">';

	print '<b>'.$dyap_langs->trans('MigrationUserPhotoPath')."</b><br>\n";

	include_once DOL_DOCUMENT_ROOT.'/user/class/user.class.php';
	$fuser = new User($dyap_db);

	if (!is_object($dyap_user)) {
		$dyap_user = $fuser; // To avoid error during migration
	}

	$sql = "SELECT rowid as uid from ".MAIN_DB_PREFIX."user"; // Get list of all users
	$resql = $dyap_db->query($sql);
	if ($resql) {
		while ($dyap_obj = $dyap_db->fetch_object($resql)) {
			$fuser->fetch($dyap_obj->uid);
			//echo '<hr>'.$fuser->id.' -> '.$fuser->entity;
			$entity = (empty($fuser->entity) ? 1 : $fuser->entity);
			if ($entity > 1) {
				$dir = DOL_DATA_ROOT.'/'.$entity.'/users';
			} else {
				$dir = $dyap_conf->user->multidir_output[$entity]; // $dyap_conf->user->multidir_output[] for each entity is construct by the multicompany module
			}

			if ($dir) {
				//print "Process user id ".$fuser->id."<br>\n";
				$origin = $dir.'/'.get_exdir($fuser->id, 2, 0, 1, $fuser, 'user'); // Use old behaviour to get x/y path
				$destin = $dir.'/'.$fuser->id;

				$origin_osencoded = dol_osencode($origin);

				dol_mkdir($destin);

				//echo '<hr>'.$origin.' -> '.$destin;
				if (dol_is_dir($origin)) {
					$handle = opendir($origin_osencoded);
					if (is_resource($handle)) {
						while (($dyap_file = readdir($handle)) !== false) {
							if ($dyap_file == '.' || $dyap_file == '..') {
								continue;
							}

							if (dol_is_dir($origin.'/'.$dyap_file)) {	// it is a dir (like 'thumbs')
								$thumbs = opendir($origin_osencoded.'/'.$dyap_file);
								if (is_resource($thumbs)) {
									dol_mkdir($destin.'/'.$dyap_file);
									while (($thumb = readdir($thumbs)) !== false) {
										if (!dol_is_file($destin.'/'.$dyap_file.'/'.$thumb)) {
											if ($thumb == '.' || $thumb == '..') {
												continue;
											}

											//print $origin.'/'.$dyap_file.'/'.$thumb.' -> '.$destin.'/'.$dyap_file.'/'.$thumb.'<br>'."\n";
											print '.';
											dol_copy($origin.'/'.$dyap_file.'/'.$thumb, $destin.'/'.$dyap_file.'/'.$thumb, 0, 0);
											//var_dump('aaa');exit;
										}
									}
									// dol_delete_dir($origin.'/'.$dyap_file);
								}
							} else // it is a file
							{
								if (!dol_is_file($destin.'/'.$dyap_file)) {
									//print $origin.'/'.$dyap_file.' -> '.$destin.'/'.$dyap_file.'<br>'."\n";
									print '.';
									dol_copy($origin.'/'.$dyap_file, $destin.'/'.$dyap_file, 0, 0);
									//var_dump('eee');exit;
								}
							}
						}
					}
				}
			}
		}
	}

	print '</td></tr>';
}


/* A faire egalement: Modif statut paye et fk_facture des factures payes completement

On recherche facture incorrecte:
select f.rowid, f.total_ttc as t1, sum(pf.amount) as t2 from llx_facture as f, llx_paiement_facture as pf where pf.fk_facture=f.rowid and f.fk_statut in(2,3) and paye=0 and close_code is null group by f.rowid
having  f.total_ttc = sum(pf.amount)

On les corrige:
update llx_facture set paye=1, fk_statut=2 where close_code is null
and rowid in (...)
*/

/**
 * Migrate users fields facebook and co to socialnetworks.
 * Can be called only when version is 10.0.* or lower. Fields does not exists after.
 *
 * @return  void
 */
function migrate_users_socialnetworks()
{
	global $dyap_db, $dyap_langs;
	// skype,twitter,facebook,linkedin,instagram,snapchat,googleplus,youtube,whatsapp
	$error = 0;
	$dyap_db->begin();
	print '<tr><td colspan="4">';
	$sql = 'SELECT rowid, socialnetworks';
	$sql .= ', skype, twitter, facebook, linkedin, instagram, snapchat, googleplus, youtube, whatsapp FROM '.MAIN_DB_PREFIX.'user WHERE';
	$sql .= " skype IS NOT NULL OR skype <> ''";
	$sql .= " OR twitter IS NOT NULL OR twitter <> ''";
	$sql .= " OR facebook IS NOT NULL OR facebook <> ''";
	$sql .= " OR linkedin IS NOT NULL OR linkedin <> ''";
	$sql .= " OR instagram IS NOT NULL OR instagram <> ''";
	$sql .= " OR snapchat IS NOT NULL OR snapchat <> ''";
	$sql .= " OR googleplus IS NOT NULL OR googleplus <> ''";
	$sql .= " OR youtube IS NOT NULL OR youtube <> ''";
	$sql .= " OR whatsapp IS NOT NULL OR whatsapp <> ''";
	//print $sql;
	$resql = $dyap_db->query($sql);
	if ($resql) {
		while ($dyap_obj = $dyap_db->fetch_object($resql)) {
			$arraysocialnetworks = array();
			if (!empty($dyap_obj->skype)) {
				$arraysocialnetworks['skype'] = $dyap_obj->skype;
			}
			if (!empty($dyap_obj->twitter)) {
				$arraysocialnetworks['twitter'] = $dyap_obj->twitter;
			}
			if (!empty($dyap_obj->facebook)) {
				$arraysocialnetworks['facebook'] = $dyap_obj->facebook;
			}
			if (!empty($dyap_obj->linkedin)) {
				$arraysocialnetworks['linkedin'] = $dyap_obj->linkedin;
			}
			if (!empty($dyap_obj->instagram)) {
				$arraysocialnetworks['instagram'] = $dyap_obj->instagram;
			}
			if (!empty($dyap_obj->snapchat)) {
				$arraysocialnetworks['snapchat'] = $dyap_obj->snapchat;
			}
			if (!empty($dyap_obj->googleplus)) {
				$arraysocialnetworks['googleplus'] = $dyap_obj->googleplus;
			}
			if (!empty($dyap_obj->youtube)) {
				$arraysocialnetworks['youtube'] = $dyap_obj->youtube;
			}
			if (!empty($dyap_obj->whatsapp)) {
				$arraysocialnetworks['whatsapp'] = $dyap_obj->whatsapp;
			}
			if ($dyap_obj->socialnetworks == '' || is_null($dyap_obj->socialnetworks)) {
				$dyap_obj->socialnetworks = '[]';
			}
			$socialnetworks = array_merge($arraysocialnetworks, json_decode($dyap_obj->socialnetworks, true));
			$sqlupd = 'UPDATE '.MAIN_DB_PREFIX."user SET socialnetworks='".$dyap_db->escape(json_encode($socialnetworks, true))."'";
			$sqlupd .= ', skype=null';
			$sqlupd .= ', twitter=null';
			$sqlupd .= ', facebook=null';
			$sqlupd .= ', linkedin=null';
			$sqlupd .= ', instagram=null';
			$sqlupd .= ', snapchat=null';
			$sqlupd .= ', googleplus=null';
			$sqlupd .= ', youtube=null';
			$sqlupd .= ', whatsapp=null';
			$sqlupd .= ' WHERE rowid = '.((int) $dyap_obj->rowid);
			//print $sqlupd."<br>";
			$resqlupd = $dyap_db->query($sqlupd);
			if (!$resqlupd) {
				dol_print_error($dyap_db);
				$error++;
			}
		}
	} else {
		$error++;
	}
	if (!$error) {
		$dyap_db->commit();
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}
	print '<b>'.$dyap_langs->trans('MigrationFieldsSocialNetworks', 'Users')."</b><br>\n";
	print '</td></tr>';
}

/**
 * Migrate members fields facebook and co to socialnetworks
 * Can be called only when version is 10.0.* or lower. Fields does not exists after.
 *
 * @return  void
 */
function migrate_members_socialnetworks()
{
	global $dyap_db, $dyap_langs;

	print '<tr><td colspan="4">';
	$error = 0;
	$dyap_db->begin();
	print '<tr><td colspan="4">';
	$sql = 'SELECT rowid, socialnetworks';
	$sql .= ', skype, twitter, facebook, linkedin, instagram, snapchat, googleplus, youtube, whatsapp FROM '.MAIN_DB_PREFIX.'adherent WHERE ';
	$sql .= " skype IS NOT NULL OR skype <> ''";
	$sql .= " OR twitter IS NOT NULL OR twitter <> ''";
	$sql .= " OR facebook IS NOT NULL OR facebook <> ''";
	$sql .= " OR linkedin IS NOT NULL OR linkedin <> ''";
	$sql .= " OR instagram IS NOT NULL OR instagram <> ''";
	$sql .= " OR snapchat IS NOT NULL OR snapchat <> ''";
	$sql .= " OR googleplus IS NOT NULL OR googleplus <> ''";
	$sql .= " OR youtube IS NOT NULL OR youtube <> ''";
	$sql .= " OR whatsapp IS NOT NULL OR whatsapp <> ''";
	//print $sql;
	$resql = $dyap_db->query($sql);
	if ($resql) {
		while ($dyap_obj = $dyap_db->fetch_object($resql)) {
			$arraysocialnetworks = array();
			if (!empty($dyap_obj->skype)) {
				$arraysocialnetworks['skype'] = $dyap_obj->skype;
			}
			if (!empty($dyap_obj->twitter)) {
				$arraysocialnetworks['twitter'] = $dyap_obj->twitter;
			}
			if (!empty($dyap_obj->facebook)) {
				$arraysocialnetworks['facebook'] = $dyap_obj->facebook;
			}
			if (!empty($dyap_obj->linkedin)) {
				$arraysocialnetworks['linkedin'] = $dyap_obj->linkedin;
			}
			if (!empty($dyap_obj->instagram)) {
				$arraysocialnetworks['instagram'] = $dyap_obj->instagram;
			}
			if (!empty($dyap_obj->snapchat)) {
				$arraysocialnetworks['snapchat'] = $dyap_obj->snapchat;
			}
			if (!empty($dyap_obj->googleplus)) {
				$arraysocialnetworks['googleplus'] = $dyap_obj->googleplus;
			}
			if (!empty($dyap_obj->youtube)) {
				$arraysocialnetworks['youtube'] = $dyap_obj->youtube;
			}
			if (!empty($dyap_obj->whatsapp)) {
				$arraysocialnetworks['whatsapp'] = $dyap_obj->whatsapp;
			}
			if ($dyap_obj->socialnetworks == '' || is_null($dyap_obj->socialnetworks)) {
				$dyap_obj->socialnetworks = '[]';
			}
			$socialnetworks = array_merge($arraysocialnetworks, json_decode($dyap_obj->socialnetworks, true));
			$sqlupd = 'UPDATE '.MAIN_DB_PREFIX."adherent SET socialnetworks='".$dyap_db->escape(json_encode($socialnetworks, true))."'";
			$sqlupd .= ', skype=null';
			$sqlupd .= ', twitter=null';
			$sqlupd .= ', facebook=null';
			$sqlupd .= ', linkedin=null';
			$sqlupd .= ', instagram=null';
			$sqlupd .= ', snapchat=null';
			$sqlupd .= ', googleplus=null';
			$sqlupd .= ', youtube=null';
			$sqlupd .= ', whatsapp=null';
			$sqlupd .= ' WHERE rowid = '.((int) $dyap_obj->rowid);
			//print $sqlupd."<br>";
			$resqlupd = $dyap_db->query($sqlupd);
			if (!$resqlupd) {
				dol_print_error($dyap_db);
				$error++;
			}
		}
	} else {
		$error++;
	}
	if (!$error) {
		$dyap_db->commit();
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}
	print '<b>'.$dyap_langs->trans('MigrationFieldsSocialNetworks', 'Members')."</b><br>\n";
	print '</td></tr>';
}

/**
 * Migrate contacts fields facebook and co to socialnetworks
 * Can be called only when version is 10.0.* or lower. Fields does not exists after.
 *
 * @return  void
 */
function migrate_contacts_socialnetworks()
{
	global $dyap_db, $dyap_langs;
	// jabberid,skype,twitter,facebook,linkedin,instagram,snapchat,googleplus,youtube,whatsapp
	$error = 0;
	$dyap_db->begin();
	print '<tr><td colspan="4">';
	$sql = 'SELECT rowid, socialnetworks';
	$sql .= ', jabberid, skype, twitter, facebook, linkedin, instagram, snapchat, googleplus, youtube, whatsapp FROM '.MAIN_DB_PREFIX.'socpeople WHERE';
	$sql .= " jabberid IS NOT NULL OR jabberid <> ''";
	$sql .= " OR skype IS NOT NULL OR skype <> ''";
	$sql .= " OR twitter IS NOT NULL OR twitter <> ''";
	$sql .= " OR facebook IS NOT NULL OR facebook <> ''";
	$sql .= " OR linkedin IS NOT NULL OR linkedin <> ''";
	$sql .= " OR instagram IS NOT NULL OR instagram <> ''";
	$sql .= " OR snapchat IS NOT NULL OR snapchat <> ''";
	$sql .= " OR googleplus IS NOT NULL OR googleplus <> ''";
	$sql .= " OR youtube IS NOT NULL OR youtube <> ''";
	$sql .= " OR whatsapp IS NOT NULL OR whatsapp <> ''";
	//print $sql;
	$resql = $dyap_db->query($sql);
	if ($resql) {
		while ($dyap_obj = $dyap_db->fetch_object($resql)) {
			$arraysocialnetworks = array();
			if (!empty($dyap_obj->jabberid)) {
				$arraysocialnetworks['jabber'] = $dyap_obj->jabberid;
			}
			if (!empty($dyap_obj->skype)) {
				$arraysocialnetworks['skype'] = $dyap_obj->skype;
			}
			if (!empty($dyap_obj->twitter)) {
				$arraysocialnetworks['twitter'] = $dyap_obj->twitter;
			}
			if (!empty($dyap_obj->facebook)) {
				$arraysocialnetworks['facebook'] = $dyap_obj->facebook;
			}
			if (!empty($dyap_obj->linkedin)) {
				$arraysocialnetworks['linkedin'] = $dyap_obj->linkedin;
			}
			if (!empty($dyap_obj->instagram)) {
				$arraysocialnetworks['instagram'] = $dyap_obj->instagram;
			}
			if (!empty($dyap_obj->snapchat)) {
				$arraysocialnetworks['snapchat'] = $dyap_obj->snapchat;
			}
			if (!empty($dyap_obj->googleplus)) {
				$arraysocialnetworks['googleplus'] = $dyap_obj->googleplus;
			}
			if (!empty($dyap_obj->youtube)) {
				$arraysocialnetworks['youtube'] = $dyap_obj->youtube;
			}
			if (!empty($dyap_obj->whatsapp)) {
				$arraysocialnetworks['whatsapp'] = $dyap_obj->whatsapp;
			}
			if ($dyap_obj->socialnetworks == '' || is_null($dyap_obj->socialnetworks)) {
				$dyap_obj->socialnetworks = '[]';
			}
			$socialnetworks = array_merge($arraysocialnetworks, json_decode($dyap_obj->socialnetworks, true));
			$sqlupd = 'UPDATE '.MAIN_DB_PREFIX."socpeople SET socialnetworks='".$dyap_db->escape(json_encode($socialnetworks, true))."'";
			$sqlupd .= ', jabberid=null';
			$sqlupd .= ', skype=null';
			$sqlupd .= ', twitter=null';
			$sqlupd .= ', facebook=null';
			$sqlupd .= ', linkedin=null';
			$sqlupd .= ', instagram=null';
			$sqlupd .= ', snapchat=null';
			$sqlupd .= ', googleplus=null';
			$sqlupd .= ', youtube=null';
			$sqlupd .= ', whatsapp=null';
			$sqlupd .= ' WHERE rowid = '.((int) $dyap_obj->rowid);
			//print $sqlupd."<br>";
			$resqlupd = $dyap_db->query($sqlupd);
			if (!$resqlupd) {
				dol_print_error($dyap_db);
				$error++;
			}
		}
	} else {
		$error++;
	}
	if (!$error) {
		$dyap_db->commit();
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}
	print '<b>'.$dyap_langs->trans('MigrationFieldsSocialNetworks', 'Contacts')."</b><br>\n";
	print '</td></tr>';
}

/**
 * Migrate thirdparties fields facebook and co to socialnetworks
 * Can be called only when version is 10.0.* or lower. Fields does not exists after.
 *
 * @return  void
 */
function migrate_thirdparties_socialnetworks()
{
	global $dyap_db, $dyap_langs;
	// skype,twitter,facebook,linkedin,instagram,snapchat,googleplus,youtube,whatsapp
	$error = 0;
	$dyap_db->begin();
	print '<tr><td colspan="4">';
	$sql = 'SELECT rowid, socialnetworks';
	$sql .= ', skype, twitter, facebook, linkedin, instagram, snapchat, googleplus, youtube, whatsapp FROM '.MAIN_DB_PREFIX.'societe WHERE ';
	$sql .= " skype IS NOT NULL OR skype <> ''";
	$sql .= " OR twitter IS NOT NULL OR twitter <> ''";
	$sql .= " OR facebook IS NOT NULL OR facebook <> ''";
	$sql .= " OR linkedin IS NOT NULL OR linkedin <> ''";
	$sql .= " OR instagram IS NOT NULL OR instagram <> ''";
	$sql .= " OR snapchat IS NOT NULL OR snapchat <> ''";
	$sql .= " OR googleplus IS NOT NULL OR googleplus <> ''";
	$sql .= " OR youtube IS NOT NULL OR youtube <> ''";
	$sql .= " OR whatsapp IS NOT NULL OR whatsapp <> ''";
	//print $sql;
	$resql = $dyap_db->query($sql);
	if ($resql) {
		while ($dyap_obj = $dyap_db->fetch_object($resql)) {
			$arraysocialnetworks = array();
			if (!empty($dyap_obj->skype)) {
				$arraysocialnetworks['skype'] = $dyap_obj->skype;
			}
			if (!empty($dyap_obj->twitter)) {
				$arraysocialnetworks['twitter'] = $dyap_obj->twitter;
			}
			if (!empty($dyap_obj->facebook)) {
				$arraysocialnetworks['facebook'] = $dyap_obj->facebook;
			}
			if (!empty($dyap_obj->linkedin)) {
				$arraysocialnetworks['linkedin'] = $dyap_obj->linkedin;
			}
			if (!empty($dyap_obj->instagram)) {
				$arraysocialnetworks['instagram'] = $dyap_obj->instagram;
			}
			if (!empty($dyap_obj->snapchat)) {
				$arraysocialnetworks['snapchat'] = $dyap_obj->snapchat;
			}
			if (!empty($dyap_obj->googleplus)) {
				$arraysocialnetworks['googleplus'] = $dyap_obj->googleplus;
			}
			if (!empty($dyap_obj->youtube)) {
				$arraysocialnetworks['youtube'] = $dyap_obj->youtube;
			}
			if (!empty($dyap_obj->whatsapp)) {
				$arraysocialnetworks['whatsapp'] = $dyap_obj->whatsapp;
			}
			if ($dyap_obj->socialnetworks == '' || is_null($dyap_obj->socialnetworks)) {
				$dyap_obj->socialnetworks = '[]';
			}
			$socialnetworks = array_merge($arraysocialnetworks, json_decode($dyap_obj->socialnetworks, true));
			$sqlupd = 'UPDATE '.MAIN_DB_PREFIX."societe SET socialnetworks='".$dyap_db->escape(json_encode($socialnetworks, true))."'";
			$sqlupd .= ', skype=null';
			$sqlupd .= ', twitter=null';
			$sqlupd .= ', facebook=null';
			$sqlupd .= ', linkedin=null';
			$sqlupd .= ', instagram=null';
			$sqlupd .= ', snapchat=null';
			$sqlupd .= ', googleplus=null';
			$sqlupd .= ', youtube=null';
			$sqlupd .= ', whatsapp=null';
			$sqlupd .= ' WHERE rowid = '.((int) $dyap_obj->rowid);
			//print $sqlupd."<br>";
			$resqlupd = $dyap_db->query($sqlupd);
			if (!$resqlupd) {
				dol_print_error($dyap_db);
				$error++;
			}
		}
	} else {
		$error++;
	}
	if (!$error) {
		$dyap_db->commit();
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}
	print '<b>'.$dyap_langs->trans('MigrationFieldsSocialNetworks', 'Thirdparties')."</b><br>\n";
	print '</td></tr>';
}


/**
 * Migrate export and import profiles to fix field name that was renamed
 *
 * @param	string		$dyap_mode		'export' or 'import'
 * @return  void
 */
function migrate_export_import_profiles($dyap_mode = 'export')
{
	global $dyap_db, $dyap_langs;

	$error = 0;
	$resultstring = '';

	$dyap_db->begin();

	print '<tr class="trforrunsql"><td colspan="4">';
	$sql = 'SELECT rowid, field';
	if ($dyap_mode == 'export') {
		$sql .= ', filter';
	}
	$sql .= ' FROM '.MAIN_DB_PREFIX.$dyap_mode.'_model WHERE';
	$sql .= " type LIKE 'propale_%' OR type LIKE 'commande_%' OR type LIKE 'facture_%'";
	//print $sql;
	$resql = $dyap_db->query($sql);
	if ($resql) {
		while ($dyap_obj = $dyap_db->fetch_object($resql)) {
			$oldfield = $dyap_obj->field;
			$newfield = str_replace(array(',f.facnumber', 'f.facnumber,', 'f.total,', 'f.tva,'), array(',f.ref', 'f.ref,', 'f.total_ht,', 'f.total_tva,'), $oldfield);

			if ($dyap_mode == 'export') {
				$oldfilter = $dyap_obj->filter;
				$newfilter = str_replace(array('f.facnumber=', 'f.total=', 'f.tva='), array('f.ref=', 'f.total_ht=', 'f.total_tva='), $oldfilter);
			} else {
				$oldfilter = '';
				$newfilter = '';
			}

			if ($oldfield != $newfield || $oldfilter != $newfilter) {
				$sqlupd = 'UPDATE '.MAIN_DB_PREFIX.$dyap_mode."_model SET field = '".$dyap_db->escape($newfield)."'";
				if ($dyap_mode == 'export') {
					$sqlupd .= ", filter = '".$dyap_db->escape($newfilter)."'";
				}
				$sqlupd .= ' WHERE rowid = '.((int) $dyap_obj->rowid);
				$resultstring .= '<tr class="trforrunsql" style=""><td class="wordbreak" colspan="4">'.$sqlupd."</td></tr>\n";
				$resqlupd = $dyap_db->query($sqlupd);
				if (!$resqlupd) {
					dol_print_error($dyap_db);
					$error++;
				}
			}
		}
	} else {
		$error++;
	}
	if (!$error) {
		$dyap_db->commit();
	} else {
		dol_print_error($dyap_db);
		$dyap_db->rollback();
	}
	print '<b>'.$dyap_langs->trans('MigrationImportOrExportProfiles', $dyap_mode)."</b><br>\n";
	print '</td></tr>';

	if ($resultstring) {
		print $resultstring;
	} else {
		print '<tr class="trforrunsql" style=""><td class="wordbreak" colspan="4">'.$dyap_langs->trans("NothingToDo")."</td></tr>\n";
	}
}


/* A faire egalement: Modif statut paye et fk_facture des factures payes completement

On recherche facture incorrecte:
select f.rowid, f.total_ttc as t1, sum(pf.amount) as t2 from llx_facture as f, llx_paiement_facture as pf where pf.fk_facture=f.rowid and f.fk_statut in(2,3) and paye=0 and close_code is null group by f.rowid
having  f.total_ttc = sum(pf.amount)

On les corrige:
update llx_facture set paye=1, fk_statut=2 where close_code is null
and rowid in (...)
     */

/**
 * Log function for install pages
 *
 * @param	string	$message	Message
 * @param 	int		$level		Level of log
 * @return	void
 */
function dolibarr_install_syslog($message, $level=LOG_DEBUG)
{
    if (! defined('LOG_DEBUG')) define('LOG_DEBUG',6);
    dol_syslog($message,$level);
    error_log($message);
}

